/*------------------------------------------------------------------------
 * MAGNET Bid Manager component
 * $Id: BidCollector.java,v 1.3.2.1 2002/05/05 19:41:44 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Sat May  5 17:55:19 2001  John Collins
 *   Add sessionId accessors to support new interaction layer.
 * 
 * Sat Feb 24 16:22:19 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.bid;

import edu.umn.magnet.protocol.cust.RFQ;

import java.util.Collection;

/**
 * This interface defines a synchronous bid collector.  You give it a
 * RFQ, it goes out and collects the bids, returning them all
 * in a single Collection at the conclusion of the bidding cycle.
 * @author John Collins
 * @version $Revision: 1.3.2.1 $
 */
public interface BidCollector
{
  /**
   * Accepts a RFQ, goes out and gets a Collection of bids
   * somehow, and returns them.
   */
  public Collection collectBids (RFQ cfb);

  /**
   * Returns the Session ID of the market session from which bids are
   * to be collected.
   */
  public String getSessionId ();

  /**
   * Sets the Session ID of the market session for this collector.
   */
  public void setSessionId (String newId);
}
