/*------------------------------------------------------------------------
 * MAGNET bid processor component
 * $Id: CPMTWAdjuster.java,v 1.2.2.1 2002/05/05 19:41:45 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Mon Oct 15 10:39:12 2001  John Collins
 *   Initial Entry, refactored out of PlanTWAdjuster
 */

package edu.umn.magnet.client.bid;

import org.apache.log4j.Category;

/**
 * The methods of this class are used to adjust the time windows in a
 * "raw" plan in preparation for constructing the RFQ.  Two types of
 * adjustments are supported: overall slack, known as "plan slack",
 * and per-task slack, known as "CPM slack".  The latter is
 * accomplished by choosing a shorter-than-expected duration for the
 * task.  The choosing of the duration is done with a TaskVisitor that
 * can either apply a fixed proportion, or it may use some combination
 * of factors, such as historical variability or expected number of
 * bidders.  After the deadline and task durations are adjusted, the
 * time windows are computed with the CPM algorithm.
 *<p>
 * The underlying plan is a TaskBidPlan, which may contain some tasks
 * that are already associated with a bid.  We don't do any adjustment
 * on those.
 * </p>
 * <p>
 * It's a real shame this can't be combined with eval.CPMEval.  It's
 * so similar.  Maybe there's another way to get the adjuster passed
 * around.  A problem for another day...
 * </p>
 * @author John Collins
 * @version $Revision: 1.2.2.1 $
 */
public class CPMTWAdjuster extends PlanTWAdjuster
{
  static Category log = Category.getInstance(CPMTWAdjuster.class.getName());

  /**
   * Default constructor
   */
  public CPMTWAdjuster ()
  {
    super();
  }

  /**
   * Sets the time windows using the CPM algorithm.
   */
  protected void setTW (TaskDurationAdjuster adj)
  {
    log.debug("setTW using CPM");
    updateCPM(adj);
  }
}
