/*------------------------------------------------------------------------
 * Magnet customer agent bid manager components
 * $Id: TaskDurationAdjuster.java,v 1.3.2.1 2002/05/05 19:41:46 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Sep 27 15:09:10 2000  John Collins
 *   Move to bid package
 * 
 * Sat Sep  5 21:44:18 1998  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.bid;

import edu.umn.magnet.protocol.cust.AbstractTaskVisitor;
import edu.umn.magnet.client.eval.TaskBid;
import edu.umn.magnet.client.status.PlanStatus;

import org.apache.log4j.Category;

import java.util.Properties;

/**
 * Adjusts the duration of a task for computation of the RFQ schedule.
 * This default implementation actually doesn't do anything to the tasks. 
 * @author John Collins
 */
public class TaskDurationAdjuster extends AbstractTaskVisitor
{
  static Category log
    = Category.getInstance(TaskDurationAdjuster.class.getName());

  protected PlanStatus planStatus = null;

  /**
   * The default constructor probably won't be used.
   */
  public TaskDurationAdjuster ()
  {
    super();
  }

  /**
   * The principal constructor takes a property list and tries to
   * extract the cpmSlack value from it.
   */
  public TaskDurationAdjuster (Properties props)
  {
    super();
    init(props);
  }

  /**
   * Initialize an empty one from input properties.
   */
  protected void init (Properties props)
  {
  }

  /**
   * Gets the PlanStatus value.
   */
  public PlanStatus getPlanStatus ()
  {
    return planStatus;
  }

  /**
   * Sets the PlanStatus value.  You have to set this so we can figure
   * out whether a task is OFFERED or not.
   */
  public void setPlanStatus (PlanStatus newStatus)
  {
    planStatus = newStatus;
  }

  /**
   * This is the operator method for this visitor.  We just make the
   * Adjusted duration be the same as the Expected duration.
   */
  public void processTaskBid (TaskBid task)
  {
    task.setAdjustedDuration(Math.round(task.getExpectedDuration()));
  }

  /**
   * Static factory for duration adjusters and subclasses.
   * @param props A properties object in which the setup properties
   *   are expected to reside.
   */
  public static TaskDurationAdjuster generate (Properties props)
  {
    String adjusterType = props.getProperty("TDA.Type", null);
    log.info("prop: Task Duration Adjuster type = "
	     + adjusterType);
    if (adjusterType == null) {
      log.warn("Adjuster type not specified.  Using default");
      return new TaskDurationAdjuster(props);
    }
    else {      
      try {
	String classname = "edu.umn.magnet.client.bid." + adjusterType;
	Class target = Class.forName(classname);
	TaskDurationAdjuster result
	    = (TaskDurationAdjuster)target.newInstance();
	result.init(props);
	return result;
      }
      catch (ClassNotFoundException cnf) {
	log.error("specified adjuster not found.  Using default");
	return new TaskDurationAdjuster(props);
      }
      catch (InstantiationException ie) {
	log.error("specified adjuster can't be instantiated.  Using default");
	return new TaskDurationAdjuster(props);
      }
      catch (IllegalAccessException iae) {
	log.error("Illegal access exception.  Using default");
	return new TaskDurationAdjuster(props);
      }
    }
  }
}
