/*------------------------------------------------------------------------
 * Magnet Bid Evaluation components
 * $Id: BidSelector.java,v 1.1.2.1 2002/05/05 19:41:47 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Jan  5 13:48:06 2000  John E. Collins
 *   Add method signatures for hasMoreChoices and recycle.
 * 
 * Sat Jan  9 09:59:28 1999  John E. Collins
 *   Add BidEvaluator to arg list of selectBid.
 * 
 * Wed Sep 30 20:14:16 1998  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.eval;

import edu.umn.magnet.protocol.cust.Bid;

/**
 * This is the generic interface for Bid Selectors.
 * @author John Collins
 */
public interface BidSelector
{
  /**
     Returns a Bid to try as an expansion for the given plan node.
  */
  public Bid selectBid(SearchNode node, BidEvaluator be);

  /**
     Some selectors like to know what the temperature is and adjust
     their behavior accordingly.
  */
  public void setTemperature (double temp);

  /**
     Returns true just in case this selector has more tricks up its
     sleeve.  The intent is that the search engine will reset its
     patience counter as a result.
  */
  public boolean hasMoreChoices ();

  /**
     Signals this selector that the engine is restarting the search.
  */
  public void recycle ();
}
