/*------------------------------------------------------------------------
 * Magnet Bid Evaluation components
 * $Id: RandomUsableBidSelector.java,v 1.2.2.1 2002/05/05 19:41:48 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Mon Jan  8 19:22:07 2001  John Collins
 *   Change bid eval bid set to list form
 * 
 * Sat Dec 16 18:30:43 2000  John Collins
 *   Eliminate bid breakdowns
 * 
 * Wed Jan  5 14:06:07 2000  John E. Collins
 *   extend AbstractBidSelector
 * 
 * Tue Jan 12 20:23:29 1999  Tim Lee
 *   Replaced Math.Random with RVFManager calls.
 *
 * Sat Jan  9 10:02:24 1999  John E. Collins
 *   Pass BidEvaluator to selectBid rather than constructor
 * 
 * Tue Dec 29 18:42:32 1998  John E. Collins
 *   Reformat, change type of bid set
 * 
 * Wed Sep 30 20:14:16 1998  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.eval;

import edu.umn.magnet.protocol.cust.Bid;

import edu.umn.magnet.util.rvars.RandomVariableFactory;
import edu.umn.magnet.util.rvars.RVFManager;
import edu.umn.magnet.util.rvars.UniformVariable;

import org.apache.log4j.Category;

import java.util.Set;
import java.util.Iterator;

/**
 * This is a random bid selector for stochastic search that makes a
 * random choice only among the "usable" bids for a node.
 * @author John Collins
 */
public class RandomUsableBidSelector extends AbstractBidSelector
{
  private static Category log
    = Category.getInstance(RandomUsableBidSelector.class.getName());

  // random numbers
  private RandomVariableFactory rvFactory = null;
  private UniformVariable uVariable = null;
  private RandomBidSelector rsel = new RandomBidSelector();

  public RandomUsableBidSelector() 
  {
    super();
    rvFactory = RVFManager.getRVF("SearchSeed");
    uVariable = rvFactory.createUniformVariable(0,1);
  }

  /**
   * Returns a Bid to try as an expansion for the given plan node.
   */
  public Bid selectBid(SearchNode node, BidEvaluator be) 
  {
    Set bidSet = node.getUsableBids();
    if (bidSet == null || bidSet.size() == 0) {
      log.warn("Using random selector"
	       + ((bidSet == null)?", null bidSet":", empty bidSet"));
      return rsel.selectBid(node, be);
    }

    int choice = (int)(uVariable.nextDouble() * (bidSet.size() - 1));
    Iterator bids = bidSet.iterator();
    Bid result = null;
    while (choice-- > 0) {
      bids.next();
    }
    return (Bid)bids.next();
  }
}

 
