/*-------------------------------------------------------------------------
 * Magnet unit tests
 * $Id: SearchNodeTest.java,v 1.1.2.1 2002/05/05 19:41:52 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *-------------------------------------------------------------------------
 * Revision History:
 * 
 * Mon Jan 29 10:59:01 2001  John Collins
 *   Initial Entry, split out from TaskBidPlanTest
 */

package edu.umn.magnet.client.eval;

import edu.umn.magnet.client.SampleTaskPlan;
import edu.umn.magnet.protocol.cust.Bid;
import edu.umn.magnet.protocol.cust.TaskPlan;
import edu.umn.magnet.protocol.InvalidDataException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * This is a tester for edu.umn.magnet.client.eval.Eval and related
 * stuff.
 * @author John Collins
 * @version $Revision: 1.1.2.1 $
 */
public class SearchNodeTest extends TestCase
{
  private TaskPlan plan = null;
  private SearchNode node = null;
  private SearchNode node1 = null;
  private SearchNode node2 = null;
  private SearchNode node3 = null;
  private SearchNode node4 = null;
  private Bid b1 = new Bid();
  private Bid b2 = new Bid();
  private Bid b3 = new Bid();
  private Bid b4 = new Bid();
  private Bid b5 = new Bid();
  private Bid b6 = new Bid();
  
  public SearchNodeTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(SearchNodeTest.class);
  }

  private void setup1 ()
  {
    plan = SampleTaskPlan.generateTaskPlan();
    node = new SearchNode(new TaskBidPlan(plan));
  }

  // --------------------------------------------------------------------
  // Create a SearchNode and check it out
  public void testCreate ()
  {
    setup1();
    assertSame("Compare plan", plan, node.getTaskBidPlan().getTaskPlan());
    assertEquals("6 uncovered tasks", 6, node.getUncoveredTasks().size());
    assertEquals("No locked bids", 0, node.getLockedBids().size());
    assertEquals("Node 0", 0, node.getNodeNumber());
  }

  // --------------------------------------------------------------------
  // Expand the root node and evaluate.

  private void setup2 ()
  {
    setup1();

    // Create some Bids against the TP
    try {
      b1 = new Bid();
      b1.setId("B1");
      b1.addTask("T2");
      b1.addTask("T4");
      b1.addTask("T6");
      
      b2 = new Bid();
      b2.setId("B2");
      b2.addTask("T1");
      b2.addTask("T3");
      
      b3 = new Bid();
      b3.setId("B3");
      b3.addTask("T1");
      b3.addTask("T3");
      b3.addTask("T5");
      
      b4 = new Bid();
      b4.setId("B4");
      b4.addTask("T1");
      b4.addTask("T2");
      b4.addTask("T3");
    }
    catch (InvalidDataException ide) {
      fail("InvalidDataException");
    }

    // Run the first expansion
    node1 = node.expandNode(b1, 1);
  }

  // --------------------------------------------------------------------
  // Expand again and test bid locking
  private void setup3 ()
  {
    setup2();
    node1.lockBid(b1);
    node2 = node1.expandNode(b2, 2);
    node3 = node1.expandNode(b4, 3);
  }

  public void testExpand2 ()
  {
    setup3();
    assert("Test successful expansion", (node2 != null));
    // make sure correct tasks are uncovered...
  }

  public void testLocking ()
  {
    setup3();
    assert("Expansion should have failed", (node3 == null));
    node3 = node2.expandNode(b4, 3);
    assert("Locking should propagate to children", (node3 == null));
  }

  // --------------------------------------------------------------------
  // Add a bid that achieves coverage
  public void setup4 ()
  {
    setup3();
    node3 = node2.expandNode(b3, 3);
  }

  public void testCoverage ()
  {
    setup4();
    assert("Expansion should succeed", node3 != null);
    // should be just bids b1 and b3..
  }

  // --------------------------------------------------------------------
  // Check operation of tabu list & been-tried list
  public void testDuplicate ()
  {
    setup4();
    node4 = node3.expandNode(b3, 4);
    assert("Detect duplicate bid", (node4 == null));
  }

  public void testTabu ()
  {
    setup4();
    node4 = node3.expandNode(b2, 4);
    assert("Detect tabu bid", (node4 == null));
  }

  public void testTried ()
  {
    setup4();
    node4 = node2.expandNode(b3, 4);
    assert("Detect been-tried bid", (node4 == null));
  }
}
