/*------------------------------------------------------------------------
 * MAGNET unit test
 * $Id: TMEntryTest.java,v 1.1.2.1 2002/05/05 19:41:53 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Nov 19 13:51:10 2000  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.status;

import edu.umn.magnet.util.time.TimerAction;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.Date;

/**
 * This is a tester for edu.umn.magnet.client.Something
 * @author John Collins
 */
public class TMEntryTest extends TestCase
{
  
  public TMEntryTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(TMEntryTest.class);
  }

  final Date BASE = new Date();
  TMEntry entry = null;
  TMEntry entry2 = null;
  TMEntry entry3 = null;
  String result = null;

  public void testConstructor1 ()
  {
    entry = new TMEntry();
    assert("got created", entry != null && entry instanceof TMEntry);
    assert("not complete", !entry.isComplete());
    assertEquals("is pending", TMEntry.PENDING, entry.getState());
    assert("no predecessors", entry.getPredecessors().isEmpty());
    //assert("is valid", entry.isValid());
    assertEquals("expiration time is -1", -1L, entry.getExpirationTime());
  }

  public void testConstructor2 ()
  {
    entry = new TMEntry(new Date(BASE.getTime() + 10000),
			new TimerAction ()
			  {
			    public void action ()
			    {
			      result = "action2";
			    }
			  });
    assert("got created", entry != null && entry instanceof TMEntry);
    assert("not complete", !entry.isComplete());
    assertEquals("is pending", TMEntry.PENDING, entry.getState());
    assert("no predecessors", entry.getPredecessors().isEmpty());
    assertEquals("expiration time",
		 BASE.getTime() + 10000,
		 entry.getExpirationTime());
  }

  public void testCompletion1 ()
  {
    entry = new TMEntry(new Date(BASE.getTime() + 10000),
			new TimerAction ()
			  {
			    public void action ()
			    {
			      result = "action3";
			    }
			  });
    result = null;
    assertEquals("expiration time",
		 BASE.getTime() + 10000,
		 entry.getExpirationTime());
    entry.makeComplete();
    assert("completed", entry.isComplete());
    assertEquals("correct result", "action3", result);
    assertEquals("expiration time",
		 BASE.getTime() + 10000,
		 entry.getExpirationTime());
  }

  public void testCompletionNull ()
  {
    entry = new TMEntry(new Date(BASE.getTime() + 10000), null);
    result = "original";
    assertEquals("expiration time",
		 BASE.getTime() + 10000,
		 entry.getExpirationTime());
    entry.makeComplete();
    assert("completed", entry.isComplete());
    assertEquals("correct result", "original", result);
    assertEquals("expiration time",
		 BASE.getTime() + 10000,
		 entry.getExpirationTime());
  }

  private void setupP1 ()
  {
    entry2 = new TMEntry(new Date(BASE.getTime() + 10000), null);
    entry = new TMEntry(entry2, 10000, 
			new TimerAction ()
			  {
			    public void action ()
			    {
			      result = "action4";
			    }
			  });
    result = "original";
  }

  public void testPredecessor1 ()
  {
    setupP1();
    assert("got created", entry != null && entry instanceof TMEntry);
    assert("not complete", !entry.isComplete());
    assertEquals("is pending", TMEntry.PENDING, entry.getState());
    assertEquals("one predecessor", 1, entry.getPredecessors().size());
    assertEquals("expiration time",
		 BASE.getTime() + 20000,
		 entry.getExpirationTime());
  }

  public void testPredecessor2 ()
  {
    setupP1();
    entry3 = new TMEntry(new Date(BASE.getTime() + 10000), null);
    entry2.addPredecessor(entry3, 10000);
    assertEquals("expiration time",
		 BASE.getTime() + 30000,
		 entry.getExpirationTime());
  }
}
