/*------------------------------------------------------------------------
 * Magnet UI components
 * $Id: ClassicSM.java,v 1.2.2.1 2002/05/05 19:41:54 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Tue Nov 23 12:13:57 1999  John Collins
 *   Implement run-run variability in the link-following probability
 * 
 * Sat Nov 20 18:40:13 1999  John E. Collins
 *   Update for new JobInfo format
 * 
 * Sun Jan 17 13:05:11 1999  John E. Collins
 *   Correct handling of random time values, reformat a bit.
 *
 * Tue Jan 12 19:07:59 1999  Tim Lee
 *   Modified random variable code to use RVFManager.
 *   Removed methods getSeedsFromFile, selectSeed, setNormalVariableSeed,
 *   getNormalVariableSeed, and setRandomSeed.
 *
 * Wed Jan  6 10:35:48 1999  John E. Collins
 *   Don't use deprecated API
 * 
 * Mon Jun 29 10:54:15 1998  John E. Collins
 *   Package update.
 */

package edu.umn.magnet.client.sup;

import edu.umn.magnet.protocol.cust.RFQ;
import edu.umn.magnet.protocol.cust.Bid;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

/**
 *  The SupplyMonster provides a way of imitating a set of suppliers in a 
 *  marketplace.
 *  
 *  Before gimmeTheBids() is called the following must be called
 *  to init the SupplyMonster:
 *  
 *  setRFQ( RFQ cfb)
 *  
 *  @author John Collins
 */
public class ClassicSM extends SupplyMonster
{
  /**
   * Default constructor as required by factory implementation
   */
  public ClassicSM ()
  {
    super();
  }
    
  protected void init (Properties props) 
  {
    super.init(props);
  }
    
  /**
   * Supplies a Vector of randomly produced bids.
   */
  public Collection gimmeTheBids (RFQ cfb)
  {
    super.gimmeTheBids(cfb);

    if (getRFQ() == null) 
      return(null);

    // Use whatever bid generator was specified to generate the bids
    int bidderCount = (int)Math.round(getPropertyDouble("Sup.BidCount", 10.0));
    ArrayList theBids = new ArrayList(bidderCount);
    for (int bidNumber = 0; bidNumber < bidderCount; bidNumber++) {
      Bid nextBid = getBidGenerator().generateBid(cfb, null);
      if (nextBid != null)
	theBids.add(nextBid);
    }
	
    // calculate and print out statistics
    statistics(theBids);

    return theBids;
  }
}
