/*------------------------------------------------------------------------
 * Magnet UI components
 * $Id: GuaranteedCoverageSM.java,v 1.3.2.1 2002/05/05 19:41:54 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Jan 10 18:38:50 2001  John Collins
 *   update logging
 * 
 * Sat Dec 16 18:10:30 2000  John Collins
 *   Eliminate bid breakdowns
 * 
 * Sat Oct  7 11:31:14 2000  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.sup;

import edu.umn.magnet.protocol.cust.RFQ;
import edu.umn.magnet.protocol.cust.Bid;
import edu.umn.magnet.protocol.cust.SubtaskRequest;
import edu.umn.magnet.protocol.cust.TaskPlan;

import org.apache.log4j.Category;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

/**
 * This is a version of the SupplyMonster that guarantees to produce
 * bid sets that cover all tasks.
 *  
 *  @author John Collins
 */
public class GuaranteedCoverageSM extends SupplyMonster
{
  static Category log
    = Category.getInstance(GuaranteedCoverageSM.class.getName());

  /**
   * Default constructor as required by factory implementation
   */
  public GuaranteedCoverageSM ()
  {
    super();
  }
    
  protected void init (Properties props) 
  {
    super.init(props);
  }
    
  /**
   * Supplies a Vector of randomly produced bids.
   */
  public Collection gimmeTheBids (RFQ cfb)
  {
    super.gimmeTheBids(cfb);
    initUncoveredTasks();

    // Use whatever bid generator was specified to generate the bids
    int bidderCount = (int)Math.round(getPropertyDouble("Sup.BidCount", 10.0));
    ArrayList theBids = new ArrayList(bidderCount);
    for (int bidNumber = 0; bidNumber < bidderCount; bidNumber++) {
      Bid nextBid = getBidGenerator().generateBid(cfb, null);
      if (nextBid != null) {
	theBids.add(nextBid);
	updateCoveredTasks(nextBid);
      }
    }

    // Now, go again until we've covered all tasks
    int trials
	= (int)Math.round
	  (getPropertyDouble("Sup.GuaranteedCoverageNTrials", 20.0));
    int tcount = trials;
    while (! isCovered() && --tcount >= 0) {
      SubtaskRequest seed = chooseUncoveredTask();
      log.info("Attempting to cover task " + seed.getId());
      Bid nextBid = getBidGenerator().generateBid(cfb, seed);
      if (nextBid != null) {
	theBids.add(nextBid);
	updateCoveredTasks(nextBid);
	tcount = trials;
      }
      else if (tcount == 0) {
	log.warn("Unable to achieve coverage in specified number of trials");
      }
    }
	
    // calculate and print out statistics
    statistics(theBids);

    return theBids;
  }

  // ============ Management of uncovered tasks ===============

  HashSet m_uncoveredTasks = new HashSet();

  private void initUncoveredTasks ()
  {
    for (Iterator tasks = getPlan().iterator();
	 tasks.hasNext(); ) {
      m_uncoveredTasks.add(((SubtaskRequest)tasks.next()).getId());
    }
  }

  private void updateCoveredTasks (Bid bid)
  {
    for (Iterator tasks = bid.getTaskSet().iterator();
	 tasks.hasNext(); ) {
      m_uncoveredTasks.remove(tasks.next());
    }
  }

  private boolean isCovered ()
  {
    return m_uncoveredTasks.isEmpty();
  }

  private SubtaskRequest chooseUncoveredTask ()
  {
    Iterator ucTasks = m_uncoveredTasks.iterator();
    String stId = (String)ucTasks.next();
    return getPlan().getTaskData(stId);
  }
}
