/*------------------------------------------------------------------------
 * MAGNET Supplier component
 * $Id: GuiFrame.java,v 1.4.2.1 2002/05/05 19:41:54 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Mon Feb 4 15:50:58 2002 Anne Schoolcraft
 *  Changed screen location so that it will not get buried under the main
 *   GUI window.
 *
 * Sat Dec  8 12:48:38 2001  John Collins
 *   Move into client.sup package with SupplyMonster
 * 
 * Tue May 29 07:33:07 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.sup;

import edu.umn.magnet.util.MagnetProperties;

import org.apache.log4j.Category;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.Properties;

/**
 * Quickie GUI for the Auto Supplier
 * @author John Collins
 * @version $Revision: 1.4.2.1 $
 */

public class GuiFrame extends JFrame
{
  private static Category log =
    Category.getInstance(GuiFrame.class.getName());

  public static String BID_SIZE_SMALL = "small";
  public static String BID_SIZE_MEDIUM = "medium";
  public static String BID_SIZE_LARGE = "large";

  JPanel _pnlSettings = new JPanel();
  JPanel _pnlCurrent = new JPanel();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  GridBagLayout gridBagLayout2 = new GridBagLayout();
  TitledBorder titledBorder1;
  TitledBorder titledBorder2;
  JLabel _lblBidCount = new JLabel();
  JTextField _tfBidCount = new JTextField();
  JLabel _lblBidSize = new JLabel();
  JComboBox _cbBidSize = new JComboBox();
  DefaultComboBoxModel _cbmBidSize =
      new DefaultComboBoxModel();
  JLabel _lblSizeVar = new JLabel();
  JTextField _tfSizeVar = new JTextField();
  JLabel _lblRfqId = new JLabel();
  JTextField _tfRfqId = new JTextField();
  JLabel _lblSubmitted = new JLabel();
  JTextField _tfSubmitted = new JTextField();
  JButton _btnClear = new JButton();
  JTextField _tfMessage = new JTextField();

  private SupplyMonster _sm = null;
  private Properties _props = null;
  private static GuiFrame singleton = null;

  public GuiFrame(Properties props)
  {
    super("Supplier setup");
    //_sm = monster;
    _props = props;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    _cbmBidSize.addElement(BID_SIZE_SMALL);
    _cbmBidSize.addElement(BID_SIZE_MEDIUM);
    _cbmBidSize.addElement(BID_SIZE_LARGE);
    renderProps();
  }

  private void jbInit() throws Exception
  {
    titledBorder1 =
        new TitledBorder(BorderFactory.createLineBorder
			 (new Color(153, 153, 153), 2), "Settings");
    titledBorder2 =
        new TitledBorder(BorderFactory.createLineBorder
			 (new Color(153, 153, 153), 2),"Current RFQ");
    _pnlSettings.setLayout(gridBagLayout1);
    _pnlCurrent.setLayout(gridBagLayout2);
    _pnlSettings.setBorder(titledBorder1);
    _pnlCurrent.setBorder(titledBorder2);
    _lblBidCount.setText("Bid count");
    _tfBidCount.setToolTipText
        ("The number of bids the supplier will attempt to generate");
    _tfBidCount.setText("          ");
    _tfBidCount.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _tfBidCount_actionPerformed(e);
      }
    });
    _lblBidSize.setText("Bid size");
    _lblSizeVar.setText("Size variability");
    _tfSizeVar.setToolTipText("A number between 0.0 and 1.0");
    _tfSizeVar.setText("          ");
    _tfSizeVar.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _tfSizeVar_actionPerformed(e);
      }
    });
    _lblRfqId.setText("RFQ ID");
    _tfRfqId.setEditable(false);
    _tfRfqId.setText("          ");
    _lblSubmitted.setText("Bids submitted");
    _tfSubmitted.setEditable(false);
    _tfSubmitted.setText("          ");
    _btnClear.setToolTipText("Clears the RFQ ID and Bids submitted fields");
    _btnClear.setText("Clear");
    _btnClear.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _btnClear_actionPerformed(e);
      }
    });
    _cbBidSize.setModel(_cbmBidSize);
    _cbBidSize.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _cbBidSize_actionPerformed(e);
      }
    });
    _tfMessage.setEditable(false);
    _tfMessage.setText("Status");
    this.getContentPane().add(_pnlSettings, BorderLayout.NORTH);
    _pnlSettings.add(_lblBidCount,
		     new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
					    GridBagConstraints.WEST,
					    GridBagConstraints.NONE,
					    new Insets(5, 5, 5, 5), 0, 0));
    _pnlSettings.add(_tfBidCount,
		     new GridBagConstraints(1, 0, 1, 1, 0.5, 0.0,
					    GridBagConstraints.WEST,
					    GridBagConstraints.NONE,
					    new Insets(5, 5, 5, 5), 0, 0));
    _pnlSettings.add(_lblBidSize,
		     new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
					    GridBagConstraints.WEST,
					    GridBagConstraints.NONE,
					    new Insets(5, 5, 5, 5), 0, 0));
    _pnlSettings.add(_cbBidSize,
		     new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
					    GridBagConstraints.WEST,
					    GridBagConstraints.NONE,
					    new Insets(3, 5, 3, 5), 0, 0));
    _pnlSettings.add(_lblSizeVar,
		     new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
					    GridBagConstraints.WEST,
					    GridBagConstraints.NONE,
					    new Insets(5, 5, 5, 5), 0, 0));
    _pnlSettings.add(_tfSizeVar,
		     new GridBagConstraints(1, 2, 1, 1, 0.5, 0.0,
					    GridBagConstraints.WEST,
					    GridBagConstraints.NONE,
					    new Insets(5, 5, 5, 5), 0, 0));
    this.getContentPane().add(_pnlCurrent, BorderLayout.CENTER);
    _pnlCurrent.add(_lblRfqId,
		    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
					   GridBagConstraints.WEST,
					   GridBagConstraints.NONE,
					   new Insets(5, 5, 5, 5), 0, 0));
    _pnlCurrent.add(_tfRfqId,
		    new GridBagConstraints(1, 0, 2, 1, 0.5, 0.0,
					   GridBagConstraints.WEST,
					   GridBagConstraints.NONE,
					   new Insets(5, 5, 5, 5), 0, 0));
    _pnlCurrent.add(_lblSubmitted,
		    new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
					   GridBagConstraints.WEST,
					   GridBagConstraints.NONE,
					   new Insets(5, 5, 5, 5), 0, 0));
    _pnlCurrent.add(_tfSubmitted,
		    new GridBagConstraints(1, 1, 1, 1, 0.5, 0.0,
					   GridBagConstraints.WEST,
					   GridBagConstraints.NONE,
					   new Insets(5, 5, 0, 0), 0, 0));
    _pnlCurrent.add(_btnClear,
		    new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0,
					   GridBagConstraints.CENTER,
					   GridBagConstraints.NONE,
					   new Insets(5, 5, 5, 5), 0, 0));
    this.getContentPane().add(_tfMessage, BorderLayout.SOUTH);
  }

  /**
   * Updates the bid count property.
   */
  void _tfBidCount_actionPerformed(ActionEvent e)
  {
    _props.setProperty("Sup.BidCount", _tfBidCount.getText());
    setMessage("Set bid count to " + _tfBidCount.getText());
  }

  /**
   * Updates the bid size property.
   */
  void _cbBidSize_actionPerformed(ActionEvent e)
  {
    double size = decodeBidSize((String)_cbmBidSize.getSelectedItem());
    _props.setProperty("Sup.FollowLinkProb",
		       Double.toString(size));
    setMessage("Set bid size to " + size);
  }

  /**
   * Updates the bid size variability property.
   */
  void _tfSizeVar_actionPerformed(ActionEvent e)
  {
    String var = _tfSizeVar.getText();
    _props.setProperty("Sup.FollowLinkVariability", var);
    setMessage("Set bid size variability to " + var);
  }

  /**
   * Handles the Clear button
   */
  void _btnClear_actionPerformed(ActionEvent e)
  {
    _tfRfqId.setText("  ");
    _tfSubmitted.setText("  ");
  }

  /**
   * Sets the RFQ ID field.
   */
  public void setRFQId (String id)
  {
    _tfRfqId.setText(id);
  }

  /**
   * Sets the Bids Submitted field.
   */
  public void setBidsSubmitted (int count)
  {
    _tfSubmitted.setText(Integer.toString(count));
  }

  /**
   * Updates the display after a change in Properties
   */
  public void renderProps ()
  {
    _tfBidCount.setText(_props.getProperty("Sup.BidCount"));
    _tfSizeVar.setText(_props.getProperty("Sup.FollowLinkVariability"));
    _cbmBidSize.setSelectedItem(encodeBidSize
				(_props.getProperty("Sup.FollowLinkProb")));
  }

  /**
   * Converts the "FollowLinkProb" value into combo-box values
   */
  private String encodeBidSize (String bidSizeProp)
  {
    if(bidSizeProp == null) {
      return BID_SIZE_MEDIUM;
    }
    double result = 0.5;
    try {
      result = Double.valueOf(bidSizeProp).doubleValue();
    }
    catch (NumberFormatException nfe) {
      log.error("Bad bidSizeProperty spec " + bidSizeProp);
    }
    String code = BID_SIZE_MEDIUM;
    if (result >= 0.7) code = BID_SIZE_LARGE;
    else if (result <= 0.3) code = BID_SIZE_SMALL;
    return code;
  }

  /**
   * Turns a bid size string into a number that we can use
   */
  private double decodeBidSize (String bidSizeCode)
  {
    double result = 0.5;
    if (bidSizeCode.equals(BID_SIZE_SMALL)) result = 0.2;
    else if (bidSizeCode.equals(BID_SIZE_LARGE)) result = 0.8;
    return result;
  }

  /**
   * Updates the status message
   */
  public void setMessage (String newMessage)
  {
    _tfMessage.setText(newMessage);
  }

  // -------------- Static singleton management ----------------
  /**
   * Returns the singleton instance, initializing it if necessary
   */
  public static GuiFrame getInstance ()
  {
    if (singleton == null) {
      singleton = new GuiFrame(MagnetProperties.getProperties());
      singleton.pack();
      
      //Center the window
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension frameSize = singleton.getSize();
      if (frameSize.height > screenSize.height) {
	frameSize.height = screenSize.height;
      }
      if (frameSize.width > screenSize.width) {
	frameSize.width = screenSize.width;
      }
      singleton.setLocation((screenSize.width - frameSize.width) / 2,
			(screenSize.height - frameSize.height+300) / 2);
      singleton.show();
    }
    return singleton;
  }
}
