/* ---------------------------------------------------------------------
 * Magnet Protocol elements
 * $Id: BidAward.java,v 1.2.2.1 2002/05/05 19:41:55 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 * ---------------------------------------------------------------------
 * Revision History:
 * 
 * Thu Feb 22 17:07:34 2001  John Collins
 *   Moved to customerProtocol component, removed bid breakdown code.
 * 
 * Thu Oct 29 15:51:57 1998  John E. Collins
 *   Add serialization signature.
 * 
 * Sat Jun 27 09:36:58 1998  John E. Collins
 *   Move to protocol package.
 * 
 * Wed Jun  3 10:41:37 1998  John E. Collins
 *   Remove redundant inherited code, clean up formatting
 */

package edu.umn.magnet.protocol.cust;

import edu.umn.magnet.protocol.ProtocolElement;

import java.lang.String;
import java.util.Date;

/**
  * @author Brad Henrickson
  * Date created: 4-24-98
  * Last Modified: $Date: 2002/05/05 19:41:55 $
  */

/**
  * The customer uses this object to accept a bid.
  */

public class BidAward extends ProtocolElement
{
  // private data members
  private String   _bidId;    // ID of the associated Bid
  private String   _placeToShip;  // where to send the award

  // public member functions
  
  /**
   * Default Constructor
   */
  public BidAward()
    {
      // default inits
      _bidId = null;
      _placeToShip = null;
    }

  /**
   * Returns the Id of the associated Bid of this <code>BidAward</code>
   *
   * @return a <code>String</code> which is the ID of the
   *         associated <code>Bid</code>
   */
  public String getBidId()
    {
      return _bidId;
    }

  /**
   * Changes the association of this <code>BidAward to a <code>Bid</code>
   *
   * @param   the newBidId is the ID of the <code>Bid</code> that this
   *   <code>BidAward</code> is to be associated with
   *
   * @exception  throws an <code>InvalidDataException</code>
   *             if the newBidId is not a legal value
   */
  public void setBidId(String newBidId)
  {
    _bidId = newBidId;
  }

  /**
   * Gives this <code>BidAward</code> a new destination
   * 
   * @param	newPlaceToShip is the new location to where the
   *            BidAward is to be sent to
   *
   * @exception  an <code>InvalidDataException</code> is thrown if
   *             the newPlaceToShip value is not legal
   */
  
  public void setPlaceToShip(String newPlaceToShip)
  {
    _placeToShip = newPlaceToShip;
  }
  
  /**
   * Returns a <code>String</code> telling where the destination of the
   *		shipment is
   * 
   * @return	a <code>String</code> which is the name of the
   *            location to ship the <code>BidAward</code> to
   */
  public String	getPlaceToShip()
  {
    return _placeToShip;
  }
}
