/*-------------------------------------------------------------------------
 * Magnet Protocol Elements
 * $Id: SubtaskRqInfo.java,v 1.3.2.1 2002/05/05 19:41:56 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *-------------------------------------------------------------------------
 * Revision History:
 *     
 * Sun Apr 22 11:26:40 2001  John E. Collins
 *   #00000026 Provide access to JobInfo name as a String
 * 
 * Sun Sep 24 21:10:24 2000  John Collins
 *   Change name to SubtaskRqInfo
 * 
 * Wed Sep 16 12:01:17 1998  John E. Collins
 *   Deprecate getID, setID. Replace with getId, setId to match the
 *   rest of the Magnet system.
 * 
 * Sat Jun 27 16:19:07 1998  John E. Collins
 *   Move to protocol package
 */

package edu.umn.magnet.protocol.cust;

import edu.umn.magnet.protocol.JobInfo;
import edu.umn.magnet.protocol.InvalidDataException;

/** 
 * Annotates generic JobInfo with individual task ID
 */
public interface SubtaskRqInfo
{
  /**
   * Each SubtaskRequest within a plan must have a unique ID. It can be
   * set only once, either by a constructor or a setter.
   */
  public String getId ();
  
  /**
   * Set the ID of a subtask.
   * @exception InvalidDataException thrown if an attempt is made to change
   *     the ID after it has been set.
   */
  public void setId (String newID) throws InvalidDataException;
  
  /**
   *  Returns a description of the work to be performed.
   */
  public JobInfo getJobInfo ();

  /**
   * Returns the ID of the JobInfo for this work
   */
  public String getJobInfoName ();
  
  /**
   * Sets the job description
   */    
  public void setJobInfo (JobInfo props);

  /**
   * Sets the name of the JobInfo for this task.
   */
  public void setJobInfoName (String newJI);
}
