/*------------------------------------------------------------------------
 * MAGNET customer protocol components
 * $Id: TPEvent.java,v 1.1.2.1 2002/05/05 19:41:56 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 11Mar2001 DFK - Initial Entry
 */

package edu.umn.magnet.protocol.cust;

import java.util.EventObject;
//import edu.umn.magnet.protocol.cust.TaskPlan;  // just for javadoc's sake..


/** {@link TPEvent}s are generated by {@link TaskPlan} objects
 *  to notify listeners of changes.
 * <p>
 * From {@link EventObject} docs:
 * <i>
 * All Events are constructed with a reference to the object, the "source",
 * that is logically deemed to be the object upon which the Event in question
 * initially occurred upon.
 * </i>
 */
public class TPEvent extends EventObject {

  /** An arbitrary message string (is this of any use?) */
  protected String _message = null;

  /** Simple constructor.
   *  This communicates a generic change event:
   *  <i>"Something's changed, but I'm not sure what changed, so just
   *    update everything."</i>
   *  @param source   The object (typically keyword <tt>this</tt>) creating the event.
   */
  public TPEvent(Object source) {
    super(source);
  }

  /** A constructor that includes some arbitrary {@link String} message. */
  public TPEvent(Object source, String aMessage) {
    super(source);
    _message = aMessage;
  }

  public String getMessage() {
    return new String(_message);
  }

  /** Print out contents of this event. */
  public String toString() {
    String s = getClass().getName() +
                "[" +
                "source=" + source +
                ( (null==_message) ? "" : (",message="+_message) ) +
                "]";
    return s;
  }
}