/*------------------------------------------------------------------------
 * MAGNET Manager User Interface component
 * $Id: AwardDialog.java,v 1.6.2.1 2002/05/05 19:41:59 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 13May2001 PBG Initial Entry - Bug #0000056 and Bug #0000063
 * 14May2001 PBG Updated to add scroll bar to text area
 * 19May2001 PBG - Bug #0000091, #0000101: Keep flow similar to that of agent
 * 20May2001 PBG - Bug #0000071 Fixed so scroll bar can be scaled vertically
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.*;

import edu.umn.magnet.protocol.cust.*; // for TaskPlan
import edu.umn.magnet.client.status.*; // for rfq,bid info, status objects
import edu.umn.magnet.client.eval.TaskBidPlan;  // for the TaskBidPlan

/** This is the main class for the Magnet Client Bid Award Dialog
 *
 *  This will contain the dialog box and the respective Panel information for
 *  the Evaluation Results and Award
 *  Screen that the user sees after selecting to Award or Do Evaluate.
 *
 * This is a non-modal dialog, it should be able to allow for the following:
 *  - dispay eval results/award information
 *  - provide a button to do the award if desired
 */
public class AwardDialog {

  /** Dialog instance */
  private JDialog _awardDialog = null;

  /** Data used to refer to the bid information */
  private TaskBidPlan _bidPlan = null;
  private boolean isEvalResults = false;
  private boolean isValidAward = false;

  /** Data used for layout */
  private Frame _parent = null;
  private JPanel _awardPanel = null;
  JScrollPane _resultsScroll; // Put text area in scroll
  JTextArea _resultsText = new JTextArea();
  JButton _awardButton = new JButton();
  private BidEditorPanel _bpe = null;

  /** X-coord of dialog. (default 0) */
  private int XLocation = 0;
  /** Y-coord of dialog. (default 0) */
  private int YLocation = 0;


  /** Another constructor - this just calls the other - but defaults the
   *  isEvaluationResults to false
   */
  public AwardDialog(Frame parent, TaskBidPlan tbp, BidEditorPanel bidPanel) {
    new AwardDialog(parent, tbp, false, bidPanel);
  }

  /** Constructor - this requires lots of status variables - unfortunately */
  public AwardDialog(Frame parent, TaskBidPlan tbp, boolean isEvaluationResults,
                      BidEditorPanel bidPanel) {
   _parent = parent;
   _bidPlan = tbp;
   isEvalResults = isEvaluationResults;
   _bpe = bidPanel;
   displayDialog();

  } // end of AwardDialog

  /**
    * This displays the Dialog Box
   */
  public void displayDialog() {

    // the panel that displays the evaluation information
    JPanel resultsPanel = createResultsPanel();

    // create the eval dialog
    if (isEvalResults) {
      _awardDialog = new JDialog(_parent, "Results of Evaluation", false);
    } else {
      _awardDialog = new JDialog(_parent, "Results of Award", true);
    }
    _awardDialog.getContentPane().setLayout(new BorderLayout());
    _awardDialog.getContentPane().add(resultsPanel, BorderLayout.CENTER);
    _awardDialog.setLocation(XLocation, YLocation);
    _awardDialog.setSize(new Dimension(800, 600));
    //_awardDialog.setSize(resultsPanel.getPreferredSize());
    _awardDialog.show();

  } // end of displayDialog

  /** The following method sets up the results panel to be used in this
   *  dialog.  It keys off of the type of dialog to create for the initial
   *  part.  The remaining is the same.
   *  The dialog consists of a text box and then perhaps a button
   */
   private JPanel createResultsPanel() {
    JPanel newPanel = new JPanel(new GridBagLayout());

    _resultsText.setOpaque(false);
    _resultsText.setText(createString());
    _resultsText.setEditable(false);

    _resultsScroll = new JScrollPane(_resultsText);
    _resultsScroll.setMaximumSize(new Dimension(750, 500));
    _resultsScroll.setMinimumSize(new Dimension(200,100));
    _resultsScroll.setPreferredSize(new Dimension(600, 500));
    GuiUtil.constrain(newPanel, _resultsScroll,
                      0,0,  // Grid  x & Y
                      1,1,  // Grid Width & Height
                      GridBagConstraints.NONE,  // fill
                      GridBagConstraints.CENTER,   // anchor
                      0.5, 0.5,   // weight x & y
                      GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,GuiUtil.LAYOUT_MARGIN);

    if ((isEvalResults) && (isValidAward)) {
      _awardButton.setText("Award");
      _awardButton.setMnemonic('a');
      ActionListener aal = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
	    doAward();
       }};

	_awardButton.addActionListener(aal);
        _bpe.updateEvaluationWindows(this);

     GuiUtil.constrain(newPanel, _awardButton,
                      0,1,  // Grid  x & Y
                      1,1,  // Grid Width & Height
                      GridBagConstraints.NONE,  // fill
                      GridBagConstraints.CENTER,   // anchor
                      0.0, 0.0,   // weight x & y
                      GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,GuiUtil.LAYOUT_MARGIN);
    } else if (!isEvalResults) {
      _awardButton.setText("OK");
      _awardButton.setMnemonic('o');
      ActionListener oal = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        _awardDialog.dispose();
      }
      };
      _awardButton.addActionListener(oal);
    GuiUtil.constrain(newPanel, _awardButton,
                      0,1,  // Grid  x & Y
                      1,1,  // Grid Width & Height
                      GridBagConstraints.NONE,  // fill
                      GridBagConstraints.CENTER,   // anchor
                      0.0, 0.0,   // weight x & y
                      GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,GuiUtil.LAYOUT_MARGIN);
    } //end of isEvalResults//end of isEvalResults

    return newPanel;
  } // end of createResultsPanel

  /** The following method will create a formatted string of the results */
  private String createString() {
    String str = new String();

    if (_bidPlan == null) {
      str += "There currently is not an active TaskBidPlan - unable to continue";
      return str;
    }
    // The initial date differs for what this is - otherwise the same
    if (isEvalResults) {
      str += "Evaluation Information:\n";
      str += "\tEvaluation of This Plan:\t"+
              _bidPlan.getEvaluation()+"\n";
      str += "\tCost of This Plan:\t"+
                _bidPlan.getCost()+"\n";
      str += "\tTime to find Plan:\t" + _bidPlan.getSearchTime() + "\n";
      str +="\tMethod for this Search:\t"+_bidPlan.getSearchMethod() +"\n\n";
      str += "Bid Information for this Plan:\n";

    } else {
      str += "Awarded The Following Bids:\n";
      str += "Total Cost for All Bids:\t"+_bidPlan.getCost()+"\n\n";

    } // end of if for header

    // OK - here we are - let's just show what the bids were that we awarded
    //get the bidcollection in this particular taskbid
    HashSet bidHash = _bidPlan.getBids();
    if ((bidHash == null) || (bidHash.isEmpty())) {
      str += "No Bids Present.\n";
      return str;
    }

    for (Iterator iter=bidHash.iterator(); iter.hasNext(); ) {
      Bid _bid = (Bid) iter.next();
      str += GuiEvalUtil.printBid(_bid);
    }
    isValidAward = true;  // we are here - we know it is OK to offer for award
    return str;
  } // end of createString

  /** The following method goes through and verifies that the award date is
   *  here - if so, it will award.
   *
   * Note: The only time that this disposes of the award window is if it actually
   * calls the doAward.  The reason for this is if there is an error - the user
   * may want to go back and see what was on their screen that caused that
   * error.
   *
   */
   public void doAward() {
      SessionStatus ss = null;
      RFQStatus rs = null;
      PlanStatus ps = null;

      ss = MagnetClientMain.getBidManager().getSessionStatus();
      if (ss == null) {
        GuiUtil.inform(GuiUtil.getFrame(_awardDialog),
                        "There is no session information available",
                        "Bid Eval Error..");
        return;
      }

      ps = ss.getParent();
      if (ps == null) {
        GuiUtil.inform(GuiUtil.getFrame(_awardDialog),
                        "There is no plan information available",
                        "Bid Eval Error..");
        return;
      }
      java.util.Collection rfqSet = ss.getRFQs();
      rfqSet = ss.getRFQs();
      if (rfqSet.isEmpty()) {
         return;
      }

      for (Iterator rsi = rfqSet.iterator(); rsi.hasNext();) {
        rs = (RFQStatus)(rsi.next());
        break;
      }

      if (rs == null) {
        GuiUtil.inform(GuiUtil.getFrame(_awardDialog),
                        "There is no RFQ information available",
                        "Bid Eval Error..");
        return;
      }

      // Make sure that the time to accept bids has arrived
      if (!(rs.getState().equals(rs.AWARD_READY))) {
        GuiUtil.inform(GuiUtil.getFrame(_awardDialog),
		       "It is too early to accept bids", "Bid Award Error..");
	return;
      }
      GuiEvalUtil.doAward(_bidPlan,ps,_parent,_bpe);
      _awardDialog.dispose();

      return;
    } // end of doAward


    public void disposeAwardWindow(){
          _awardDialog.dispose();
    }

} // end of AwardDialog Class
