/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: ErrorObjectHandler.java,v 1.3.2.1 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 26Apr2001 DFK - Initial Entry
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.*;

import org.apache.log4j.Category;

/** A non-modal Dialog to display errors and switch focus to the offending GUI component(s).
 */
public class ErrorObjectHandler {
  private Vector errors = new Vector();
  private ErrorFocus errorFocus = null;
  private ActionListener cancelButtonActionListener = null;
  private JDialog delegate = null;
  private JList errorList = null;
  private JList _bogusList = new JList(new String[]{" "});    // for list cell rendering
  private static final int THISWIDTH = 400;
  private static final int THISHEIGHT = 300;
  /** X-coord of error window. (default 0) */
  private int errorXLocation = 0;
  /** Y-coord of error window. (default 0) */
  private int errorYLocation = 0;

    /** log category */
  private static Category log =
                    Category.getInstance(ErrorObjectHandler.class.getName());
  /** Construct a handler to handle the passed-in errors. */
  public ErrorObjectHandler(Vector myErrors) {
    if (null!=myErrors)  {  errors=myErrors;  }
  }

  /**
   * Displays the errors in this ErrorObjectHandler
   * @param parent the parent of the error dialog to be displayed
   * @param focus the class that implements ErrorObjectHandler.ErrorFocus which
   *        will handle the fixing the errors after a user double-clicks or selects
   *        the "Go To..." button
   * @param doneButtonActionListener This allows the user of
   *        ErrorObjectHandler to create and pass in an actionListener that repeats
   *        the same action that caused the ErrorObjectHandler to be
   *        generated to test if all the errors have
   *        been handled, if <code>doneButtonActionListener</code> is
   *        <code>null</code> a done button is not added to the error dialog
   * @param cancelButtonActionListener addition action to be taken when the
   *        error dialog is closed by selecting the cancel
   *        button, the dialog is automatically disposed of when the cancel
   *        button is selected
   */
  public void displayErrors(Frame parent, ErrorObjectHandler.ErrorFocus focus,
          ActionListener doneButtonActionListener,
          ActionListener cancelButtonActionListener) {

    // the panel that displays the list of errors
    JPanel errorPanel = buildDisplayErrorsPanel(focus);
    // the buttons for the button panel
    Vector buttons = buildDisplayErrorsButtons(doneButtonActionListener, cancelButtonActionListener);
    // the button panel on the bottom of the error dialog
//    ButtonPanel buttonPanel = new ButtonPanel(buttons, null);
    JPanel buttonPanel = new JPanel();
      for (int i=0; i < buttons.size(); i++) {
        buttonPanel.add((JButton)buttons.elementAt(i));
      }
    // create the error dialog
//    delegate = DialogWithHelp.createErrorDialog(parent, "Error Messages",
//            errorPanel, buttonPanel);
    delegate = new JDialog(parent, "Error Messages", false);
      delegate.getContentPane().setLayout(new BorderLayout());
      delegate.getContentPane().add(errorPanel, BorderLayout.CENTER);
      delegate.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
    delegate.setLocation(errorXLocation, errorYLocation);
    delegate.setSize(new Dimension(THISWIDTH, THISHEIGHT));
    delegate.show();
  }

  /**
   * Builds the panel where the errors are displayed in a list
   */
  private JPanel buildDisplayErrorsPanel(ErrorObjectHandler.ErrorFocus focus) {
    this.errorFocus = focus;
    errorList = new JList();
    // Give the JList a renderer that knows how to display ErrorObjects by title..
    errorList.setCellRenderer(new DefaultListCellRenderer (){
        public Component getListCellRendererComponent(
             JList list,
             Object value,
             int index,
             boolean isSelected,
             boolean cellHasFocus)
        {
             DefaultListCellRenderer dRenderer = new DefaultListCellRenderer();
             dRenderer.getListCellRendererComponent(_bogusList, " ", 0, isSelected, cellHasFocus);
             JLabel aLabel = new JLabel();
             aLabel.setOpaque(true);
             aLabel.setBackground(dRenderer.getBackground());
             aLabel.setForeground(dRenderer.getForeground());
             if (value instanceof ErrorObject  &&  null!=value) {
               aLabel.setText( ((ErrorObject)value).getMessage() );
             }
             return (JComponent)aLabel; //value;
        }
     });
    errorList.setListData(errors);
    final JScrollPane spErrorScrollPane = new JScrollPane(errorList);
    JPanel panel = new JPanel(new BorderLayout(GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN));

    JWrappingLabel infoLabel = null;
    infoLabel = new JWrappingLabel("These errors occured while processing" +
      " the data.  Please select an error and either" +
      " double click or press the 'Go To...' button" +
      " which will bring you to the field where you" +
      " can reenter the data.");

    // verify that we could do this for JWrappingLabel
    JLabel getFontLabel = new JLabel();
    Rectangle2D r = infoLabel.getFontMetrics(getFontLabel.getFont()).getStringBounds(infoLabel.getText(), infoLabel.getGraphics());
    getFontLabel = null;
    Double w = new Double(r.getWidth());
    int width = w.intValue();
    int labelWidth = THISWIDTH - (GuiUtil.LAYOUT_MARGIN * 4);  //margins
    int rows = width/labelWidth + 1;  // label width divided by the width of the container plus one (for the remainder)
    Double h = new Double(r.getHeight());
    int height = h.intValue();
    int labelHeight = height * rows;
    infoLabel.setPreferredSize(new Dimension(labelWidth, labelHeight));
    // verify that we could do this for JWrappingLabel

    panel.add(spErrorScrollPane, BorderLayout.CENTER);
    panel.add(infoLabel, BorderLayout.NORTH);

    // listen for the double click
    errorList.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent ev) {
        if(ev.getClickCount() == 2) {
            goToError();
        }
      }
    });
    return panel;
  }

  /**
   * Builds the buttons for the error dialog
   */
  private Vector buildDisplayErrorsButtons(ActionListener doneButtonActionListener, ActionListener cancelButtonActionListener) {

    final JButton goTo = new JButton("Go To...");
    final JButton finished = new JButton("Try Again");
    final JButton cancel = new JButton("Close");
    final JButton help = new JButton("Help");
//    try {
//      HelpManager.enableHelp(help, HelpManager.GENERIC_ERROR_MAPID);  //enable Help
//    } catch(SBException.HelpNotInitializedException e) {
//    }
    ActionListener buttonListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand().equals(goTo.getActionCommand())) {
          goToError();
        } else if(e.getActionCommand().equals(cancel.getActionCommand())) {
          delegate.dispose();
        } else if(e.getActionCommand().equals(finished.getActionCommand())) {
          delegate.dispose();
        }
      }
    };
    goTo.addActionListener(buttonListener);
    cancel.addActionListener(buttonListener);
    finished.addActionListener(buttonListener);
    finished.addActionListener(doneButtonActionListener);
    cancel.addActionListener(cancelButtonActionListener);
    Vector buttons = new Vector();

    buttons.add(goTo);
    if(doneButtonActionListener != null) {
      buttons.add(finished);
    }

    buttons.add(cancel);
    buttons.add(help);
    return buttons;
  }

  /**
   * Take the selected object in the list and call ErrorObjectHandler.ErrorFocus.setErrorFocus
   */
  private void goToError(){
    int i = errorList.getSelectedIndex();
    if(i < 0) {
      //nothing is selected
//      DialogWithHelp.showMessageDialog(delegate, "Go To Error",
//              "Please select an error from the list.  \n " +
//              "Then either double click or press the \n " +
//              "'Go To...' button to fix the error.",
//              DialogWithHelp.OK_BUTTON, DialogWithHelp.OK_BUTTON,
//              null);
      inform( "Go To Error: \n" +
              "Please select an error from the list.  \n " +
              "Then either double click or press the \n " +
              "'Go To...' button to fix the error.");
    } else {
      ErrorObject o = (ErrorObject)errorList.getModel().getElementAt(i);
      // set the error focus
      if(errorFocus != null) {
        // request that the focus is transfered from the error dialog to the parent
        //      so that the place to fix the errors is getting the focus
        delegate.getParent().requestFocus();
        errorFocus.setErrorFocus(o);
      } else {
        log.info("EOH.goToError(): errorFocus == null");
      }
    }
  }

  /** A quick temporary method to pop up the string in an info dialog.. */
  public void inform(String s) {
    int result = JOptionPane.showConfirmDialog(null/*this*/,
                        new JWrappingLabel(s),
                        "Info..", JOptionPane.CANCEL_OPTION,
                        JOptionPane.INFORMATION_MESSAGE);
  }

  /**
   * Interface for setting the focus to the field where the error occured
   */
  public interface ErrorFocus {
    void setErrorFocus(ErrorObject errorObject);
  }

  /** Set the location of the Error window to the specified (x,y). */
  public void setErrorLocation(int x, int y) {
    if (x<0  ||  y<0) {
      log.error("setErrorLocation(): Bad coords ("+x+","+y+").");
      return;
    }
    errorXLocation = x;
    errorYLocation = y;
  }

  /** Get the current X-coordinate location of the Error window. */
  public int getErrorXLocation() {
    return errorXLocation;
  }

  /** Get the current Y-coordinate location of the Error window. */
  public int getErrorYLocation() {
    return errorYLocation;
  }

  /** Position the Error window to the upper right hand corner of
   *  the current display.
   *  @todo Consider generalizing this and pass in Enums for
   *          UPPER_RH, LOWER_RH, etc..
   */
  public void parkUpperRHCorner() {
    Dimension screenSize =  Toolkit.getDefaultToolkit().getScreenSize();
    int newX = (int) screenSize.getWidth() - this.THISWIDTH;
    int newY = 0;  //(int) screenSize.getHeight() - this.THISHEIGHT;
    setErrorLocation(newX, newY);
  }
}
