/*------------------------------------------------------------------------
 * MAGNET Manager User Interface component
 * $Id: EvalDialog.java,v 1.4.2.1 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 06May2001 PBG Initial Entry
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.*;

import edu.umn.magnet.protocol.cust.*; // for TaskPlan
import edu.umn.magnet.client.status.*; // for rfq,bid info, status objects

/** This is the main class for the Magnet Client Bid Evaluation Interface
 *
 *  This will contain the dialog box and the respective Panel information for
 *  the Evaluation Screen that the user sees and configures for Bid Evaluation.
 *
 * This is a non-modal dialog, it should be able to allow for the following:
 *  - offer evaluation choices to the user
 *  - provide a progress bar and a means to interrupt the evaluation if necessary
 *  - work with the bid set that excludes the bids as configured by the user
 */
public class EvalDialog {

  /** Dialog instance */
  private JDialog _evalDialog = null;

  /** Data used to refer to the bid information */
  private Vector _include = null;
  private java.util.Vector _bidCollection = null;
  private TaskPlan _taskPlan = null;
  private RFQStatus _rfqStatus = null;

  /** Data used for layout */
  private Frame _parent = null;
  private JPanel _evalPanel = null;
  private BidEditorPanel _bpe = null;

  /** X-coord of dialog. (default 0) */
  private int XLocation = 0;
  /** Y-coord of dialog. (default 0) */
  private int YLocation = 0;

  /** Constructor - this requires lots of status variables - unfortunately */
  public EvalDialog(Frame parent, Vector includes, java.util.Vector bc,
                            TaskPlan tp, RFQStatus rfq, BidEditorPanel bidPanel) {
   _parent = parent;
   _include = includes;
   _bidCollection = bc;
   _bpe = bidPanel;
   _taskPlan = tp;
   _rfqStatus = rfq;
   displayDialog();

  } // end of EvalDialog

  /**
    * This displays the Dialog Box
   */
  public void displayDialog() {

    // the panel that displays the evaluation information
    EvalEditorPanel evalPanel = new EvalEditorPanel(_include, _bidCollection,
                                    _taskPlan, _rfqStatus, _bpe);

    // create the eval dialog
    _evalDialog = new JDialog(_parent, "Evaluation", false);
    _evalDialog.getContentPane().setLayout(new BorderLayout());
    _evalDialog.getContentPane().add(evalPanel, BorderLayout.CENTER);
    evalPanel.setDialog(_evalDialog);
    _evalDialog.setLocation(XLocation, YLocation);
    _evalDialog.setSize(new Dimension(800, 300));
    _evalDialog.show();

  } // end of displayDialog

} // end of EvalDialog Class