/*------------------------------------------------------------------------
 * MAGNET Manager User Interface component
 * $Id: EvalEditorPanel.java,v 1.13.2.3 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Jan  2 17:01:59 2002  John Collins #0000163
 *   remove dependency on Borland library
 *
 * 06May2001 PBG Initial Entry
 * 07May2001 PBG Updated Evaluation Tasks
 * 13May2001 PBG Bug #0000006,#0000063: Updated so evaluation shows results
 *                there is a timer bar for the search
 * 16May2001 PBG Passed properties to eval engine
 * 26May2001 PBG Bug #0000055: Display errors in popups to users
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.client.status.*; // for the status objects
import edu.umn.magnet.protocol.cust.Bid;     // for Bid data structure
import edu.umn.magnet.protocol.cust.SubtaskRequest; // for Subtask Structure
import edu.umn.magnet.client.eval.*;  // for the Evaluation stuff

import edu.umn.magnet.util.MagnetProperties;
import edu.umn.magnet.util.time.AgentTime;   // for example date strings
import edu.umn.magnet.client.status.TMEntry;
import edu.umn.magnet.util.time.TimerAction;

import org.apache.log4j.Category;  // for the logging

/** This is the class that will display the panel that contains the
 *  evaluation configuration options.  Currently these options are
 *  displayed contained within a dialog - however, it seemed as though
 *  this may be something that we eventually want from the NavPanel - so
 *  it has been all placed into this modular panel format.
 *
 *
 * GUI details:
 * This panel is split into 2 different panels.
 *  - the upper panel contains the evaluation type configuration
 *  - the lower panel only becomes visible after the "Go" Button is selected
 *      this will contain a text area, a JProgress Bar, and a Button to cancel.
 */
public class EvalEditorPanel extends JPanel
  implements ActionListener
{
  /** Private Variables */
  //log category information
  private static Category log =
    Category.getInstance(EvalEditorPanel.class.getName());

  /** Data used to refer to the bid information */
  private Vector _include = null;
  private Object _bidCollection[] = null;
  private TaskPlan _taskPlan = null;
  private RFQStatus _rfqStatus = null;
  private BidEvaluator be = null;

  /** Constants used within */
  private final String EPIP_STRING = "IP Search";
  private final String EPSA_STRING = "Simulated Annealing";
  private final String EPAS_STRING = "A*";
  private final String EPID_STRING = "Iterative Deepening A*";

  // milliseconds until we timeout - default to 30 minutes
  private final static int ONE_SECOND = 1000;
  private final long defaultTimeout = 30*60*60*ONE_SECOND;
  private TimeMap map = new TimeMap();
  private TMEntry EndSearchTME = null;
  private long StartTime = 0;


  private JDialog _parent = null;
  private BidEditorPanel _bpe = null;

  /** Public Variables */
  // The 2 main components
  JPanel _epSelectPanel = new JPanel(new FlowLayout());
  JPanel _epProgressPanel = new JPanel(new FlowLayout());

  // The Select Panel components
  JLabel _epsEvalLabel = new JLabel();
  JComboBox _epsEvalCombo = new JComboBox();
  JButton _epsGoButton = new JButton();
  JLabel _epsTimoutLabel = new JLabel();
  JTextField _epsTimeoutText = new JTextField();

  // The Progress Panel Components
  JTextArea _eppPatienceText = new JTextArea();
  JProgressBar _eppEvalProgress = new JProgressBar();
  JButton _eppCancelButton = new JButton();
  private javax.swing.Timer timer;
  private long curProgress = 0;
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel _epChoosePanel = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  BorderLayout borderLayout3 = new BorderLayout();


  /** This is the base and only constructor for the class
   *
   *  This requires the RFQStatus object in order to do evaluation
   *  This requires the TaskPlan in order to do evaluation
   *  This requires the Bid Collection in order to be in sync with the includes
   *  This requires the includes so it knows which bids to include in
   *  evaluation
   */
  public EvalEditorPanel(Vector includes,
			 java.util.Collection bc,
			 TaskPlan tp,
			 RFQStatus rfq,
			 BidEditorPanel bidPanel)
  {
    _include = includes;
    if (bc != null) _bidCollection = bc.toArray();
    _taskPlan = tp;
    _rfqStatus = rfq;
    _bpe = bidPanel;
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    //addComponentsToPanel();

  } // end of EvalEditorPanel

  /** Add the components to the panel
    *  This will create and display all of the components of the EvalPanel
    */
  //public void addComponentsToPanel() {
  public void jbInit ()
  {
    // Create/add Views
    // add the 2 bigger containers  to this main panel

    // This is where all of the choose Evaluation Method stuff goes
    _epsEvalLabel.setText("Choose Method:");

    _epsEvalCombo.setEditable(false);
    _epsEvalCombo.addItem(EPID_STRING);
    _epsEvalCombo.addItem(EPAS_STRING);
    _epsEvalCombo.addItem(EPIP_STRING);
    _epsEvalCombo.addItem(EPSA_STRING);

    _epsGoButton.setText("Evaluate");
    _epsGoButton.setMnemonic('e');
    ActionListener eal = new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  doEvaluate();
	  _epProgressPanel.setVisible(true);
	  _epsEvalCombo.setEnabled(false);
	  _epsGoButton.setEnabled(false);
	}
      };
    _epsGoButton.addActionListener(eal);

    _epSelectPanel.setLayout(borderLayout2);
    _epProgressPanel.setLayout(borderLayout3);
    _epSelectPanel.add(_epChoosePanel,  BorderLayout.NORTH);
    _epChoosePanel.add(_epsEvalLabel, null);
    _epChoosePanel.add(_epsEvalCombo, null);
    _epSelectPanel.add(_epsGoButton,  BorderLayout.SOUTH);

    // THis is where all of the view evaluation progress stuff goes
    _eppPatienceText.setWrapStyleWord(true);
    _eppPatienceText.setLineWrap(true);
    _eppPatienceText.setOpaque(false);
    _eppPatienceText.setText(new String("Note:  This may take several minutes and"+
					" can be difficult to accurately predict progress.  "+
					"You may cancel at any time."));

    // do we want to set the model or min/max for this here?_eppEvalProgress
    _eppCancelButton.setText("Cancel, show best current result (if any)");
    _eppCancelButton.setMnemonic('c');
    ActionListener cal = new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  doCancel();
	}
      };
    _eppCancelButton.addActionListener(cal);

    _epProgressPanel.add(_eppPatienceText,  BorderLayout.NORTH);
    _epProgressPanel.add(_eppEvalProgress, BorderLayout.CENTER);
    _epProgressPanel.add(_eppCancelButton, BorderLayout.SOUTH);
    _epProgressPanel.setVisible(false);

    // Now add the above panels to this main panel
    this.setLayout(borderLayout1);
    this.add(_epSelectPanel,  BorderLayout.NORTH);
    this.add(_epProgressPanel, BorderLayout.CENTER);
  } // end of addComponentsToPanel

  /** A Generic action listener to listen for button press events */
  public void actionPerformed(ActionEvent e)
  {
    log.info("BEvalPanel.actionPerformed: Unknown source "+e.getSource());
  }

  /** Spawns the evaluation process selected by the JComboBox */
  public void doEvaluate()
  {
    TaskBidPlan tbp = null;
    PlanStatus ps = null;
    boolean addedBid = false;

    if ((_bidCollection == null) || (_rfqStatus == null)) {
      /** @todo: Display error to the user that can't evaluate bids */
      return;
    }

    // Make sure that the time to accept bids has arrived
    if ((_rfqStatus.getState().equals(_rfqStatus.NONE)) ||
        (_rfqStatus.getState().equals(_rfqStatus.AWARDED))||
        (_rfqStatus.getState().equals(_rfqStatus.COMPLETE)) ||
        (_rfqStatus.getState().equals(_rfqStatus.ABANDONED))) {
      /** @todo: Display error to the user that it is too early to award bids */
      GuiUtil.inform(_parent, "It is too early to award Bids",
		     "Error awarding Bids");
      return;
    } //end of if OK state

    /** @todo: I need to create a props type and feed it in */
    ps = GuiUtil.getPlanStatus(AgentStatus.getInstance(), _taskPlan);
    be = GuiEvalUtil.createBidEvaluator(ps, _rfqStatus, _taskPlan);

    if (be == null) {
      // error creating Bid Evaluator
      GuiUtil.inform(_parent, "Internal Error, unable to create Bid Evaluator",
		     "Error evaluating Bids");
      return;
    }

    be.addListener(new PropertyChangeListener() {
	public void propertyChange (PropertyChangeEvent pce) {
	  if (pce instanceof BidEvaluatorEvent) {
            handleBidEvaluatorEvent((BidEvaluatorEvent) pce);
            log.debug("Property Change Status in Bid Eval");
	  }
	}
      });

    // set the properties for the bid evaluator
    be.setProperties(MagnetProperties.getProperties());

    // set the search engine type - it is SA by default
    if (_epsEvalCombo.getSelectedItem().equals(EPIP_STRING)) {
      be.setSearchEngineType(be.IPCON);
    } else if (_epsEvalCombo.getSelectedItem().equals(EPSA_STRING)) {
      be.setSearchEngineType(be.SA);
    } else if (_epsEvalCombo.getSelectedItem().equals(EPAS_STRING)) {
      be.setSearchEngineType(be.ASTAR);
    } else if (_epsEvalCombo.getSelectedItem().equals(EPID_STRING)) {
      be.setSearchEngineType(be.IDASTAR);
    }

    //Identify the bid set for the award - exclude bids that are set to
    // excluded
    if (_bidCollection != null) {
      for (int i=0; i <_include.size(); i++)  {
	if ((!(((String)_include.get(i))
	       .equals(BidTableModel.BPNO_STRING)))
	    && (!(((String)_include.get(i))
		  .equals(BidTableModel.BPAUTONO_STRING)))) {
          // add this to the collection of bids to be included
          Bid b = (Bid) _bidCollection[i];
          if (b != null) {
	    //tbp.addBid(b);
	    be.addBid(b);
	    addedBid = true;
          } // end of if can add bid
        } // end of if it equals it
      } // end of the for loop
    } // end of if bids

    if (addedBid == false) {
      GuiUtil.inform(_parent, "Unable to evaluate bids as none are entered",
		     "Error Evaluating Bids");
      return;
    }

    // OK - kick off evaluation process...  This is spawned in a
    // different thread
    be.startSearch();

    // Now set search end time
    if (EndSearchTME != null) map.deleteEntry(EndSearchTME);
    StartTime = AgentTime.getDate().getTime();
    Date newDate = new Date(StartTime+defaultTimeout);
    EndSearchTME = new TMEntry(newDate, new TimerAction() {
	public void action() {
	  be.abortSearch();
	}
      });
    map.addEntry(EndSearchTME);

    // so, we have a default timeout - can't do real progress bar - so just
    // show how long until we are there
    _eppEvalProgress.setMaximum((int) defaultTimeout);
    _eppEvalProgress.setValue(0);
    _eppEvalProgress.setStringPainted(true);

    // need to create a timer so that we can wake up and update
    // Need to update using agent time
    timer = new javax.swing.Timer(ONE_SECOND, new ActionListener() {
	public void actionPerformed(ActionEvent evt) {
	  _eppEvalProgress.setValue((int)
				    (AgentTime.getDate().getTime()-
				     StartTime));
	}
      });
    timer.start();
  } // end of doEvaluate

  /** Cleans up and cancels the evaluation process */
  public void doCancel()
  {
    timer.stop();
    _eppEvalProgress.setValue(0);
    be.abortSearch();
    //abortSearch will throw an event that will take care of cleanup
  } // end of doCancel


  /** Handles any updates from our Bid Evaluation.
   *  The updates that we can receive are:
   *  <ul>
   *  <li> BidEvaluator.NEW_RESULT -- a new Best Result Was found
   *  <li> BidEvaluator.SEARCH_COMPLETE -- the search ended normally
   *  <li> BidEvaluator.SEARCH_ABORTED -- abortSearch was called on the Bid Eval
   *  </ul>
   */
  public void handleBidEvaluatorEvent(BidEvaluatorEvent bee)
  {
    if (bee.getPropertyName() == null) {
      log.info("BidEvaluatorEvent set with no value");
      return;
    } else if (bee.getPropertyName().equals(BidEvaluator.NEW_RESULT)) {
      // update the screen showing current best result
      TaskBidPlan tbp = ((BidEvaluator) bee.getSource()).getBestPlan();
      String str = "Current Best Plan is:\n";
      str += "Cost:\t"+tbp.getCost()+"\n";
      str += "Evaluation Ranking is:\t"+tbp.getEvaluation();
      _eppPatienceText.setText(str);
      log.debug("A new better result was found");
      return;
    } else if (bee.getPropertyName().equals(BidEvaluator.SEARCH_COMPLETE)) {
      // search is ended - show the current bestTaskBidPlan
      new AwardDialog(GuiUtil.getFrame(_epSelectPanel),
		      ((BidEvaluator) bee.getSource()).getResult(),true, _bpe);
      timer.stop();
      log.debug("Finished the evaluation");
      _parent.dispose();
      return;
    } else if (bee.getPropertyName().equals(BidEvaluator.SEARCH_ABORTED)) {
      // search was aborted
      TaskBidPlan tbp = null;
      tbp = ((BidEvaluator) bee.getSource()).getResult();
      if (tbp == null) {
        tbp = ((BidEvaluator) bee.getSource()).getBestPlan();
      }
      if (tbp != null) {
        /** @todo: Display that only an intermediate plan is being shown */
        new AwardDialog(GuiUtil.getFrame(_epSelectPanel),
                        tbp, true, _bpe);
      } else {
        /** @todo: Display that no intermediate diagnostic could be made */
      }

      _parent.dispose();
      log.info("Search was aborted");
      return;
    } else {
      log.info("Unkown BidEvaluatorEvent Type seen");
    }
    return;
  } // end of handleBidEvaluatorEvent

  /** Sets the _parent once it is defined */
  public void setDialog(JDialog _dialog)
  {
    _parent = _dialog;
  }
}
