/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: GanttPanel.java,v 1.9.2.1 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 11Apr2001 DFK - Initial Entry
 * 02May2001 DFK - Bug #0000022: refresh ganttTask list after plan changes
 * 13May2001 DFK - Bug #0000053: Added support for bid Gantt objs.
 * 21May2001 DFK - Bug #0000099: Add support for slack tails (to visualize slack.)
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;

import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.protocol.JobInfo;

import org.apache.log4j.Category;

public class GanttPanel extends TimeframePanel {

  /** Our set of {@link SubtaskRequest} Gantt objects */
//  protected GanttSlackTask[] ganttTasks = null;
  protected Vector ganttTasks = null;
  /** Our list of task bid Gantt objects. */
  protected Vector ganttTaskBids = null;

  /** @todo add getters/setters for drawXXX (tasks,bids). */
  protected boolean drawTasks = true;
  protected boolean drawBids = true;

  /** log category */
  private static Category log =
                    Category.getInstance(GanttPanel.class.getName());

  /** @todo Add hooks into TPTableModel to query JTable cell/hdr sizes */
  /** @todo Add member refs to the TaskPlan (and eventually TaskBidPlan). */
  /** @todo Consider adding hook in TPTableModel to listen to JTable row resizes,
   *          to fire "recalc dimensions" events to corresponding Gantt objs..??
   */



  public GanttPanel() {
    super(null);
    doSetup();
  }
  public GanttPanel(TPGanttModel m) {
    super(m);
    m.setView(this);
    doSetup();
  }

//  /** Do L1 test setup stuff..
//   *  @todo Delete this stuff...
//   */
//  private void doSetup_old() {
//    // Extra stuff..  delete!!!!!!!!!!!!!!!!!!!!!
//    // Draw additional stuff
//    SubtaskRequest subtask1=null;
//    try {
//      subtask1 = new SubtaskRequest("TaskID1");
//          GregorianCalendar c = new GregorianCalendar();
//          subtask1.setEarliestTimeToStart(c.getTime());
//          c.add(c.DAY_OF_YEAR, 2);
//          subtask1.setDeadline(c.getTime());
//          JobInfo jobInfo = new JobInfo("name1", "descrip for name1", 1.0, 1.0,
//                                1.11, 1.0,
//                                1.0,1.0,1.0 );
//          subtask1.setJobInfo(jobInfo);
//    } catch (Exception e) {  e.printStackTrace(); }
//    ganttTasks = new Vector();
//    ganttTasks.addElement(new GanttSlackTask(subtask1, this));
//  }

  /** Get task info from the associated task plan;  Reload and redraw the entire list. */
  private void doSetup() {
    log.debug("GP.doSetup(): entering");
    TPGanttModel gm=null;
    TPTableModel tm=null;
    ganttTasks = null;
    /** @todo Should check for nulls before dereferencing... */
    if (null!=(gm = this.getGanttModel())) {
      if (null!=(tm = gm.getTPTableModel())) {
        int tasksRowCount = tm.getRowCount();
        if (tasksRowCount > 0) {
//          ganttObjs = new Vector();

        // debug... DELETE THIS...
        String tempListIDs = new String("");
        for (int zz=0; zz < tasksRowCount; zz++) {
          tempListIDs += tm.getTask(zz).getId() + ", ";
        }
        log.debug("GP.doSetup(): TaskPlan now contains IDs: " + tempListIDs);

          // Add Tasks to the list of Gantt objects.
          ganttTasks = new Vector();
          for (int i=0; i < tasksRowCount; i++) {
            SubtaskRequest req = tm.getTask(i);
            GanttSlackTask aGanttTask = new GanttSlackTask(req, this);
            ganttTasks.addElement(aGanttTask);
            Color[] color = new Color[]{Color.red, Color.orange, Color.yellow, Color.green,
                                        Color.blue, Color.magenta, Color.gray};
            ((GanttSlackTask)ganttTasks.elementAt(i)).middleColor = color[i % color.length];
          }//for each task
        }//if there were any tasks.
      }
    }
//    log.debug("GP.doSetup(): about to call 'this.repaint()'..");
//    this.repaint();   // GET RID OF THIS
  }//doSetup()

  /** Overlay the specified bid, by overlaying all component tasks.
   *  <B>NOTE</B>: I'm assuming only 1 bid selected at a time!
   */
  public void addBid(Bid b) {
    if (null==b) {  return;  }
    log.debug("GP.addBid(id="+b.getId()+"): entering");
    TPTableModel tptm = getTPTableModel();
    if (null==tptm) {  return;  }
    TaskPlan tp = tptm.getTaskPlan();
    if (null==tp) {  return;  }
    ganttTaskBids = new Vector();     // flush out the list of bids
    // Get list of taskIDs for this bid.
    Set sTaskIDs = b.getTaskSet();
    if (!sTaskIDs.isEmpty()) {
      // Add each task row with highlight style 0.
      for (Iterator i=sTaskIDs.iterator(); i.hasNext(); ) {
        String taskID = (String)i.next();
        SubtaskRequest req = tp.getTaskData(taskID);
        TaskWithPrecedence twp = tp.getTWP(taskID);
        GanttSlackTaskBid aGanttSlackTaskBid = new GanttSlackTaskBid(twp, b, this);
        ganttTaskBids.addElement(aGanttSlackTaskBid);
      }//for
      // Finally, fire a "change of gantt view" event.
      TPGanttModel gm = getGanttModel();
      if (null!=gm) {
        gm.fireGanttDataChanged("Bid "+b.getId()+" superimposed on task(s)");
        this.repaint();   // how to force redraw to see bids??
      }
    }
  }

  /** Tell the panel to repaint itself, then all its gantt objs. */
  public void paintComponent(Graphics g) {
    log.debug("GP.paintComponent(): entering");
    super.paintComponent(g);
    Graphics2D g2 = (Graphics2D) g;
    if (drawTasks) {
      if (null!=ganttTasks  &&  0<ganttTasks.size()) {
        for (int i=0; i < ganttTasks.size(); i++) {
          ((GanttSlackTask)ganttTasks.elementAt(i)).paint(g2);
        }
      } else {
        log.debug("GP.paintComponent(): no tasks to draw");
      }
    }
    if (drawBids) {
      if (null!=ganttTaskBids  &&  0<ganttTaskBids.size()) {
        log.debug("GP.paintComponent(): about to draw "+ganttTaskBids.size()+" bids");
        for (int i=0; i < ganttTaskBids.size(); i++) {
          ((GanttSlackTaskBid)ganttTaskBids.elementAt(i)).paint(g2);
        }
      } else {
        log.debug("GP.paintComponent(): no taskBids to draw");
      }
    }
  }

  // Extension from TimeframePanel...
  /** Indicates (in a vague fashion) that something in the chart changed. */
  public void chartChanged(TPGanttEvent e) {
    super.chartChanged(e);
    log.debug("GP.chartChanged(): entering with e="+e.toString());
    // Objs may have been added/deleted;  reload and redraw entire list.
    doSetup();
    log.debug("GP.chartChanged(): about to call paintComponent().");
    this.revalidate();
    this.repaint();
//    Graphics g = this.getGraphics();
//    if (null!=g)  {  this.paintComponent(this.getGraphics());  }
  }



  /** Inner class to represent dependency arrows.
   *  @todo Needo to implement dependency arrows!
   */
  public class GanttArrow {

  }



  // DAVE:
  //   Try drawing a gantt item from a SubtaskRequest obj.
  //      (For now, use JTable cell default sizes
  //        ( x 16+1 for RowHeight + IntercellSpacing)
  //      for hardcoded Gantt object sizes.)
  //      See also JTable#getCellRect() for cell dimensions, or
  //        JTable.getTableHeader().getHeaderRect(int) for header dimensions.
  //   Since row heights can vary from row to row, the TableModel should perhaps
  //    have a method to make an int[] rowHeights array (corresp w/ the # of rows),
  //    where each elt is that row's height.  (Remember RowMargin/IntercellSpacing is extra..)
  //
  //   Next, consider making a JSplitPane, and putting a table in LH ScrollPaned pane
  //   and GanttPanel in RH ScrollPaned pane.   See what sizing issues there are.


  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) {
      e.printStackTrace();
    }
    JFrame f = new JFrame("Just a quickie L1 test main");
    GanttPanel gPanel = new GanttPanel();
    f.getContentPane().add(new JScrollPane(gPanel));
    f.addKeyListener(gPanel._keyListener);    // Had to do this manually; frame didnt' exist yet.....
    Dimension d = new Dimension(1600,400);
    gPanel.setPreferredSize(d);
    f.setSize(600,300);
//    Dimension screenSize =  Toolkit.getDefaultToolkit().getScreenSize();
//    f.setSize((int)screenSize.getWidth()-150, (int)screenSize.getHeight()-150);
    f.setLocation(0,0);
//    f.pack();
    WindowListener wndCloser = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    };
    f.addWindowListener(wndCloser);


    f.setVisible(true);
    f.show();
  }
}
