/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: GanttSlackTask.java,v 1.3.2.1 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 21May2001 DFK - Bug #0000099: Initial Entry
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;

import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.protocol.JobInfo;

import org.apache.log4j.Category;

/** A composite that contains a GanttTask obj for the box, and one for the slack tail.
 *
 */
public class GanttSlackTask extends GenericGanttObject {
  /** The {@link SubtaskRequest} that this obj represents */
  protected SubtaskRequest req = null;
  /** Our link to the parent {@link GanttPanel} instance. */
  protected GanttPanel ganttPanel = null;
  /** Our instance of the "box" portion of a task, from EarlyStart to JobInfo.ExpectedDuration. */
  protected GanttTask taskBox = null;
  /** Our instance of the "tail" portion of a task, from (EStart+JobInfo.ExpDuration) to Deadline. */
  protected GanttTask taskTail = null;

    /** log category */
  private static Category log =
                    Category.getInstance(GanttSlackTask.class.getName());

  /** Constructor */
  public GanttSlackTask(SubtaskRequest req, GanttPanel ganttPanel) {
    super();    // what does this accomplish??   Nothing, at this point...
    this.req = req;
    if (null==req) { GuiUtil.inform(ganttPanel, "Encountered null SubtaskRequest");  return; }
    this.ganttPanel = ganttPanel;
    if (null==ganttPanel) { GuiUtil.inform(ganttPanel, "Encountered null GanttPanel");  return; }

    /**
     * Set up the "box" and "tail" portions of the task.
     */
    taskBox = new GanttTask(req, ganttPanel, SHAPE_LINE_WIDE);
    taskTail = new GanttTask(req, ganttPanel, SHAPE_TAIL_WIDE);
    // for now, Just use the shape/color defaults..
  }

  /** Overridden paint method to paint the component objs. */
  public void paint(Graphics2D g2) {
    if (null==g2)  {  return;  }
    /** Set the child colors/shapes from this instance.
     *  This couldn't have been done prior since colors/shapes are overriden
     *  after this class's constructor is called..
     */
    if (null!=taskTail) {
      log.debug("GanttSlackTask.paint(): Drawing tail at rect="
            + taskTail.boundingBox.toString()+" for Task "+req.getId());
      taskTail.startColor = this.startColor;
      taskTail.startShape = this.startShape;
      taskTail.middleColor = this.middleColor;
      taskTail.endColor = this.endColor;
      taskTail.endShape = SHAPE_ENDTAIL_WIDE;
      taskTail.paint(g2);
    }
    if (null!=taskBox) {
      log.debug("GanttSlackTask.paint(): Drawing box at rect="
            + taskTail.boundingBox.toString()+" for Task "+req.getId());
      taskBox.startColor = this.startColor;
      taskBox.startShape = this.startShape;
      taskBox.middleColor = this.middleColor;
//      taskBox.middleShape = this.middleShape;   // this is already set above..
      taskBox.endColor = this.endColor;
      taskBox.endShape = this.endShape;
      taskBox.paint(g2);
    }
  }
}
