/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: GanttSlackTaskBid.java,v 1.3.2.1 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 21May2001 DFK - Bug #0000099: Initial Entry
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;

import edu.umn.magnet.client.eval.TaskBid;
import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.protocol.JobInfo;

import org.apache.log4j.Category;

/**
 * A parent class that handles draws/repaints of bid Gantt objs with slack tails.
 */
public class GanttSlackTaskBid extends GenericGanttObject {
  /** The {@link TaskWithPrecedence} that this obj maps to */
  protected TaskWithPrecedence twp = null;
  /** The {@link Bid} that this obj maps to */
  protected Bid bid = null;
  /** A {@link TaskBid} that maps this task to its corresponding bid. */
  private TaskBid taskBid = null;
  /** Our link to the parent {@link GanttPanel} instance. */
  protected GanttPanel ganttPanel = null;
  /** Our instance of the "box" portion of a task, from EarlyStart to JobInfo.ExpectedDuration. */
  protected GanttTaskBid taskBidBox = null;
  /** Our instance of the "tail" portion of a task, from (EStart+JobInfo.ExpDuration) to Deadline. */
  protected GanttTaskBid taskBidTail = null;

  /** log category */
  private static Category log =
                    Category.getInstance(GanttSlackTaskBid.class.getName());
  /** Constructor */
  public GanttSlackTaskBid(TaskWithPrecedence twp, Bid bid, GanttPanel ganttPanel) {
    super();    // what does this accomplish??   Nothing, at this point...
    this.twp = twp;
    if (null==twp) { GuiUtil.inform(ganttPanel, "Encountered null TaskWithPrecedence");  return; }
    this.bid = bid;
    if (null==bid) { GuiUtil.inform(ganttPanel, "Encountered null Bid");  return; }
    this.ganttPanel = ganttPanel;
    if (null==ganttPanel) { GuiUtil.inform(ganttPanel, "Encountered null GanttPanel");  return; }

    // Create the TaskBid instance.
    taskBid = new TaskBid(twp, bid);
    if (null==taskBid) { GuiUtil.inform(ganttPanel, "Encountered null TaskBid");  return; }


    /**
     * Set up the "box" and "tail" portions of the task.
     */
    taskBidBox  = new GanttTaskBid(taskBid, ganttPanel, SHAPE_LINE_THIN);
    taskBidTail = new GanttTaskBid(taskBid, ganttPanel, SHAPE_TAIL_THIN);

    // Apply default bid coloring
    Color c = new Color(0,127,0); // burnt green
    this.startColor = c;
    this.middleColor = c;
    this.endColor = c;
  }

  /** Overridden paint method to paint the component objs. */
  public void paint(Graphics2D g2) {
    if (null==g2)  {  return;  }
    /** Set the child colors/shapes from this instance.
     *  This couldn't have been done prior since colors/shapes are overriden
     *  after this class's constructor is called..
     */
    if (null!=taskBidTail) {
      log.debug("GanttSlackTaskBid.paint(): Drawing tail at rect="
            + taskBidTail.boundingBox.toString()+" for Task "+twp.getTaskData().getId());
      taskBidTail.startColor = this.startColor;
      taskBidTail.startShape = this.startShape;
      taskBidTail.middleColor = this.middleColor;
      taskBidTail.endColor = this.endColor;
      taskBidTail.endShape = SHAPE_ENDTAIL_THIN;
      taskBidTail.paint(g2);
    }
    if (null!=taskBidBox) {
      log.debug("GanttSlackTaskBid.paint(): Drawing box at rect="
            + taskBidBox.boundingBox.toString()+" for Task "+twp.getTaskData().getId());
      taskBidBox.startColor = this.startColor;
      taskBidBox.startShape = this.startShape;
      taskBidBox.middleColor = this.middleColor;
//      taskBidBox.middleShape = this.middleShape;   // this is already set above..
      taskBidBox.endColor = this.endColor;
      taskBidBox.endShape = this.endShape;
      taskBidBox.paint(g2);
    }
  }
}
