/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: GanttTaskBid.java,v 1.4.2.1 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 13May2001 DFK - Bug #0000053: Initial Submission, based on GanttTask.java v1.3
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;

import edu.umn.magnet.client.eval.TaskBid;
import edu.umn.magnet.protocol.cust.*;

import org.apache.log4j.Category;

/** A class that represents a TaskBid graphically. */
public class GanttTaskBid extends GenericGanttObject {
  /** The {@link Bid} that this obj represents */
  protected TaskBid taskBid = null;
  /** Our link to the parent {@link GanttPanel} instance. */
  protected GanttPanel ganttPanel = null;

    /** log category */
  private static Category log =
                    Category.getInstance(GanttTaskBid.class.getName());
  /** Constructor */
  public GanttTaskBid(TaskBid taskBid, GanttPanel ganttPanel, int ganttShape) {
    super();
//    // Make overlapped bids be thin different-colored lines
//    this.middleShape = SHAPE_LINE_THIN;
//    this.middleColor = new Color(0,128,0);

    this.taskBid = taskBid;
    if (null==taskBid) { GuiUtil.inform(ganttPanel, "Encountered null TaskBid");  return; }
    this.ganttPanel = ganttPanel;
    if (null==ganttPanel) { GuiUtil.inform(ganttPanel, "Encountered null GanttPanel");  return; }
    SubtaskRequest req = taskBid.getTask();
    if (null==req) { GuiUtil.inform(ganttPanel, "Encountered null SubtaskRequest");  return; }
    int x1=0, x2=0;
    log.debug("GanttTaskBid: taskID="+req.getId()+": about to calc x1 (millis="+req.getEarliestTimeToStart().getTime()
                            +") and x2 (millis="+req.getDeadline().getTime() +")");

    if (0>ganttShape  ||  SHAPE_MAX<=ganttShape)  {  return;  }
    this.middleShape = ganttShape;
    switch (ganttShape) {
      case SHAPE_TAIL_WIDE:
      case SHAPE_TAIL_THIN:
          x1 = ganttPanel.t2x(new Date(taskBid.getEarlyStart().getTime() +
                                       taskBid.getExpectedDuration()));
          x2 = ganttPanel.t2x(taskBid.getLateFinish());
          this.endShape = ganttShape;
          break;
      case SHAPE_LINE_WIDE:
      case SHAPE_LINE_THIN:
      default:
          x1 = ganttPanel.t2x(taskBid.getEarlyStart());
          x2 = ganttPanel.t2x(new Date(taskBid.getEarlyStart().getTime() +
                                       taskBid.getExpectedDuration()));
          break;
    }//switch
//    int x1 = ganttPanel.t2x(req.getEarliestTimeToStart());
//    int x2 = ganttPanel.t2x(req.getDeadline());
    log.debug("GanttTaskBid: after t2x: coords x1="+x1+", and x2="+x2);
    if  (x2-x1<3)  x2+=3;  // REMOVE THIS ; it's just here to ensure SOME width....
    int y = 50;
    int h = 16;
    // Override y and height if they're available (from table model).
    if (null!=ganttPanel) {
      TPTableModel tm = ganttPanel.getTPTableModel();
      String taskID = req.getId();
      Rectangle r = tm.getRowRect(taskID);
      if (null!=r) {
        y = r.y;
          y+=28;  // REMOVE THIS
        h = r.height;
          if (0==h) h=16;  // REMOVE THIS
        log.debug("GanttTaskBid: Task="+taskID
            + ", earliestDate="+DateFormat.getDateTimeInstance().format(req.getEarliestTimeToStart())
            + ", latestDate="+DateFormat.getDateTimeInstance().format(req.getDeadline())
            + ", x="+x1+",y="+y+",w="+(x2-x1)+",h="+h);
      }
    }
    this.boundingBox = new Rectangle(x1, y, (x2-x1), h);
    // for now, Just use the shape/color defaults..
  }
}
