/**
 * ------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: GuiUtil.java,v 1.11 2001/10/17 03:41:08 jcollins Exp $
 * Copyright (c) 2000,2001 University of Minnesota
 * ------------------------------------------------------------------------
 * Revision History:
 *
 * 11Apr2001 DFK - added getFrame()
 * 23Apr2001 DFK - Bug #0000009: cleanup some javadocs
 * 02May2001 DFK - Added inform()
 * 05May2001 PBG - Added another inform()
 * 06May2001 PBG - Moved getStatuses to here
 * 13May2001 PBG - Bug #0000090 Added a function to find agent time and return
 *                  a string
 */

package edu.umn.magnet.client.userif;

import edu.umn.magnet.client.status.*;  //for status objects
import edu.umn.magnet.protocol.cust.*;  // for TaskPlan
import edu.umn.magnet.util.time.AgentTime;

import org.apache.log4j.Category;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;                     // for iterator
import java.text.SimpleDateFormat;
import java.text.ParseException;

public class GuiUtil
{
  private static Category log =
    Category.getInstance(GuiUtil.class.getName());

  // Some handy constants..

  /** Specifies a margin inset of 5px. */
  public static final int LAYOUT_MARGIN   = 5;
  public static final int LAYOUT_EXTRAMARGIN = 10;
  public static final int LAYOUT_INTRA    = 5;
  public static final int LAYOUT_NOMARGIN = 0;

  public static final int ONE_SECOND=1000;

  // -----------------------------------------------------
  // Some all-purpose fns that belong in a "Util" class..

  /** Apply the specified constraints to the provided {@link java.awt.GridBagConstraints} object. */
  static public void constrain(Container container, Component component,
                           int grid_x, int grid_y,
                           int grid_width, int grid_height,
                           double weight_x, double weight_y)
  {
    constrain(container,component,grid_x,grid_y,grid_width,grid_height,
        GridBagConstraints.NONE,GridBagConstraints.WEST,weight_x,
        weight_y,0,5,0,0);
  }

  /** Apply the specified constraints to the provided {@link java.awt.GridBagConstraints} object. */
  static public void constrain(Container container, Component component,
                         int grid_x, int grid_y,
                         int grid_width, int grid_height, int fill,
                         int anchor, double weight_x, double weight_y,
                         int top, int left, int bottom, int right)
  {
    constrain(container,component,grid_x,grid_y,grid_width,grid_height,
        fill,anchor,weight_x,weight_y,
        new Insets(top, left, bottom, right));
  }

  /** Apply the specified constraints to the provided {@link java.awt.GridBagConstraints} object. */
  static public void constrain(Container container, Component component,
                         int grid_x, int grid_y,
                         int grid_width, int grid_height, int fill,
                         int anchor, double weight_x, double weight_y,
                         Insets insets)
  {
    GridBagConstraints cst = new GridBagConstraints();
    cst.gridx = grid_x;
    cst.gridy = grid_y;
    cst.gridwidth = grid_width;
    cst.gridheight = grid_height;
    cst.fill = fill;
    cst.anchor = anchor;
    cst.weightx = weight_x;
    cst.weighty = weight_y;
    cst.insets = insets;
    ((GridBagLayout)container.getLayout()).setConstraints(component, cst);
    container.add(component);
  }

    /**
    *   Return the frame that contains the specified component
    *   @param component
    */
    static public Frame getFrame(Component component)
    {
        Component child = component;
        Component parent = child;       // to handle case where component IS the frame..
        while(true)
        {
//            parent = child.getParent();
            if (parent == null || parent instanceof Frame)
                return (Frame)parent;
//            child = parent;
            parent = parent.getParent();
        }
    }

    /**
    *   Return the {@link Window} that contains the specified component
    *   @param component
    */
    static public Window getWindow(Component component)
    {
        Component child = component;
        Component parent = child;       // to handle case where component IS the Window..
        while(true)
        {
//            parent = child.getParent();
            if (parent == null || parent instanceof Window)
                return (Window)parent;
//            child = parent;
            parent = parent.getParent();
        }
    }

  /** A quick temporary method to pop up the string in an info dialog.. */
  public static void inform(Component parentComponent, String s) {
    int result = JOptionPane.showConfirmDialog(parentComponent,
                        new JLabel(s),
                        "Info..", JOptionPane.CANCEL_OPTION,
                        JOptionPane.INFORMATION_MESSAGE);
  }

  /** A quick temporary method to pop up the string in an info dialog.. */
  public static void inform(Component parentComponent, String s, String title) {
    int result = JOptionPane.showConfirmDialog(parentComponent,
                        s,
                        title, JOptionPane.DEFAULT_OPTION,
                        JOptionPane.PLAIN_MESSAGE);
  }

   /** getRFQStatus(agentStatus, TaskPlan)
   *  This method will return the appropriate RFQStatus to be used - based upon
   *  the AgentStatus object and the current plan/session
   *
   * NOTE:  Currently this assumes that there is only one RFQStatus per session.
   * Looking at the description as to what a session is and what an RFQ is - I
   * have a hard time believing that we want more than one per session.  So, this
   * will just return the first one that it finds.
   */
  public static RFQStatus getRFQStatus(AgentStatus as, TaskPlan tp ) {
    SessionStatus ss = null;
    java.util.Collection rfqSet = null;

    ss = getSessionStatus(as, tp);
    if (ss == null) return null;
    log.debug("Got matching Session Status Object");

    rfqSet = ss.getRFQs();
    if (rfqSet.isEmpty()) return null;
    log.debug("Session Status contains RFQs");
    for (Iterator rsi = rfqSet.iterator(); rsi.hasNext();) {
      RFQStatus rfqs = (RFQStatus)(rsi.next());
      log.debug("RFQ found and returned");
      return rfqs;
    }

    /** if we got here - no data */
    return null;
  }



  /** getSessionStatus(agentStatus, TaskPlan)
   *  This method will return the appropriate SessionStatus to be used - based
   *  upon the AgentStatus object and the current plan/session
   *
   * NOTE:  Currently, this assumes that there is only *ONE* session with a
   * plan and will only return the first SessionStatus Object.  This is due to
   * the fact that there was no clear and easy way to identify which session
   * status object is related to this session.  Once that is in place, we need
   * to address that.
   */
  public static SessionStatus getSessionStatus(AgentStatus as, TaskPlan tp ) {
    PlanStatus ps = null;
    java.util.Collection sessionStatusSet = null;

    ps = getPlanStatus(as, tp);

    if (ps == null) return null;
    sessionStatusSet = ps.getSessions();
    if (sessionStatusSet.isEmpty()) return null;
    log.debug("found sessionStatus Objects");

    for (Iterator ssi = sessionStatusSet.iterator(); ssi.hasNext();) {
          SessionStatus ss = (SessionStatus)(ssi.next());
          log.debug("found matching ss object");
          return ss;
    } // end of for loop

    /** If we got here - nothing was found */
    return null;
  } // end of getSessionStatus


  /** getPlanStatus(agentStatus, TaskPlan)
   *  This method will return the appropriate PlanStatus to be used - based
   *  upon the AgentStatus object and the current task plan
   *
   */
  public static PlanStatus getPlanStatus(AgentStatus as, TaskPlan tp ) {
    java.util.Collection planStatusSet = null;

    if (as == null) return null;

    planStatusSet = as.getPlans();
    if (planStatusSet.isEmpty()) return null;

    log.debug("Found a plan Status object");
    for (Iterator psi = planStatusSet.iterator(); psi.hasNext();) {
      PlanStatus ps = (PlanStatus)(psi.next());
      if (ps.getTaskPlan() == tp) {
        log.debug("found matching plan object");
        return ps;
      }
    } // end of for loop
    return null;
  } // end of getPlanStatus

  /** getAgentTimeString will find the current agent time and then
   *  transpose it into a string of the format
   */
  public static String getAgentTimeString()
  {
    SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss");
    Date agentDate = AgentTime.getDate();
    return agentDate.toString();
    //return formatter.format(agentDate);

  } // end of getAgentTimeString

  /**
   * Returns an interval in milliseconds, given a string of the form
   * "D H:M:S".  You can leave off the D, the H: or the H:M: for convenience.
   * @exception ParseException thrown if no useful parse of your input
   *   string can be accomplished.
   */
  public static long string2interval (String input)
    throws ParseException
  {
    // this offset stuff is used to get rid of time zone offset.  I
    // don't see any other easy way to do this.
    Date offset = new Date(0L);
    try {
      offset = new SimpleDateFormat("s").parse("0");
    }
    catch (ParseException pe) {
      log.error("Can't parse \"0\" into a Date");
    }

    Date result = offset;
    try {
      result = new SimpleDateFormat("d H:m:s").parse(input);
    }
    catch (ParseException pe1) {
      try {
	result = new SimpleDateFormat("H:m:s").parse(input);
      }
      catch (ParseException pe2) {
	try {
	  result = new SimpleDateFormat("m:s").parse(input);
	}
	catch (ParseException pe3) {
	  result = new SimpleDateFormat("s").parse(input);
	}
      }
    }
    return result.getTime() - offset.getTime();
  }

  /**
   * Returns a String that represents an interval in milliseconds, in
   * the form "D H:M:S".  The Days part is left off if the interval is
   * less than 24 hr.
   */
  public static final long DAY = 24*60*60*1000;
  public static String interval2String (long interval)
  {
    // this offset stuff is used to get rid of time zone offset.  I
    // don't see any other easy way to do this.
    Date offset = new Date(0L);
    try {
      offset = new SimpleDateFormat("s").parse("0");
    }
    catch (ParseException pe) {
      log.error("Can't parse \"0\" into a Date");
    }
    log.debug("Offset = " + offset
	      + ", Offset (long) = " + offset.getTime());
    Date value = new Date(interval + offset.getTime());
    if (interval >= DAY)
      return new SimpleDateFormat("d HH:mm:ss").format(value);
    else
      return new SimpleDateFormat("HH:mm:ss").format(value);
  }
}
