/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: MagnetClientConstants.java,v 1.7.2.1 2002/05/05 19:42:01 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 26Apr2001 DFK - Initial Entry
 * 27Apr2001 DFK - Added enums for more TP gui fields
 * 30Apr2001 DFK - Bug #0000032: TaskPlan editor support.
 * 30Apr2001 DFK - Added enums for TP SubmitRFQ gui fields.
 * 02May2001 DFK - Bug #0000022: Added enum for TP column header height
 * 13May2001 DFK - Bug #0000053: Added enums for TPTable highlighting styles
 */

package edu.umn.magnet.client.userif;

/** An interface containing Magnet Client GUI constants.
 *  Includes enums for TaskPlanEditor fields, for error focus capability.
 */
public interface MagnetClientConstants {
  /** A general error not associated w/ any particular GUI field. */
  public static final int ERRFOCUS_GENERAL            = 0;

  //
  // TaskPlan Editor fields
  //

  /** The TaskID textfield */
  public static final int ERRFOCUS_TP_TASK_ID         = 10;
  /** The TaskName textfield */
  public static final int ERRFOCUS_TP_TASK_NAME       = 11;
  /** The Dependencies textfield */
  public static final int ERRFOCUS_TP_DEPENDENCIES    = 12;
  /** The EarliestStart textfield */
  public static final int ERRFOCUS_TP_TASK_ESTART     = 13;
  /** The Deadline textfield */
  public static final int ERRFOCUS_TP_TASK_DEADLINE   = 14;
  /** The JobInfo combobox */
  public static final int ERRFOCUS_TP_TASK_JOBINFO    = 15;

  /** The TaskPlan details Plan Title textfield */
  public static final int ERRFOCUS_TP_DETAILS_TITLE   = 30;
  /** The TaskPlan details Plan Start Date textfield */
  public static final int ERRFOCUS_TP_DETAILS_ESTART  = 31;
  /** The TaskPlan details Plan Deadline textfield */
  public static final int ERRFOCUS_TP_DETAILS_DEADLINE= 32;

  /** The RFQ Earliest Bid Consider textfield */
  public static final int ERRFOCUS_TP_RFQ_EBID_CONSIDER= 50;
  /** The RFQ Earliest Bid Offer textfield */
  public static final int ERRFOCUS_TP_RFQ_EBID_OFFER   = 51;
  /** The RFQ Bid Deadline textfield */
  public static final int ERRFOCUS_TP_RFQ_BID_DEADLINE = 52;

  //
  // Bid Viewer fields
  //

  /** Some Bid screen's foobar textfield */
  public static final int ERRFOCUS_BID_FOOBAR         = 1000;


  // etc..

  //
  // Misc useful constants
  //
  /** The height of the TaskPlan table's column header, set to sync with GanttPanel. */
  public static final int TP_TABLE_COLUMN_HEADER_HEIGHT     = 32;

  /** An enum indicating a highlight style, as used in TPTableModel. */
  public static final int H_TYPE0 = 0;
  public static final int H_TYPE1 = 1;
  public static final int H_TYPE2 = 2;
  public static final int H_TYPE3 = 3;
  public static final int H_TYPE4 = 4;
}