/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: TaskPlanDetailsEditor.java,v 1.6.2.1 2002/05/05 19:42:02 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 30Apr2001 DFK - Bug #0000032: Initial Entry, for TaskPlan editing support.
 * 30Apr2001 DFK - Comment out annoying interim button handler informs.
 * 15May2001 DFK - Bug #0000079: Convert Time/Date specs to mm/dd/yy hh:mm.
 * 20May2001 PBG - Bug #0000050, #0000098: Know when to save the plan
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
//import java.awt.geom.*;
//import java.awt.font.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.apache.log4j.Category;

import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.protocol.JobInfo;
//import edu.umn.magnet.client.plan.JobInfoFactory;  // for testing
//import edu.umn.magnet.util.*;  // for testing
import edu.umn.magnet.util.time.AgentTime;   // for example date strings

/** Class that launches an editor for editing global details of a TaskPlan.
 *  Users can edit the Plan Name.
 *  They can also adjust the Plan Start Date or Plan Deadline, to shift the
 *  entire plan to start or end by a particular date.
 */
public class TaskPlanDetailsEditor
              implements ActionListener, MagnetClientConstants,
                          ErrorObjectHandler.ErrorFocus
{
  private JWrappingLabel _lblDescription = new JWrappingLabel();
  private JLabel _lblPlanTitle = new JLabel();
  private JTextField _tfPlanTitle = new JTextField();
//  private JLabel _lblPlanStartDate = new JLabel();
  private JTextField _tfPlanStartDate = new JTextField();
//  private JLabel _lblPlanDeadline = new JLabel();
  private JTextField _tfPlanDeadline = new JTextField();
  private JButton _btnOK = new JButton();
  private JButton _btnCancel = new JButton();
  private JButton _btnHelp = new JButton();
  private JRadioButton _rbPlanStartDate = new JRadioButton();
  private JRadioButton _rbPlanDeadline  = new JRadioButton();
  private ButtonGroup  _bgrpDates = new ButtonGroup();

  private JPanel pMain = null;

  /** Our handle to a TaskPlan object. */
  private TaskPlan _tp = null;
  /** Our handle to the parent Frame. */
  private Frame _parent = null;
  /** Our JDialog instance. */
  private JDialog _dialog = null;
  /** Member for Plan Start date. */
  private Date planStartDate = null;
  /** Member for Plan Deadline date. */
  private Date planDeadline = null;
  /** member for setting that you need to save a plan */
  private MagnetClientMain _mcmParent = null;

  /** Constructor */
  public TaskPlanDetailsEditor(Frame f, TaskPlan tp, MagnetClientMain mcm) {
    _parent = f;
    _tp = tp;
    _mcmParent = mcm;
    doSetup();
  }

  /** Do initialization stuff, like loading static strings into components. */
  private void doSetup() {
    /** @todo These static strings ought to come from a ResourceBundle instead.. */
    _lblDescription = new JWrappingLabel(
                            "You may edit the Plan Title, or adjust " +
                            "all Task times such that the Plan starts " +
                            "or ends by a desired date/time.");
    _lblPlanTitle.setText("Plan Title:");
    _rbPlanStartDate.setText("Plan Start Date:");
    _rbPlanDeadline.setText("Plan Deadline:");
    _btnOK.setText("OK");
    _btnCancel.setText("Cancel");
    _btnHelp.setText("Help");

    _btnOK.addActionListener(this);
    _btnCancel.addActionListener(this);
    _btnHelp.addActionListener(this);

    _bgrpDates.add(_rbPlanStartDate);
    _bgrpDates.add(_rbPlanDeadline);

    // Always start with StartDate radiobutton selected.
    _rbPlanStartDate.setSelected(true);
    _rbPlanDeadline.setSelected(false);
    _rbPlanStartDate.addActionListener(this);
    _rbPlanDeadline.addActionListener(this);
    doEnableDisable();
  }

  /** Create the editor. */
  public void createEditor() {
    if (null==_tp) { return; }
    pMain = new JPanel(new GridBagLayout());
    _lblDescription.setPreferredSize(new Dimension(250,60));
    GuiUtil.constrain(pMain, _lblDescription,
                      0, 0,
                      2,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );
    GuiUtil.constrain(pMain, _lblPlanTitle,
                      0, 1,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );
    GuiUtil.constrain(pMain, _tfPlanTitle,
                      1, 1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );
    GuiUtil.constrain(pMain, _rbPlanStartDate,
                      0, 2,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );
    GuiUtil.constrain(pMain, _tfPlanStartDate,
                      1, 2,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );
    GuiUtil.constrain(pMain, _rbPlanDeadline,
                      0, 3,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );
    GuiUtil.constrain(pMain, _tfPlanDeadline,
                      1, 3,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );

    JPanel pButtons = new JPanel(new FlowLayout());
    pButtons.add(_btnOK);
    pButtons.add(_btnCancel);
    pButtons.add(_btnHelp);
    GuiUtil.constrain(pMain, pButtons,
                      0, 4,
                      2,1,
                      GridBagConstraints.NONE, GridBagConstraints.EAST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,//GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA
                      );
  }

  /** Display the editor. */
  public void displayEditor() {
    // Load the dynamic fields
    _tfPlanTitle.setText(_tp.getPlanName());
    _tfPlanStartDate.setText(format(_tp.getPlanStartDate()));
    _tfPlanDeadline.setText(format(_tp.getPlanDeadline()));

    // Display the panel in a nonmodal JDialog.
    _dialog = new JDialog(_parent, "Change TaskPlan details", false);
    createEditor();
    _dialog.getContentPane().add(pMain);
    _dialog.pack();
    _dialog.show();
  }

  /** Format a Date obj into our favorite String convention. */
  private String format(Date date) {
    DateFormat df1 = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
    df1.setLenient(true);
    return df1.format(date);
  }

  /** Button listener code */
  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand() == _btnOK.getText()) {
//      inform("TPDE.actionPerformed(): 'OK' button pressed");
      if (validateInfo()) {
        _tp.setPlanName(_tfPlanTitle.getText());
        /** @todo Should only do whatever's indicated by radiobuttons */
        if (_rbPlanStartDate.isSelected()) {
          _tp.setPlanStartDate(planStartDate);
        }
        if (_rbPlanDeadline.isSelected()) {
          _tp.setPlanDeadline(planDeadline);
        }
        _mcmParent.setNeedSave();    // indicate it needs saving
        _dialog.dispose();  // only on success
      }

    } else if (e.getActionCommand() == _btnCancel.getText()) {
//      inform("TPDE.actionPerformed(): 'Cancel' button pressed");
      _dialog.dispose();
    } else if (e.getActionCommand() == _btnHelp.getText()) {
      inform("TPDE.actionPerformed(): 'Help' button pressed");
    } else if (e.getActionCommand() == _rbPlanStartDate.getText()) {
//      inform("TPDE.actionPerformed(): 'Plan Start Date' radiobutton pressed");
      doEnableDisable();
    } else if (e.getActionCommand() == _rbPlanDeadline.getText()) {
//      inform("TPDE.actionPerformed(): 'Plan Deadline' radiobutton pressed");
      doEnableDisable();
    } else {
      inform("TPDE.actionPerformed(): unknown action "
              +e.getActionCommand()+" from unknown source "+e.getSource());
    }
  }

  /** A quick temporary method to pop up the string in an info dialog.. */
  public void inform(String s) {
    int result = JOptionPane.showConfirmDialog(_dialog,
                        new JLabel(s),
                        "Info..", JOptionPane.CANCEL_OPTION,
                        JOptionPane.INFORMATION_MESSAGE);
  }

  /** Enable/Disable the radiobuttons and associated fields. */
  public void doEnableDisable() {
    if (_rbPlanStartDate.isSelected()) {
      _tfPlanStartDate.setEnabled(true);
      _tfPlanDeadline.setEnabled(false);
    } else if (_rbPlanDeadline.isSelected()) {
      _tfPlanStartDate.setEnabled(false);
      _tfPlanDeadline.setEnabled(true);
    }
  }

  /** Validate the input fields */
  public boolean validateInfo() {
    Vector errors = new Vector();
    // Plan Title must be non-empty
    if (0==_tfPlanTitle.getText().length()) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_DETAILS_TITLE, "The Plan Title cannot be empty."));
    }
    DateFormat df1 = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
    df1.setLenient(true);
    String exampleDateString = df1.format(AgentTime.getDate());  // example string
    String formatBadDateString = "{0}: Bad format.  Please format like this: {1}";

    if (_rbPlanStartDate.isSelected()) {
      // The Plan Start Date must parse tolerably as a date.
      try {
        DateFormat df2 = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
        df2.setLenient(true);
        planStartDate = new Date( df2.parse(_tfPlanStartDate.getText()).getTime());
      } catch (ParseException pe1) {
        String badEStartString = MessageFormat.format( formatBadDateString,
                                  new Object[]{ _rbPlanStartDate.getText(), exampleDateString }  );
        errors.addElement(new ErrorObject(ERRFOCUS_TP_DETAILS_ESTART, badEStartString));
      }
    }

    if (_rbPlanDeadline.isSelected()) {
      // The Plan Deadline must parse tolerably as a date.
      try {
        DateFormat df3 = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
        df3.setLenient(true);
        planDeadline = new Date( df3.parse(_tfPlanDeadline.getText()).getTime());
      } catch (ParseException pe2) {
        String badDeadlineString = MessageFormat.format( formatBadDateString,
                                  new Object[]{ _rbPlanDeadline.getText(), exampleDateString }  );
        errors.addElement(new ErrorObject(ERRFOCUS_TP_DETAILS_DEADLINE, badDeadlineString));
      }
    }

    /* At the end, if there are no errors, return true;
     *  else display errors in focusing Dialog and return false.
     */
    return handleErrors(errors);
  }

  /** If no errors, return true;  If errors, launch errors dialog, then return false. */
  public boolean handleErrors(Vector errors) {
    if (0==errors.size()) {
      return true;
    } else {
      // Display the errors (if any)
        ErrorObjectHandler eH = new ErrorObjectHandler(errors);
        eH.parkUpperRHCorner();  // reposition error window.
        eH.displayErrors(_parent, this, null, null);
        return false;
    }
  }

  /**
   * Handles the errors by determining what to do for each error
   * @param errorObject the error to resolve
   */
  public void setErrorFocus(ErrorObject errorObject) {
    if (null==errorObject) {  return;  }
    switch (errorObject.getField()) {
      case ERRFOCUS_TP_DETAILS_TITLE:
          _tfPlanTitle.requestFocus();
          break;

      case ERRFOCUS_TP_DETAILS_ESTART:
          _tfPlanStartDate.requestFocus();
          break;

      case ERRFOCUS_TP_DETAILS_DEADLINE:
          _tfPlanDeadline.requestFocus();
          break;

      //etc..

      default:
          // DEFAULT MESSAGE IF ERROR NOT HANDLED
          inform("The error you selected is not currently being handled.");
    }//switch
    return;
  }


}
