/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: TimeframePanel.java,v 1.11.2.1 2002/05/05 19:42:03 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 11Apr2001 DFK - Initial Entry
 * 15Apr2001 DFK - Turned off some superVerbose println's, until I get log4j added..
 * 02May2001 DFK - Bug #0000022: Correctly recalc timeframes,etc upon plan change;
 *                  Timeframe bkgd height now resizes to keep pace w/ TPTable;
 *                  Overrode scrollable to move in 10px increments (faster, more "normal".)
 * 02May2001 DFK - Bug #0000022: Hooked in zoom functionality that updates the panel.
 * 07May2001 DFK - Bug #0000022: Reworked a more efficient t2x mechanism;
 *                  Changes to fix layout discrepancies;  Start cleaning up ugly fonting.
 * 26May2001 DFK - Bug #0000127: Fix redraw problem.
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.awt.image.BufferedImage;

import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.util.time.AgentTime;   // for example date strings

import org.apache.log4j.Category;

// The majority of the java2d stuff was inspired by a Sun online tutorial "Quad.java",
// at  http://java.sun.com/docs/books/tutorial/2d/display/strokeandfill.html
/**
 * Panel that draws a Gantt chart based on the provided {@link TPGanttModel}.
 */
public class TimeframePanel extends JComponent  //JPanel
                        implements GanttListener,
                                    Scrollable
{
  /** Holds all the zoom level info. */
  private ZoomInfo[] _zLvls = null;
  /** Current zoom level index (default 1). */
  private int _zLvl = 1;
  /** Our copy of the current {@link TPGanttModel} we're listening to. */
  private TPGanttModel _gm = null;
  /** A ref to the current {@link TPTableModel} we're listening to. */
  private TPTableModel _tm = null;
  /** Earliest date we need to display */
  private Date _earliestDate = null;
  /** Latest date we need to display */
  private Date _latestDate = null;
//  /** The {@link JPanel} containing the timeframe. */
//  private JPanel _timeframePanel = new JPanel();
  /** Our handle to a pre-drawn background image.
   *  Be sure to null it out (to force redraws) when the zoom level changes.
   */
  private BufferedImage _bkgdImage = null;


  /** A keylistener that gets attached to the enclosing frame.
   *  This has to be done manually after it's added to a frame. :)
   */
  public KeyAdapter _keyListener = null;
  /** Our thread for async keypress notifications */
  protected Thread _runner;

  /** For t2x() conversion, a precomputed array mapping x to t.. */
  private long[] x2tArray = null;

  /** log category */
  private static Category log =
                    Category.getInstance(TimeframePanel.class.getName());
/**
  This is the timeline ruler that sits above the Gantt bars.
 */
//public class Timeline extends JPanel {
//
//  protected Color m_lightShadow;
//  protected Color m_darkShadow;
//  protected int m_longMark = 14;
//  protected int m_shortMark = 8;
//  protected int m_height = m_longMark + 2;
//  protected int m_width = 0;
//
//  protected double m_ppd = 100.0;    // pixels/division
//  protected double m_subdivisions = 5; // subdivision/division
//
//  /**
//    Creates a new Timeline, with default font and color.
//   */
//  public /*Timeline*/GanttPanel() {
//    super();
//    setFont(new Font("Dialog", Font.PLAIN, 12));
//    setBackground(UIManager.getColor("control"));
//  }
//
//  /**
//    The setBackground method is overridden to allow computation of the
//    tick-mark colors.
//   */
//  public void setBackground (Color color) {
//    super.setBackground(color);
//    m_lightShadow = null;
//  }
//
//  /**
//     Makes the preferred height be 2 more than the length of a long mark.
//  */
//  public Dimension getPreferredSize () {
//    return new Dimension(m_width, m_height);
//  }
//
//  public void setWidth (int width) {
//    m_width = width;
//    setSize(width, m_height);
//  }
//
//  /**
//    Paints this thing.
//   */
//  public void paintComponent(Graphics g) {
//    //setSize(m_width, m_height);
//    Dimension d = getSize();
//    Rectangle r = g.getClipBounds();
//    Color light = getLightShadow();
//    Color dark = getDarkShadow();
//
//    double markX =
//	(double)(Math.floor(((double)r.x)/m_ppd)) * m_ppd;
//    double startX = (double)(r.x);
//    double endX = (double)(r.x + r.width);
//    while (markX < endX) {
//      // draw a big mark and the necessary number of little ones
//      int x = (int)Math.round(markX);
//      if (x >= r.x) {
//	g.setColor(light);
//	g.drawLine(x + 1, m_height, x + 1, m_height - m_longMark);
//	g.setColor(dark);
//	g.drawLine(x, m_height, x, m_height - m_longMark);
//      }
//      for (int i = 0; i < m_subdivisions; i++) {
//	x = (int)Math.round(markX + ((double)(i) * m_ppd/m_subdivisions));
//	if (x >= r.x && x <= (r.x + r.width)) {
//	  g.setColor(light);
//	  g.drawLine(x + 1, m_height, x + 1, m_height - m_shortMark);
//	  g.setColor(dark);
//	  g.drawLine(x, m_height, x, m_height - m_shortMark);
//	}
//      }
//      markX += m_ppd;
//    }
//  }
//
//  protected Color getLightShadow() {
//    if (m_lightShadow == null) {
//      Color  bk = getBackground();
//      m_lightShadow = bk.brighter();
//      m_darkShadow = bk.darker();
//    }
//    return m_lightShadow;
//  }
//
//  protected Color getDarkShadow() {
//    getLightShadow();
//    return m_darkShadow;
//  }


  /** Quickie L1 test main */
  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) {
      e.printStackTrace();
    }
    JFrame f = new JFrame("Just a quickie L1 test main");
    TimeframePanel tPanel = new TimeframePanel();
    f.getContentPane().add(new JScrollPane(tPanel));
    Dimension d = new Dimension(1600,400);
    tPanel.setPreferredSize(d);
    f.setSize(600,300);
//    Dimension screenSize =  Toolkit.getDefaultToolkit().getScreenSize();
//    f.setSize((int)screenSize.getWidth()-150, (int)screenSize.getHeight()-150);
    f.setLocation(0,0);
//    f.pack();
    WindowListener wndCloser = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    };
    f.addWindowListener(wndCloser);
    f.setVisible(true);
    f.show();
  }


  /** Indicates (in a vague fashion) that something in the chart changed. */
  public void chartChanged(TPGanttEvent e) {
    log.debug("TfP.chartChanged(): entering with e="+e.toString());
    /** @todo Figure out more efficient way to redraw..  For now, redo entire chart. */
    getMinMaxDisplayDates();
    _bkgdImage=null;
//    this.invalidate();
    this.repaint();
  }

  /** Constructor */
  public TimeframePanel() {
    doSetup(null);
  }
  public TimeframePanel(TPGanttModel m) {
    doSetup(m);
  }
  /** Do all the setup chores */
  public void doSetup(TPGanttModel m) {
    initZoomLevels();
    //_gm = m;    // keep a copy of the gantt model.
    this.setGanttModel(m);
    if (null!=m) {
      m.addGanttListener(this);
      _tm = m.getTPTableModel();
    }
    getMinMaxDisplayDates();
//    _timeframePanel = new JPanel();
//    this.setLayout(new BorderLayout());
//    this.add(_timeframePanel, BorderLayout.NORTH);

    enableEvents(ComponentEvent.COMPONENT_RESIZED);// dfk: listen to resizes??
    // Attach a key listener to zoom in/out wrt. '+' and '-' keys..
    _keyListener = new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        log.debug("TP.keyPressed: caught event: "+e.toString());
        switch (e.getKeyChar()) {
          case '-':
              log.debug("User hit -");
              if (_zLvl < _zLvls.length-1) {
                _zLvl++;
                _bkgdImage = null;
                repaint();
              }
              break;
          case '+':
              log.debug("User hit +");
              if (_zLvl > 0) {
                _zLvl--;
                _bkgdImage = null;
                repaint();
              }
              break;
        }//switch
      }
    };
//    GuiUtil.getFrame(this).addKeyListener(_keyListener);  // can't do this, frame doesnt exist yet..
  }

  /** Get an instance of the associated Gantt model. */
  public TPGanttModel getGanttModel() {
    return _gm;
  }
  /** Set the current Gantt model. */
  public void setGanttModel(TPGanttModel m) {
    _gm = m;
  }

  /** Get an instance of the associated table model.
   *  Should this even exist, or should people get this themselves?
   */
  public TPTableModel getTPTableModel() {
    if (null!=_gm) {
      return _gm.getTPTableModel();
    }
    return null;
  }

  /** Set up the various zoom levels.
   *  @todo Maybe some of this should be Localed, TimeZoned, and ResourceBundled...
   */
  public void initZoomLevels() {
    _zLvls = new ZoomInfo[] {
      // Part of a day
//dfk02May2001
//      new ZoomInfo( "Level -1: 1 day @ 5\" (350px), twelve 2-hr subdivisions",
//                    "EEE M/d", "h a", 12, /*2*60*60*1000,*/ Calendar.HOUR_OF_DAY, 2,
//                    350, Arrays.asList(new String[]{
//                        "0","1","2","3","9","10","11"})
//      ),
      new ZoomInfo( "Level -1: 1 day @ 6\" (420px), twelve 2-hr subdivisions",
                    "EEE M/d", "h a", 12, /*2*60*60*1000,*/ Calendar.HOUR_OF_DAY, 2,
                    420, Arrays.asList(new String[]{
                        "0","1","2","3","9","10","11"})
      ),
      new ZoomInfo( "Level 0 (default): 1 day @ 2\"(150px), four 6-hr subdivisions",
                     "EEE M/d", "h a", 4, /*6*60*60*1000,*/ Calendar.HOUR_OF_DAY, 6,
                     150, Arrays.asList(new String[]{"0","3"})
      ),
      // A few days
      new ZoomInfo( "Level 1: 1 week @ 1.5\"(110px), seven 1-day subdivisions",
                    "MMM d, ''yy", "E", 7, /*24*60*60*1000,*/ Calendar.DAY_OF_WEEK, 1,
                    110, Arrays.asList(new String[]{"0","6"}/*{"1","7"}DAY_OF_WEEK is 1..7, not 0..6*/)
      )
    };
  }

  /** Convert a time to an x-coordinate inside the timeframe panel. */
  public int t2x(Date d) {
    return t2x(d.getTime());
  }

  /** Convert a time to an x-coordinate inside the timeframe panel. */
  public int t2x_OLD(long millis) {
   long maxTime = _latestDate.getTime() - _earliestDate.getTime();
   /** @todo Don't use panel width!  Use zoom.unitWidth * zoom.numSubdivisions instead. */
   int panelWidth = /*_timeframePanel.*/getWidth();
   // how far (percentage-wise) are we towards the right?  0-100%..
   double posRatio = (double)(millis - _earliestDate.getTime()) / (double)maxTime;
   return (int)(0 + (posRatio * panelWidth));
  }
  /** Convert a time to an x-coordinate inside the timeframe panel. */
  public int t2x_OLD2(long millis) {
    ZoomInfo zoom = _zLvls[ _zLvl ];
    // Number of pixels in one subdivision
    double subdivPix = (double)zoom.unitWidth / (double)zoom.numSubdivisions;
    // Assume that earlyDate corresponds w/ 0 px, and increases according to zoomInfo.
    // x = St'/D = S(t-t0)/D
    int offset = (int)(subdivPix * (double)(millis - _earliestDate.getTime())
                                 / (double)millisPerSubdivision(millis));
    // Error checking..  if x<0, return 0.
    if (offset<0)  { return 0; }
    return offset;
  }
  /** Convert a time to an x-coordinate inside the timeframe panel. */
  public void t2xInit() {
    ZoomInfo zoom = _zLvls[ _zLvl ];
    // Number of pixels in a subdivision
    double subdivPix = (double)zoom.unitWidth / (double)zoom.numSubdivisions;
    // Assume that earlyDate corresponds w/ 0 px, and increases according to zoomInfo.
    // x = St'/D = S(t-t0)/D
    double x=0.0;
    long aTime=_earliestDate.getTime();
    Vector keys = new Vector();
    Vector values = new Vector();

    //
    while (true) {
      long delta = millisPerSubdivision(aTime);
      // it's disturbing that the "long" is a big negative number;
      //  DateFormat it to see what it really is...
//      if(false)log.debug("TfP.t2xInit(): does input '"+millis+"'("
//          + DateFormat.getDateTimeInstance().format(new Date(millis))
//          + ") lie within range '"+aTime+"'("
//          + DateFormat.getDateTimeInstance().format(new Date(aTime))
//          + ") and '"+(aTime+delta)+"' ("
//          + DateFormat.getDateTimeInstance().format(new Date(aTime+delta))
//          + ")?" );
      if (aTime > _latestDate.getTime()) {
        break;
      } else {
//        /*if(false)*/log.debug("TfP.t2xInit(): no, input ("+millis+
//            ") is not in this subdiv range (["+aTime+", "+(aTime+delta)+
//            "]); Advancing to the next subdivision..");
        // Fill in this chunk of the array w/ linear interpolation
//        log.debug("\t--------- Linear interpolate from "+((int)x)+" to "+((int)x+subdivPix) );
        for (int i = (int)x; i <= (int)(x+subdivPix); i++) {
          double aRatio = 1.0d - ((x+subdivPix)-(double)i)/(x+subdivPix);
          keys.addElement(new Integer(i));
          values.addElement(new Long(aTime + (long)(delta * aRatio)));
//          log.debug("\t"+i+"\t->\t" + ((Long)values.elementAt(i)).toString());
        }
        aTime += delta;
        x+=subdivPix;
      }
    }//while

    // Apply the values to the array.
    // Go backwards to get the lowest of each repeated x value..
    int arrayMax = (keys.size()>0) ? ((Integer)keys.elementAt(keys.size()-1)).intValue()+1 : 0;
    log.debug("t2xInit(): arrayMax is "+arrayMax);
    x2tArray = new long[ arrayMax ];
//    for (int i=keys.size()-1; i >= 0; i--) {
//      int aKey = ((Integer)keys.elementAt(i)).intValue();
//  //dfk05May2001; full of discontinuities!!
//      long aValue = ((Long)values.elementAt(i)).longValue();
//      log.debug("\tt2xInit(): setting x2tArray["+aKey+" of "+keys.size()+"]="+aValue);
//      x2tArray[aKey] = aValue;
//    }
    long valMin = (values.size()>0) ? ((Long)values.elementAt(0)).longValue() : 0;
    long valMax = (values.size()>0) ? ((Long)values.elementAt(values.size()-1)).longValue() : 0;
    for (int i=0; i < arrayMax; i++) {
      long aValue = valMin +  (long)( (double)i/(double)arrayMax*(valMax-valMin) );
//      log.debug("\tt2xInit(): setting x2tArray["+i+" of "+arrayMax+"]="+aValue);
      x2tArray[i] = aValue;
    }

    //

//    while (aTime <= _latestDate.getTime()) {
//      keys.addElement(new Integer((int)x));
//      values.addElement(new Long(aTime));
//      long delta = millisPerSubdivision(aTime);
//      aTime += delta;
//      x+=subdivPix;
//    }//while
//    // Now, build the longarray
//    int count=0;
//    x2tArray = new long[keys.size()];
//    for (int i=0; i < keys.size(); i++) {
//      Integer aKey = (Integer)ki.next();
//      x2tArray[count++] = ((Long)hm.get(aKey)).longValue();
//      log.debug("t2xInit(): Mapped "+ aKey.toString()+" to "
//            + ((Long)hm.get(aKey)).longValue());
//    }
  }

  /** Convert a time to an x-coordinate inside the timeframe panel. */
  public int t2x(long millis) {
    if (null==x2tArray) {
      t2xInit();
    }
    if (0==x2tArray.length) { return 0; }
    if (millis < _earliestDate.getTime())  {
      log.error("t2x(): millis is earlier than earliestDate; return 0.");
      return 0;
    }
    ZoomInfo zoom = _zLvls[ _zLvl ];
    // Number of pixels in a subdivision
    double subdivPix = (double)zoom.unitWidth / (double)zoom.numSubdivisions;
    // Assume that earlyDate corresponds w/ 0 px, and increases according to zoomInfo.
    // x = St'/D = S(t-t0)/D
    double x=0.0;
    long aTime=_earliestDate.getTime();
    int i=0;
    for (i=0; i < x2tArray.length; i++) {
      if (millis < x2tArray[i]) {
        i--;
        if (i<0) i=0;
        break;
      }
    }
    return i;
    // kill the rest of this??
//    x += ((double)i * subdivPix);
//    aTime = x2tArray[i];
//    while (true) {
//      long delta = millisPerSubdivision(aTime);
//      // it's disturbing that the "long" is a big negative number;
//      //  DateFormat it to see what it really is...
//      if(false)log.debug("TfP.t2x(): does input '"+millis+"'("
//          + DateFormat.getDateTimeInstance().format(new Date(millis))
//          + ") lie within range '"+aTime+"'("
//          + DateFormat.getDateTimeInstance().format(new Date(aTime))
//          + ") and '"+(aTime+delta)+"' ("
//          + DateFormat.getDateTimeInstance().format(new Date(aTime+delta))
//          + ")?" );
//      // If it's in this span, do the ratio math and return an answer;
//      // otherwise, increment x & time pointer and continue.
//      if (millis >= aTime  &&  millis <= aTime + delta) {
//        /*if(false)*/log.debug("TfP.t2x(): yes, input ("+millis+
//            ") is in this subdiv range (["+aTime+", "+(aTime+delta)+
//            "]); done.");
//  //dfk05May2001; doesnt scale addition by pixels..;  x += (int)((double)(millis - aTime)/(double)delta);
//        x += (int)((double)(millis - aTime)/(double)delta)*(double)subdivPix;
//        return (int)x;
//      } else {
//        //
//        // Should never be executed
//        //
//        /*if(false)*/log.debug("TfP.t2x(): no, input ("+millis+
//            ") is not in this subdiv range (["+aTime+", "+(aTime+delta)+
//            "]); Advancing to the next subdivision..");
//        aTime += delta;
//        x+=subdivPix;
//      }
//    }
  }
  /** Convert a time to an x-coordinate inside the timeframe panel. */
  public int t2x_OLD3(long millis) {
    if (millis < _earliestDate.getTime())  {
      log.error("t2x(): millis is earlier than earliestDate; return 0.");
      return 0;
    }
    ZoomInfo zoom = _zLvls[ _zLvl ];
    // Number of pixels in a subdivision
    double subdivPix = (double)zoom.unitWidth / (double)zoom.numSubdivisions;
    // Assume that earlyDate corresponds w/ 0 px, and increases according to zoomInfo.
    // x = St'/D = S(t-t0)/D
    double x=0.0;
    long aTime=_earliestDate.getTime();
    while (true) {
      long delta = millisPerSubdivision(aTime);
      // it's disturbing that the "long" is a big negative number;
      //  DateFormat it to see what it really is...
      if(false)log.debug("TfP.t2x(): does input '"+millis+"'("
          + DateFormat.getDateTimeInstance().format(new Date(millis))
          + ") lie within range '"+aTime+"'("
          + DateFormat.getDateTimeInstance().format(new Date(aTime))
          + ") and '"+(aTime+delta)+"' ("
          + DateFormat.getDateTimeInstance().format(new Date(aTime+delta))
          + ")?" );
      // If it's in this span, do the ratio math and return an answer;
      // otherwise, increment x & time pointer and continue.
      if (millis >= aTime  &&  millis <= aTime + delta) {
        /*if(false)*/log.debug("TfP.t2x(): yes, input ("+millis+
            ") is in this subdiv range (["+aTime+", "+(aTime+delta)+
            "]); done.");
//dfk05May2001; doesnt scale addition by pixels..;  x += (int)((double)(millis - aTime)/(double)delta);
        x += (int)((double)(millis - aTime)/(double)delta)*(double)subdivPix;
        return (int)x;
      } else {
        /*if(false)*/log.debug("TfP.t2x(): no, input ("+millis+
            ") is not in this subdiv range (["+aTime+", "+(aTime+delta)+
            "]); Advancing to the next subdivision..");
        aTime += delta;
        x+=subdivPix;
      }
    }
//    int offset = (int)(subdivPix * (double)(millis - _earliestDate.getTime())
//                                 / (double)millisPerSubdivision(millis));
//    // Error checking..  if x<0, return 0.
//    if (offset<0)  { return 0; }
//    return offset;
  }

  /** Convert an x-coordinate inside the timeframe to a {@link Date}. */
  public Date x2t_OLD(int xCoord) {
   /** @todo Don't use panel width!  Use zoom.unitWidth * zoom.numSubdivisions instead. */
    int panelWidth = getWidth();
    if (xCoord <= 0)   {  return new Date(_earliestDate.getTime());  }
    if (xCoord >= panelWidth)    {  return new Date(_latestDate.getTime());  }
    double posRatio = ((double)xCoord) / ((double)panelWidth);
    long maxTime = _latestDate.getTime() - _earliestDate.getTime();
    long millis = _earliestDate.getTime() + (long)(posRatio * maxTime);
    return new Date(millis);
  }
  /** Convert an x-coordinate inside the timeframe to a {@link Date}.
   *  @deprecated It's dangerous to use this function..  Know that you'll
   *  suffer (possibly) a great loss of precision;  the difference of 1 pixel
   *  can mean the difference of millions of milliseconds..
   *  You're better off just keeping a calendar instance handy, and t2x() it...
   */
  public Date x2t(int xCoord) {
    long millis;
    ZoomInfo zoom = _zLvls[ _zLvl ];
    // Number of pixels in one subdivision
    double subdivPix = (double)zoom.unitWidth / (double)zoom.numSubdivisions;
    // Assume that earlyDate corresponds w/ 0 px, and increases according to zoomInfo.
    // t' = Dx/S; t-t0 = Dx/S; t = Dx/S + t0
    millis = (long)((double)millisPerSubdivision(_earliestDate.getTime()) * (double)xCoord / subdivPix
                        + _earliestDate.getTime());
    /** @todo Error checking..  Bound the time between earliest & latest??? */
    return new Date(millis);
  }

  public void getMinMaxDisplayDates() {
      ZoomInfo zoom = _zLvls[ _zLvl ];
//    if (null==_gm) {
//      // No gantt model yet;  Set min,max to today,tomorrow.
//      _earliestDate = new Date();
//      GregorianCalendar c = new GregorianCalendar();
//      c.setTime( _earliestDate );
//      c.add(Calendar.DAY_OF_YEAR, 1);
//      _latestDate = c.getTime();
      // No gantt model yet;  Set min,max to today,today+10 units.
      GregorianCalendar c = new GregorianCalendar();
      getEarliestLatestDates();   // sets _earliestDate, _latestDate..
//      if (null==_gm) {
//        _earliestDate = new Date();
//      } else {
//        _earliestDate = getEarliestDate();
//      }
      // Set earliest time to "beginning of day" (eg. today 12:00:00am)
      c.setTime( _earliestDate );
      c.set(c.HOUR_OF_DAY, 0);
      c.set(c.MINUTE, 0);
      c.set(c.SECOND, 0);
      c.set(c.MILLISECOND, 0);
      // If we have a different view (eg Week view), go to beginning of that unit.
      if (zoom.subdivScale == Calendar.DAY_OF_WEEK) {
        c.set(c.DAY_OF_WEEK, 1 /*not 0!*/);
      }
      _earliestDate = new Date(c.getTime().getTime());  // reset it to midnight of its previous date
//      // Set latest time to "last millisecond before beginning of 11th day (ie. end of 10th day)
//      c.add(zoom.subdivScale, (int)(11 * zoom.numSubdivisions * zoom.subdivScaleMultiplier));
      // First, find 12:00:00am of latest day
      c.setTime( _latestDate );
      c.set(c.HOUR_OF_DAY, 0);
      c.set(c.MINUTE, 0);
      c.set(c.SECOND, 0);
      c.set(c.MILLISECOND, 0);
      // If we have a different view (eg Week view), go to beginning of that unit.
      if (zoom.subdivScale == Calendar.DAY_OF_WEEK) {
        c.set(c.DAY_OF_WEEK, 1 /*not 0!*/);
      }
      // Then, set latest time to "last millisecond before beginning of latest's next set of subdiv's".
      c.add(zoom.subdivScale, (int)(1 * zoom.numSubdivisions * zoom.subdivScaleMultiplier));
      c.add(Calendar.MILLISECOND, -1);
      _latestDate = new Date(c.getTime().getTime());

      // Reset the x2t shortcut.
      x2tArray = null;

      log.debug("getMinMaxDisplayDates(): at end, just before summary..");
      log.debug("getMinMaxDisplayDates(): _earliest="
          + _earliestDate.getTime()+"("
          + DateFormat.getDateTimeInstance().format(_earliestDate)
          + ", x="+t2x(_earliestDate)
          + "), _latest="
          + _latestDate.getTime()+"("
          + DateFormat.getDateTimeInstance().format(_latestDate)
          + ", x="+t2x(_latestDate)
          + ")");

//    } else {
//      /** @todo Get min/max from model. */
//    }
  }

//  /** Get the earliest date available in the task plan, or current date if errors. */
//  public void getEarliestLatestDates_old() {
//    GregorianCalendar cEarly = new GregorianCalendar();
//    GregorianCalendar cLatest = new GregorianCalendar();
//    TPTableModel tm;
//    if (null!=(tm = getTPTableModel())) {
//      SubtaskRequest req = tm.getTask(0);
//      log.debug("TfP.getEarliestLatestDates(): "+req.getId()+": "
//          + "earliest="+DateFormat.getDateTimeInstance().format(req.getEarliestTimeToStart())
//          + ", latest="+DateFormat.getDateTimeInstance().format(req.getDeadline()) );
//      cEarly.setTime(req.getEarliestTimeToStart());
//      cLatest.setTime(req.getDeadline());
//      for (int i=1; i < tm.getRowCount(); i++) {
//        SubtaskRequest req2 = tm.getTask(i);
//        System.out.println("TfP.getEarliestLatestDates(): "+req2.getId()+": "
//            + "earliest="+DateFormat.getDateTimeInstance().format(req2.getEarliestTimeToStart())
//            + ", latest="+DateFormat.getDateTimeInstance().format(req2.getDeadline()) );
//        if (req2.getEarliestTimeToStart().before(cEarly.getTime())) {
//          cEarly.setTime(req2.getEarliestTimeToStart());
//        }
//        if (req2.getDeadline().after(cLatest.getTime())) {
//          cLatest.setTime(req2.getDeadline());
//        }
//      }//for
//    }
//    _earliestDate = cEarly.getTime();
//    _latestDate = cLatest.getTime();
//    log.debug("TfP.getEarliestLatestDates(): at end: "
//        + "earliest="+DateFormat.getDateTimeInstance().format(_earliestDate)
//        + ", latest="+DateFormat.getDateTimeInstance().format(_latestDate) );
//  }

  /** Get the earliest and latest dates in the TaskPlan. */
  public void getEarliestLatestDates() {
    // Set the dates to "something" just in case we bail early..
    _earliestDate = AgentTime.getDate();
    _latestDate = AgentTime.getDate();

    if (null==_tm  ||  null==_tm.getTaskPlan()) {
      log.info("TfP.getEarliestLatestDates(): null TaskPlan");
      return;
    }
    _earliestDate = _tm.getTaskPlan().getPlanStartDate();
    _latestDate = _tm.getTaskPlan().getPlanDeadline();
    if ((_earliestDate == null) || (_latestDate == null)) {
      _earliestDate = AgentTime.getDate();
      _latestDate = AgentTime.getDate();
      return;
    }

    log.info("TfP.getEarliestLatestDates(): at end: "
        + "earliest="+DateFormat.getDateTimeInstance().format(_earliestDate)
        + ", latest="+DateFormat.getDateTimeInstance().format(_latestDate) );
  }

  /**
   * Calculate the number of milliseconds from this date to the date
   * that's one subdiv away. The GregorianCalendar hopefully accounts
   * for leap-yrs/days/seconds and other quirks.
   */
  public long millisPerSubdivision(long millis) {
    Date date = new Date(millis);
    GregorianCalendar c1 = new GregorianCalendar();
    GregorianCalendar c2 = new GregorianCalendar();
    c1.setTime(date);
    c2.setTime(date);
    // Use the current zoom to jump fwd.
    ZoomInfo zoom = _zLvls[ _zLvl ];
    c2.add(zoom.subdivScale, zoom.subdivScaleMultiplier);
    return c2.getTime().getTime() - c1.getTime().getTime();
  }

  /**
   * Paints this thing.
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    GregorianCalendar c = new GregorianCalendar();
    //setSize(m_width, m_height);
    // Scale panel size to whatever the max data span is.....
    Dimension d = /*_timeframePanel.*/_tm.getView().getSize();  // size of jtable
    Rectangle headerRect = _tm.getView().getTableHeader().getHeaderRect(0);
    Dimension tableSize = _tm.getView().getSize();
    log.info("TP.paintComponent(): Before setSize, _latestDate="+
      _latestDate.getTime() + "(" +
      DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.LONG)
                                         .format(_latestDate) + ")");
      // Get the current zoom info
      ZoomInfo zoom = _zLvls[ _zLvl ];
      double subdivPix = (double)zoom.unitWidth / (double)zoom.numSubdivisions;

//dfk05May2001;    Dimension d2 = new Dimension(t2x(_latestDate),
//                  tableSize.height+MagnetClientConstants.TP_TABLE_COLUMN_HEADER_HEIGHT);// 2000);//d.height+headerRect.height);
    Dimension d2 = new Dimension(t2x(_latestDate),// + (int)subdivPix,
                  tableSize.height+MagnetClientConstants.TP_TABLE_COLUMN_HEADER_HEIGHT);// 2000);//d.height+headerRect.height);
    setSize(d2);
//    Rectangle r = g.getClipBounds();
    Graphics2D g2 = (Graphics2D) g;  //_timeframePanel.getGraphics();

    if (null!=_bkgdImage) {
      log.info("TP.paintComponent(): doing drawImage() instead of reconstructing..");
      g2.drawImage(_bkgdImage, 0,0, /*null*/this);
    } else {
      log.info("TP.paintComponent(): reconstructing instead of doing drawImage()..");
      // Associate a new graphics context w/ the bufferedImage;
      //  This time thru, draw into the Image;
      //  then, at the end, re-call this method with the orig "g",
      //  and it'll use the icon the next time.
//      _bkgdImage = new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_RGB);
     _bkgdImage = new BufferedImage(d2.width, d2.height, BufferedImage.TYPE_INT_RGB);
      g2 = (Graphics2D) _bkgdImage.getGraphics();
      // Draw all stuff into a "path", then render it at the very end.
      GeneralPath path = new GeneralPath();

  //    // TEST:: draw diag line across entire panel
  //    g2.setColor(Color.black);
  //    path.moveTo(0,0);
  //    path.lineTo(d.width, d.height);
  //    g2.draw(path);

      // First, draw the background and ledger lines
      // What subdiv do we start with?
  //    int subdivCounter = -1;
      int subdivCounter = getSubdivAt(_earliestDate.getTime()) - 1/*why 2??*/;
      int subdivWidth = zoom.unitWidth / zoom.numSubdivisions;
  //    for (int x=0; x < d.width; x+=subdivWidth) {
  //    for (long millis=_earliestDate.getTime(); millis < _latestDate.getTime();
  //              millis += millisPerSubdivision(millis)) {
  //      int x = t2x(millis);
      for (c.setTime(_earliestDate);
           c.getTime().getTime() <= _latestDate.getTime();
           c.add(zoom.subdivScale, zoom.subdivScaleMultiplier)  )
      {
        int x = t2x(c.getTime());
//        Shape s = new Rectangle2D.Double(x, 0, x+subdivWidth, d.height);
        Shape s = new Rectangle2D.Double(x, 0, x+subdivWidth, d2.height);
        // set the appropriate fill color
        subdivCounter = (subdivCounter + 1) % zoom.numSubdivisions;
        boolean isShaded = zoom.whichShaded.contains(""+subdivCounter);
        if (isShaded) {
          g2.setColor(Color.lightGray);
        } else {
          g2.setColor(Color.white);
        }
        g2.fill(s);
        // Draw the rectangle w/ lightGray pen
        g2.setColor(Color.gray);
  //      s = (new BasicStroke()).createStrokedShape(s);
        g2.draw(s);
      }

      // Next, draw the "up" (eg. Date) headers.
      SimpleDateFormat df = new SimpleDateFormat( zoom.upFormat );
  //    Font upFont = new Font("Helvetica", Font.PLAIN, 12);
      Font upFont = new Font("Arial", Font.TRUETYPE_FONT, 10);
  //    for (int x=0; x < d.width; x+=zoom.unitWidth) {
      for (c.setTime(_earliestDate);
           c.getTime().getTime() <= _latestDate.getTime();
           c.add(zoom.subdivScale, zoom.numSubdivisions * zoom.subdivScaleMultiplier)  )
      {
        int x = t2x(c.getTime());
        int y=0, width=zoom.unitWidth, height=15;
        // Draw an opaque background
        g2.setColor(Color.lightGray/*getBackground()*/);
        g2.fillRect(x,y, width, height);
        // Draw slightly darker border
        g2.setColor(Color.gray);
        g2.drawRect(x,y, width, height);
        // Render the text
//        Date date = x2t(x);    dfk05May2001
        Date date = c.getTime();
        String dateString = df.format(date);
        log.info("TfP.paintComponent():About to render string: "+ dateString);
        FontRenderContext frc = g2.getFontRenderContext();
        TextLayout t1 = new TextLayout(dateString, upFont, frc);
        AffineTransform myChunk = AffineTransform.getTranslateInstance(0d, 0d);
        Shape src = t1.getOutline(myChunk);
        Rectangle rText = src.getBounds();
        // Left-justify it
        float xText = x - rText.x;
        float yText = y + height/2 + t1.getAscent()/4;
        // Position it
        AffineTransform shift = AffineTransform.getTranslateInstance(xText + 5/*inset*/, yText + 1/*inset*/);
        Shape shp = shift.createTransformedShape(src);
        // Draw it
        g2.setColor(Color.black);
  //      g2.setStroke(new BasicStroke(0f));
        g2.draw(shp);
      }

//      // dfk02May2001: REMOVE THIS   --  just seeing what fonts are available
//      String[] fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment().
//    getAvailableFontFamilyNames();
//    for (int zzz=0; zzz<fontNames.length; zzz++) {
//      log.info("\t** TP: Detected font: "+ fontNames[zzz]);
//    }

      // Next, do the Down headers.
           df = new SimpleDateFormat( zoom.downFormat );
  //    Font upFont = new Font("Helvetica", Font.PLAIN, 12);
//dfk02May2001;      Font downFont = new Font("Arial", Font.TRUETYPE_FONT, 10);//dfk02May2001; 8);
      Font downFont = new Font("Arial Black", Font.PLAIN, 12);//dfk02May2001; 8);
  //    for (int x=0; x < d.width; x+=zoom.unitWidth/zoom.numSubdivisions) {
  //    for (long millis=_earliestDate.getTime(); millis < _latestDate.getTime();
  //              millis += zoom.millisPerSubdivision) {
  //      int x = t2x(millis);
      for (c.setTime(_earliestDate);
           c.getTime().getTime() <= _latestDate.getTime();
           c.add(zoom.subdivScale, zoom.subdivScaleMultiplier)  )
      {
        int x = t2x(c.getTime());
        log.info("\tDownHdrs: Current calendar time: " +
            DateFormat.getDateTimeInstance().format(c.getTime()) +
            ", xCoord="+x + ", millis="+c.getTime().getTime() );
        int y=15, width=zoom.unitWidth/zoom.numSubdivisions, height=15;
        // Draw an opaque background
        g2.setColor(Color.white);//dfk02May2001;  lightGray/*getBackground()*/);
        g2.fillRect(x,y, width, height);
        // Draw slightly darker border
        g2.setColor(Color.gray);
        g2.drawRect(x,y, width, height);
        // Render the text
        // dfk02May2001; Specify antialiasing w/ a RenderingHints obj,
        //  then plug it into my Graphics2D instance.
        RenderingHints rh = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2.addRenderingHints(rh);
        g2.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        //
        Date date = c.getTime();  //x2t(x);
        String dateString = df.format(date);
        if (2==_zLvl) {
          dateString = dateString.substring(0,1);   // One-lettered days: M T W T F S S
        }
        log.info("TfP.paintComponent():About to render string: "+ dateString
            +", for date "+ DateFormat.getDateTimeInstance().format(date)
            );
        FontRenderContext frc = g2.getFontRenderContext();
        TextLayout t1 = new TextLayout(dateString, downFont, frc);
        AffineTransform myChunk = AffineTransform.getTranslateInstance(0d, 0d);
        Shape src = t1.getOutline(myChunk);
        Rectangle rText = src.getBounds();
        // Center-justify it
        float xText = x + (width-rText.width)/2;
        float yText = y + height/2 + t1.getAscent()/4;
        // Position it
  //      AffineTransform shift = AffineTransform.getTranslateInstance(xText + 5/*inset*/, yText + 1/*inset*/);
        AffineTransform shift = AffineTransform.getTranslateInstance(xText + 0/*inset*/, yText + 0/*inset*/);
        Shape shp = shift.createTransformedShape(src);
        // Draw it
        g2.setColor(Color.black);
  //      g2.setStroke(new BasicStroke(0f));
        g2.draw(shp);

        GregorianCalendar zz = new GregorianCalendar();
        zz.setTime(new Date(c.getTime().getTime()));
        zz.add(zoom.subdivScale, zoom.subdivScaleMultiplier);
        log.info("\tDownHdrs: Next loop will occur if " +
            zz.getTime().getTime()+" <= "+_latestDate.getTime() );
      }

      // Save the image so we don't have to keep redrawing;  only when zoomlevel changes..
//      _bkgdImage = g2.getDeviceConfiguration().createCompatibleImage(d.width, d.height);
      // dfk; not necessary, already done the first time thru...       _bkgdImage = (BufferedImage) this.createImage(d.width, d.height);
      paintComponent(g);   // call w/ orig screen-graphics context
    }//if needed redraw from scratch

  }//paintComponent()

  /** Based on current zoomInfo, determine what subdiv the specified time occurs in. */
  public int getSubdivAt_OLD(long millis) {
    if (millis < _earliestDate.getTime())  {  return 0;  }
    ZoomInfo zoom = _zLvls[ _zLvl ];
    GregorianCalendar c = new GregorianCalendar();
    c.setTime( new Date(millis));
    switch (zoom.subdivScale) {
      case Calendar.HOUR_OF_DAY:
          int hour = c.get(c.HOUR_OF_DAY);
          return (int)Math.floor((double)hour / (double)zoom.subdivScaleMultiplier);
      case Calendar.DAY_OF_WEEK:
          /** NOTE:: DAY_OF_WEEK runs 1..7, not 0..6 !!!  Yikes! */
          int day = c.get(c.DAY_OF_WEEK) - 1;
          return (int)Math.floor((double)day / (double)zoom.subdivScaleMultiplier);
      default:
          System.err.println("Unexepected subdivScale "+zoom.subdivScale);
          return 0;
    }//switch
//    int subdiv = -1;
//    // prev,next point to time of beginning of subdiv boundaries;
//    //  ie.     prev, millis, next
//    long prev, next;
//    prev = next = _earliestDate.getTime();
//    while (true) {
//      if (prev <= millis  &&  next >= millis) {
//        return subdiv;
//      } else {
//        prev = next;
//        next += zoom.millisPerSubdivision;
//        subdiv = (subdiv + 1) % zoom.numSubdivisions;
//      }
//    }
  }

  /** Based on current zoomInfo, determine what subdiv the specified time occurs in. */
  public int getSubdivAt(long millis) {
    if (millis < _earliestDate.getTime())  {  return 0;  }
    ZoomInfo zoom = _zLvls[ _zLvl ];
    int subdiv = 0;
    long aTime=_earliestDate.getTime();
    while (true) {
      // If it's in this span, return the (subdiv) counter;
      // otherwise, increment counter & time pointer and continue.
      long delta = millisPerSubdivision(aTime);
      if (millis >= aTime  &&  millis <= aTime + delta) {
        return subdiv;
      } else {
        subdiv = (subdiv+1) % zoom.numSubdivisions;
        aTime += delta;
      }
    }
  }

  /** Return the current zoom level */
  public int getZoomLevel() {
    return _zLvl;
  }
  /** Set the current zoom level */
  public void setZoomLevel(int lvl) {
    _zLvl = lvl;
    this.chartChanged(new TPGanttEvent(this, "zoom level changed"));
    repaint();
  }
  /** Change current zoomlevel so we're zooming in.
   *  This method will be aware of boundary conditions.
   */
  public void zoomIn() {
    int currentZoom = getZoomLevel();
    if (--currentZoom < 0 ) { currentZoom = 0; }
    setZoomLevel(currentZoom);
  }
  /** Change current zoomlevel so we're zooming Out.
   *  This method will be aware of boundary conditions.
   */
  public void zoomOut() {
    int currentZoom = getZoomLevel();
    if (++currentZoom > _zLvls.length-1 ) { currentZoom = _zLvls.length-1; }
    setZoomLevel(currentZoom);
  }

  // ----------------------------------------------------------------------
  // BEGIN Scrollable impl  (from online swing book, chap 7.)
  public Dimension getPreferredScrollableViewportSize() {
    return getPreferredSize();
  }
  public int getScrollableBlockIncrement(Rectangle r,
    int orietation, int direction) {
      return 10;
  }
  public boolean getScrollableTracksViewportHeight() {
    return false;
  }
  public boolean getScrollableTracksViewportWidth() {
    return false;
  }
  public int getScrollableUnitIncrement(Rectangle r,
    int orientation, int direction) {
      return 10;
  }
  // End Scrollable impl  (from online swing book, chap 7.)
  // ----------------------------------------------------------------------


  /** Inner class that contains labelling info for a particular zoom level.
   *  I plan on offering only a predetermined set of zoom levels, like MS Project.
   *  The below examples correspond w/ MS Proj defaults..
   */
  public class ZoomInfo {
    /** Not used..   Name like "1 day @ 150 pixels wide, four 6-hr subdivisions". */
    public String name;
    /** Holds the {@link SimpleDateFormat} format string for the
     *  upper line of columnHeader, like <tt>"EEE M/d"</tt> for "Wed 2/28".
     *  (No clean way to abbrev months to 3 letters. :(  )
     */
    public String upFormat;
    /** Holds the {@link SimpleDateFormat} format string for the
     *  lower cells of columnHeader, like <tt>"h a"</tt> for "6 PM".
     */
    public String downFormat;
    /** Indicates the number of subdivisions (eg. 4). */
    public int numSubdivisions;
    /** Indicates total width (in pixels) of a single "unit", like
     *  150 pixels for 1 day.  This includes 1 px for each subdiv divider
     *  ("column margin").
     */
    public int unitWidth;
// NOTE: millisPerSubdivision is DEPRECATED; See millisPerSubdivision(long) instead..
//    /** Indicates the number of milliseconds contained in a subdivision, like
//     *  6hr * 60min/hr * 60sec/min * 1000ms/sec
//     */
//    public long millisPerSubdivision;
    /** A field from {@link Calendar} indicating the "scale" of the subdivision
     *  so we know how to "go back to the nearest integer subdivision";
     *  eg. {@link Calendar#HOUR_OF_DAY} to indicate subdivision in terms of hours.
     */
    public int subdivScale;
    /** Indicates the multiplier for ZoomInfo.subdivScale.
     *  For example, subdivs of 6 hr intervals will be value "6".
     */
    public int subdivScaleMultiplier;
    /** Indicates which ones should be shaded, if any.
     *  For example, the midnight-6am and 6pm-midnight timeframes are shaded
     *  so people's attention is instead drawn twds the timeframe of "normal
     *  waking hours".  List of integer strings.
     */
    public java.util.List whichShaded;

    /** Quickie constructor */
    public ZoomInfo(String name, String upFormat, String downFormat,
                    int numSubdivisions, //long millisPerSubdivision,
                    int subdivScale, int subdivScaleMultiplier,
                    int unitWidth, java.util.List whichShaded)
    {
      this.name = new String(name);
      this.upFormat = new String(upFormat);
      this.downFormat = new String(downFormat);
      this.numSubdivisions = numSubdivisions;
      //this.millisPerSubdivision = millisPerSubdivision;
      this.subdivScale = subdivScale;
      this.subdivScaleMultiplier = subdivScaleMultiplier;
      this.unitWidth = unitWidth;
      this.whichShaded = whichShaded;
    }
  }//class ZoomInfo
}
