/*-------------------------------------------------------------------------
 * Magnet unit tests
 * $Id: TPTableModelTest.java,v 1.1.2.1 2002/05/05 19:42:03 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *-------------------------------------------------------------------------
 * Revision History:
 *  22Feb2001 DFK - Initial submission
 */

package edu.umn.magnet.client.userif;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.*; // dfk; for jtable stuff (not necessary)
import java.awt.event.*;  // dfk; for jtable stuff (not necessary)

import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.protocol.JobInfo;

public class TPTableModelTest extends TestCase
{
  private TPTableModel model1,model2;
  private TaskPlan plan1, plan2;
  private TaskComparator c1,c2;
  private SubtaskRequest subtask1;
  private SubtaskRequest subtask2;
  private SubtaskRequest subtask3;
  private SubtaskRequest subtask4;
  private TaskWithPrecedence twp1;
  private TaskWithPrecedence twp2;
  private TaskWithPrecedence twp3;
  private TaskWithPrecedence twp4;
  private JobInfo jobInfo;
  private ArrayList list1;

  public TPTableModelTest(String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  /**
   * Here's the suite that includes all these tests
   */
  public static Test suite ()
  {
    return new TestSuite(TPTableModelTest.class);
  }

  private void add2tasks() {
    setup1();
    plan1 = new TaskPlan();
    try {
      plan1.addTask(subtask1);
      plan1.addTask(subtask2);
      plan1.addSuccessor(subtask1, subtask2);
    } catch (Exception e) {
      e.printStackTrace();
      fail("Exception in add2tasks");
    }
  }

  // --------------------------------------------------------------------
  // 1.0 Test the TPTableModel constructors.
  public void testCreate ()
  {
    // Empty constructor: taskPlan=null; "table" dimension is MAX_COL x 0.
    model1 = new TPTableModel();
    assertEquals("model1 TaskPlan is null", null, model1.getTaskPlan());
    assertEquals("model1 number of rows", 0, model1.getRowCount());
    assertEquals("model1 value at any ol' (r,c) should be emptystring",
                    0, ((String)model1.getValueAt(999,999)).length());

    // Populated constructor; taskPlan w/ 2 events.
    setup1();
    plan2 = new TaskPlan();
    try {
      plan2.addTask(subtask1);
      plan2.addTask(subtask2);
      plan2.addSuccessor(subtask1, subtask2);
      model2 = new TPTableModel(plan2);
    } catch (Exception e) {
      e.printStackTrace();
      fail("Exception adding task");
    }
    assertEquals("model2 TaskPlan is correct", plan2, model2.getTaskPlan());
    assertEquals("model2 number of rows", 2, model2.getRowCount());
    assertEquals("model2 value at (1,COL_TASK_ID)", subtask2.getId(),
                  model2.getValueAt(1,model2.COL_TASK_ID));
  }

  // --------------------------------------------------------------------
  // 1.5 Retrieve all cols of a row.
  public void testRetrievingAllCols() {
    add2tasks();  // to plan1
    model1 = new TPTableModel(plan1);
    System.out.println("Contents of task1 within plan1: \n" + model1.dumpRow(0));
  }

  // --------------------------------------------------------------------
  // 1.6 Examine the taskNum <-> taskId mappings.
  public void testMappingsBetwTaskNumAndTaskId() {
    add2tasks();  // to plan1
    model1 = new TPTableModel(plan1);
    // Even though plan order is task1,task2, the model re-sorts to task2,task1.
    assertEquals("subtask1 num -> id", subtask1.getId(), model1.taskNum2TaskId(1));
    assertEquals("subtask2 num -> id", subtask2.getId(), model1.taskNum2TaskId(2));

    assertEquals("subtask1 id -> num", 1, model1.taskId2TaskNum(subtask1.getId()));
    assertEquals("subtask2 id -> num", 2, model1.taskId2TaskNum(subtask2.getId()));
  }

  // --------------------------------------------------------------------
  // 1.7 Switch taskPlans, make sure things get updated correctly.
  public void testSwitchingTaskPlans() {
    setup1();
    plan1 = new TaskPlan();
    plan2 = new TaskPlan();
    try {
      plan1.addTask(subtask1);
      plan1.addTask(subtask2);
      plan1.addSuccessor(subtask1, subtask2);
      plan2.addTask(subtask3);
      plan2.addTask(subtask4);
      plan2.addSuccessor(subtask3, subtask4);

      model1 = new TPTableModel();
    assertEquals("model1 taskPlan is null", null, model1.getTaskPlan());
      model1.setTaskPlan(plan1);
    assertEquals("model1 taskPlan is plan1", plan1, model1.getTaskPlan());
      for (int i=0; i < model1.getRowCount(); i++) {
        System.out.println("model1 of plan1: dumping row "+i+":\n" + model1.dumpRow(i));
      }

      model1.setTaskPlan(plan2);
    assertEquals("model1 taskPlan is plan2", plan2, model1.getTaskPlan());
      for (int i=0; i < model1.getRowCount(); i++) {
        System.out.println("model1 of plan2: dumping row "+i+":\n" + model1.dumpRow(i));
      }

    } catch (Exception e) {
      e.printStackTrace();
      fail("Exception adding task");
    }
  }

  public void testSortingAllTasks() {
    try {
      addAllTasksToPlan();
      model1 = new TPTableModel(plan1);
      System.out.println("---------------------------------------------------");
      System.out.println("---------------------------------------------------");
      System.out.println("Result of sorting all tasks:");
      for (int i=0; i < model1.getRowCount(); i++) {
        System.out.println( model1.dumpRow(i));
      }
    } catch (Exception e) {
      e.printStackTrace();
      fail("Exception adding task");
    }
  }

  // --------------------------------------------------------------------
  // 2.0 Create some tasks
  private void setup1 ()
  {
    try {
      c1 = new TaskComparator();
      c2 = new TaskComparator();
      list1 = new ArrayList();
      /**
       * <pre>
       *  subtask1 "ww":    [-------]        (eStart=3, deadline=10, dur=7)
       *  subtask2 "xx":    [------]         (eStart=3, deadline=9, dur=6)
       *  subtask3 "yy":   [--------]        (eStart=2, deadline=10, dur=8)
       *  subtask4 "zz":    [------]         (eStart=3, deadline=9, dur=6)
       * </pre>
       * Regardless of initial order, final order should be:
       * task3 (earliest eStart), task2 (xx before zz), task4, task1 (longer duration).
       */
      subtask1 = new SubtaskRequest("TaskID1");
          subtask1.setEarliestTimeToStart(new Date(300000L));  // just add 5 zeroes..
          subtask1.setDeadline(new Date(1000000L));
          jobInfo = new JobInfo("name1", "descrip for name1", 1.0, 1.0,
                                1.11, 1.0,
                                1.0,1.0,1.0 );
          subtask1.setJobInfo(jobInfo);
      subtask2 = new SubtaskRequest("TaskID2");
          subtask2.setEarliestTimeToStart(new Date(300000L));
          subtask2.setDeadline(new Date(900000L));
          jobInfo = new JobInfo("name2", "descrip for name2", 1.0, 1.0,
                                2.22, 1.0,
                                1.0,1.0,1.0 );
          subtask2.setJobInfo(jobInfo);
      subtask3 = new SubtaskRequest("TaskID3");
          subtask3.setEarliestTimeToStart(new Date(200000L));
          subtask3.setDeadline(new Date(1000000L));
          jobInfo = new JobInfo("name3", "descrip for name3", 1.0, 1.0,
                                3.33, 1.0,
                                1.0,1.0,1.0 );
          subtask3.setJobInfo(jobInfo);
      subtask4 = new SubtaskRequest("TaskID4");
          subtask4.setEarliestTimeToStart(new Date(300000L));
          subtask4.setDeadline(new Date(900000L));
          jobInfo = new JobInfo("name4", "descrip for name4", 1.0, 1.0,
                                4.44, 1.0,
                                1.0,1.0,1.0 );
          subtask4.setJobInfo(jobInfo);
      twp1 = new TaskWithPrecedence(new SubtaskRequest(subtask1));
//      twp1.getTaskData().setId("twp1");
      twp2 = new TaskWithPrecedence(new SubtaskRequest(subtask2));
//      twp2.getTaskData().setId("twp2");
      twp3 = new TaskWithPrecedence(new SubtaskRequest(subtask3));
//      twp3.getTaskData().setId("twp3");
      twp4 = new TaskWithPrecedence(new SubtaskRequest(subtask4));
//      twp4.getTaskData().setId("twp4");

    } catch (Exception e) {
      e.printStackTrace();  //dfk; see whats going on
      fail("Exception adding task");
    }
  }

  /** Add all tasks and subtasks into a (sortable) arraylist.
   *  @todo Someday, consider adding in random order....
   */
  private void setup2() {
    list1.add(subtask1);
    list1.add(subtask2);
    list1.add(subtask3);
    list1.add(subtask4);
    list1.add(twp4);
    list1.add(twp3);
    list1.add(twp2);
    list1.add(twp1);
  }

  private void addAllTasksToPlan() throws InvalidTaskException {
    setup1();
    plan1 = new TaskPlan();
    plan1.addTask(subtask1);
    plan1.addTask(subtask2);
    plan1.addTask(subtask3);
    plan1.addTask(subtask4);
  }

  /** Do some sorta "toString()" of a SubtaskRequest or TaskWithPrecedence. */
  private String printTask(Object obj) {
    SubtaskRequest t = null;
    String s = new String("");
    if (obj instanceof SubtaskRequest) {
      t = (SubtaskRequest)obj;
    } else {
      t = ((TaskWithPrecedence)obj).getTaskData();
    }
    s = t.toString();   // add more goodies???
    s+=", eStart="+t.getEarliestTimeToStart()
      +", dead="+t.getDeadline()
      +", desc="+t.getJobInfo().getDescription()
      +", id="+t.getId()
      ;
    return s;
  }


  /** Experiment w/ some of the default implementations of ancestor class {@link AbstractTableModel}. */
  public void testAbstractTableModelMethods() {
    try {
      addAllTasksToPlan();
      plan1.addSuccessor(subtask4, subtask2);
      model1 = new TPTableModel(plan1);
      System.out.println("model1.findColumn(\"Task ID\") = " + model1.findColumn("Task ID") );
      System.out.println("model1.getColumnClass(1) = " + model1.getColumnClass(1).getName() );
      System.out.println("model1.getColumnName(1) = " + model1.getColumnName(1) );
    } catch (Exception e) {
      e.printStackTrace();  //dfk; see whats going on
      fail("Exception adding task");
    }


  }

  /** Experiment w/ a {@link JTable} view. */
  public void testJTableView() {
    try {
      addAllTasksToPlan();
      plan1.addSuccessor(subtask4, subtask2);
      model1 = new TPTableModel(plan1);
      JFrame f = new JFrame("Quickie JTable test -- Close window to proceed");
      f.setDefaultCloseOperation(f.DISPOSE_ON_CLOSE);
//      f.addWindowListener(new WindowAdapter() {
//          public void windowClosing(WindowEvent e) {((java.awt.Window)e.getSource()).dispose(); return;}
//      });
      JTable t = new JTable(model1);
      f.getContentPane().add(new JScrollPane(t));
      f.pack();
      f.setSize(800,200);
      f.show();

      /*inf loop */for (int n=0;n<100000000;n++) {}


    } catch (Exception e) {
      e.printStackTrace();  //dfk; see whats going on
      fail("Exception adding task");
    }


  }

//  public void testSortingOneTask ()
//  {
//    setup1();
//    list1.add(subtask1);
//    Object[] list2 = list1.toArray();
//    assertEquals("Check list size", 1, list2.length);
//    assertSame("Check task lookup[0] pre-sort", subtask1, list2[0]);
//    Arrays.sort(list2, c1);
//    assertSame("Check task lookup[0] post-sort", subtask1, list2[0]);
//  }
//
//  public void testSortingStartTimes ()
//  {
//    setup1();
//    list1.add(subtask2);
//    list1.add(subtask3);
//    Object[] list2 = list1.toArray();
//    assertEquals("Check list size", 2, list2.length);
//    assertSame("Check task lookup[0] pre-sort", subtask2, list2[0]);
//    assertSame("Check task lookup[1] pre-sort", subtask3, list2[1]);
//    Arrays.sort(list2, c1);
//    assertSame("Check task lookup[0] post-sort", subtask3, list2[0]);
//    assertSame("Check task lookup[1] post-sort", subtask2, list2[1]);
//  }
//
//  public void testSortingDeadlines ()
//  {
//    setup1();
//    list1.add(subtask1);
//    list1.add(subtask2);
//    Object[] list2 = list1.toArray();
//    assertEquals("Check list size", 2, list2.length);
//    assertSame("Check task lookup[0] pre-sort", subtask1, list2[0]);
//    assertSame("Check task lookup[1] pre-sort", subtask2, list2[1]);
//    Arrays.sort(list2, c1);
//    assertSame("Check task lookup[0] post-sort", subtask2, list2[0]);
//    assertSame("Check task lookup[1] post-sort", subtask1, list2[1]);
//  }
//
//  public void testSortingDescrips ()
//  {
//    setup1();
//    list1.add(subtask4);
//    list1.add(subtask2);
//    Object[] list2 = list1.toArray();
//    assertEquals("Check list size", 2, list2.length);
//    assertSame("Check task lookup[0] pre-sort", subtask4, list2[0]);
//    assertSame("Check task lookup[1] pre-sort", subtask2, list2[1]);
//    Arrays.sort(list2, c1);
//    assertSame("Check task lookup[0] post-sort", subtask2, list2[0]);
//    assertSame("Check task lookup[1] post-sort", subtask4, list2[1]);
//  }
//
//  public void testStableSorting ()
//  {
//    setup1();
//    try {
//      // adjust some irrelevant property in each, then test stable after sort.
//      subtask1 = new SubtaskRequest("TaskID1");
//          subtask1.setEarliestTimeToStart(new Date(300000L));
//          subtask1.setDeadline(new Date(1000000L));
//          jobInfo = new JobInfo();
//          jobInfo.createFromSpec("name:1 expected-price:1");
//          subtask1.setJobInfo(jobInfo);
//      list1.add(subtask1);
//      subtask2 = new SubtaskRequest("TaskID1");
//          subtask2.setEarliestTimeToStart(new Date(300000L));
//          subtask2.setDeadline(new Date(1000000L));
//          jobInfo = new JobInfo();
//          jobInfo.createFromSpec("name:1 expected-price:2");
//          subtask2.setJobInfo(jobInfo);
//      list1.add(subtask2);
//      Object[] list2 = list1.toArray();
//      assertEquals("Check list size", 2, list2.length);
//      assertSame("Check task lookup[0] pre-sort", subtask1, list2[0]);
//      assertSame("Check task lookup[1] pre-sort", subtask2, list2[1]);
//      Arrays.sort(list2, c1);
//      assertSame("Check task lookup[0] post-sort", subtask1, list2[0]);
//      assertSame("Check task lookup[1] post-sort", subtask2, list2[1]);
//    } catch (Exception e) {
//      e.printStackTrace();
//      fail(e.toString());
//    }
//  }
//
//  public void testSortingDifferentTaskTypes ()
//  {
//    try {
//      setup1();
//      list1.add(subtask4);
//      list1.add(twp2);
//      Object[] list2 = list1.toArray();
//      assertEquals("Check list size", 2, list2.length);
//      assertSame("Check task lookup[0] pre-sort", subtask4, list2[0]);
//      assertSame("Check task lookup[1] pre-sort", twp2, list2[1]);
//      Arrays.sort(list2, c1);
//      assertSame("Check task lookup[0] post-sort", twp2, list2[0]);
//      assertSame("Check task lookup[1] post-sort", subtask4, list2[1]);
//    } catch (Exception e) {
//      fail("Exception adding task");
//    }
//  }
//
//  public void testBarfBiggieToConsoleWindow ()
//  {
//    try {
//      assert("Entering testBarfBiggieToConsoleWindow()", true);
//      setup1();
//      setup2();
//      Object[] list2 = list1.toArray();
//      System.out.println("Before sorting: list contains:");
//      for (int i=0; i < list2.length; i++) {
//        System.out.println("\t"+i+": "+printTask(list2[i]));
//      }
//      Arrays.sort(list2, c1);
//      System.out.println("After sorting: list contains:");
//      for (int i=0; i < list2.length; i++) {
//        System.out.println("\t"+i+": "+printTask(list2[i]));
//      }
//    } catch (Exception e) {
//      fail("Exception adding task");
//    }
//  }

//  // --------------------------------------------------------------------
//  // 3.0 Check handling of duplicate and non-existent tasks
//  public void testNonexistentTask ()
//  {
//    setup1();
//    assert("Should be no task named T3",
//	   (plan.getTaskData("T3") == null));
//  }
//
//  public void testDuplicate ()
//  {
//    setup1();
//    try {
//      // Try adding a second task called "T1"
//      plan.addTask(new SubtaskRequest("T1"));
//    }
//    catch (InvalidTaskException e) {
//      return;
//    }
//    fail("Expected duplicate task exception");
//  }
//
//  public void testBogusRemove ()
//  {
//    setup1();
//    try {
//      // Try removing a task called "T2" that is different from the "T2" in
//      // the plan
//      plan.removeTask(new SubtaskRequest("T2"));
//    }
//    catch (InvalidTaskException e) {
//      return;
//    }
//    fail("Expected exception removing invalid task");
//  }
//
//  // --------------------------------------------------------------------
//  // 4.0 Check out precedence relationships
//  private void setup2 ()
//  {
//    setup1();
//    // First, we make T2 be a successor of T1
//    try {
//      plan.addSuccessor("T1", "T2");
//    }
//    catch (Exception e) {
//      fail("Unexpected exception" + e);
//    }
//  }
//
//  public void testPrecedence ()
//  {
//    setup2();
//    // make sure T2 is the only successor
//    Vector vector = enumToVector(plan.getSuccessors("T1"));
//    assertEquals("Check number of successors", 1, vector.size());
//    assertEquals("Check identity of successor",
//		 subtask2,
//		 vector.elementAt(0));
//  }
//
//  public void testCircularDependency ()
//  {
//    setup2();
//    try {
//      // Try making T1 be a successor of T2
//      plan.addSuccessor("T2", "T1");
//    }
//    catch (CircularDependencyException e) {
//      return; // this is what we expect
//    }
//    catch (InvalidTaskException e) {
//      fail("Unexpected exception" + e);
//    }
//    fail("Failed to detect circular dependency");
//  }
//
//  // --------------------------------------------------------------------
//  // 5.0 Check out transitive precedence relationships
//  private void setup3 ()
//  {
//    setup2();
//    try {
//      plan.addTask(new SubtaskRequest("T3"));
//    }
//    catch (InvalidTaskException e) {
//      fail("Bogus InvalidTaskException");
//    }
//    try {
//      plan.addSuccessor("T1", "T3");
//    }
//    catch (CircularDependencyException e) {
//      fail("T3 should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      fail("T3 should not be an invalid task");
//    }
//  }
//
//  public void testMultiplePrecedence ()
//  {
//    setup3();
//    Iterator enum = plan.getSuccessors("T1");
//    Vector vector = new Vector();
//    while (enum.hasNext()) {
//      vector.addElement(((SubtaskRequest)enum.next()).getId());
//      }
//    assertEquals("Should be 2 succesors", 2, vector.size());
//    assert("T2 should be a successor", (vector.indexOf("T2") != -1));
//    assert("T3 should be a successor", (vector.indexOf("T3") != -1));
//  }
//
//  public void testIsInSuccessors ()
//  {
//    setup3();
//    assert("T2 is successor of T1",
//	   plan.isInSuccessors(subtask1, subtask2));
//    assert("T3 is successor of T1",
//	   plan.isInSuccessors("T1", "T3"));
//  }
//
//  public void testRootsAgain ()
//  {
//    setup3();
//    // Check that T1 is now the only root
//    Vector vector = enumToVector(plan.getSuccessorRoots());
//    assertEquals("Only 1 root", 1, vector.size());
//    assertEquals("T1 is only root",
//		 subtask1,
//		 (SubtaskRequest)vector.elementAt(0));
//  }
//
//  // Add 3 more successors to form a diamond and check results
//  private void setup4 ()
//  {
//    setup3();
//    // -- T4 --
//    try {
//      plan.addTask(new SubtaskRequest("T4"));
//    }
//    catch (InvalidTaskException e) {
//      fail("T4 should not be an invalid task");
//    }
//    try {
//      plan.addSuccessor("T2", "T4");
//    }
//    catch (CircularDependencyException e) {
//      fail("T2 -> T4 should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      fail("Unexpected " + e);
//    }
//    // -- T5 --
//    try {
//      plan.addTask(new SubtaskRequest("T5"));
//    }
//    catch (InvalidTaskException e) {
//      fail("T5 should not be an invalid task");
//    }
//    try {
//      plan.addSuccessor("T3", "T5");
//    }
//    catch (CircularDependencyException e) {
//      fail("T3 -> T5 should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      fail("Unexpected " + e);
//    }
//    // -- T6 --
//    try {
//      plan.addTask(new SubtaskRequest("T6"));
//    }
//    catch (InvalidTaskException e) {
//      fail("T6 should not be an invalid task");
//    }
//    try {
//      plan.addSuccessor("T4", "T6");
//    }
//    catch (CircularDependencyException e) {
//      fail("T4 -> T6 should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      fail("Unexpected " + e);
//    }
//    try {
//      plan.addSuccessor("T3", "T6");
//    }
//    catch (CircularDependencyException e) {
//      fail("T3 -> T6 should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      fail("Unexpected " + e);
//    }
//    // We now have
//    //   2-4-6
//    //  /   /
//    // 1   /
//    //  \ /
//    //   3-5
//  }
//
//  public void testSuccessors2 ()
//  {
//    setup4();
//    // Make sure T1 still has 2 successors, and now has 5 in allSuccessors
//    Vector vector = enumToVector(plan.getSuccessors("T1"));
//    assertEquals("T1 has 2 successors",
//		 2, vector.size());
//
//    vector = enumToVector(plan.getAllSuccessors(subtask1));
//    assertEquals("T1 has 5 tasks in allSuccessors",
//		 5, vector.size());
//    assert("T6 is in transitive closure of successors of T1",
//	   plan.isInSuccessors("T1", "T6"));
//  }
//
//    // --------------------------------------------------------------------
//    // 6.0 Check out bogus task error handling
//  public void testBogusTask1 ()
//  {
//    setup4();
//    // Create a bogus task, try putting it before and after good tasks
//    SubtaskRequest bogusTask = new SubtaskRequest("bogus");
//    try {
//      plan.addSuccessor("bogus", "T6");
//    }
//    catch (CircularDependencyException e) {
//      fail("this should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      return; // this is what we expect
//    }
//    fail("Failed to throw InvalidTaskException");
//  }
//
//  public void testBogusTask2 ()
//  {
//    setup4();
//    SubtaskRequest bogusTask = new SubtaskRequest("bogus");
//    try {
//      plan.addSuccessor("T3", "bogus");
//    }
//    catch (CircularDependencyException e) {
//      fail("this should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      return; // this is what we expect
//    }
//    fail("Failed to throw InvalidTaskException");
//  }
//
//  public void testBogusTask3 ()
//  {
//    setup4();
//    SubtaskRequest bogusTask = new SubtaskRequest("bogus");
//    try {
//      plan.addSuccessor(plan.getTaskData("T3"), bogusTask);
//    }
//    catch (CircularDependencyException e) {
//      fail("this should not cause circular dependency");
//    }
//    catch (InvalidTaskException e) {
//      return; // this is what we expect
//    }
//    fail("Failed to throw InvalidTaskException");
//  }
//
//  private Vector enumToVector (Iterator input)
//  {
//    Vector result = new Vector();
//    while (input.hasNext()) {
//      result.addElement(input.next());
//    }
//    return result;
//  }
//
//  // ------------ Static TaskPlan generator for other tests ------------
//
//  private static final long s_msph = 3600000l;
//
//  public static TaskPlan generateTaskPlan ()
//  {
//    TaskPlan plan = new TaskPlan();
//    // populate the plan with 6 tasks
//    try {
//      plan.addTask(new SubtaskRequest("T1"));
//      plan.addTask(new SubtaskRequest("T2"));
//      plan.addSuccessor("T1", "T2");
//      plan.addTask(new SubtaskRequest("T3"));
//      plan.addSuccessor("T1", "T3");
//      plan.addTask(new SubtaskRequest("T4"));
//      plan.addSuccessor("T2", "T4");
//      plan.addTask(new SubtaskRequest("T5"));
//      plan.addSuccessor("T3", "T5");
//      plan.addTask(new SubtaskRequest("T6"));
//      plan.addSuccessor("T4", "T6");
//      plan.addSuccessor("T3", "T6");
//    }
//    catch (Exception e) {
//      fail("Exception creating plan: " + e);
//    }
//    // We now have
//    //   2-4-6
//    //  /   /
//    // 1   /
//    //  \ /
//    //   3-5
//    //System.out.println("Subtask Tree:");
//    //printSuccessorTree(plan, plan.getSuccessorRoots(), 1);
//    // put timings on the tasks
//    long baseTime = (new Date().getTime()) + s_msph * 12;
//    try {
//      plan.getTaskData("T1").setEarliestTimeToStart(new Date(baseTime));
//      plan.getTaskData("T1").setDeadline(new Date(baseTime + s_msph * 175));
//      plan.getTaskData("T2").setEarliestTimeToStart
//	                       (new Date(baseTime + s_msph * 75));
//      plan.getTaskData("T2").setDeadline(new Date(baseTime + s_msph * 250));
//      plan.getTaskData("T3").setEarliestTimeToStart
//	                       (new Date(baseTime + s_msph * 75));
//      plan.getTaskData("T3").setDeadline(new Date(baseTime + s_msph * 325));
//      plan.getTaskData("T4").setEarliestTimeToStart
//	                       (new Date(baseTime + s_msph * 150));
//      plan.getTaskData("T4").setDeadline(new Date(baseTime + s_msph * 325));
//      plan.getTaskData("T5").setEarliestTimeToStart
//	                       (new Date(baseTime + s_msph * 150));
//      plan.getTaskData("T5").setDeadline(new Date(baseTime + s_msph * 400));
//      plan.getTaskData("T6").setEarliestTimeToStart
//	                       (new Date(baseTime + s_msph * 225));
//      plan.getTaskData("T6").setDeadline(new Date(baseTime + s_msph * 400));
//    }
//    catch (Exception e) {
//      fail("Exception setting task timings: " + e);
//    }
//    return plan;
//  }
}
