/*------------------------------------------------------------------------
 * MAGNET utility components
 * $Id: Cons.java,v 1.1.2.1 2002/05/05 19:42:04 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Nov 19 13:51:10 2000  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.util;

/**
 * This is your basic cons cell.
 * @author John Collins
 */
public class Cons
{
  private Object m_car = null;
  private Object m_cdr = null;

  /**
   * You can create one of these with a null cdr like this
   */
  public Cons (Object car)
  {
    super();
    m_car = car;
  }

  /**
   * Here's how you usually make a cons.  If the cdr is a cons, then
   * the result is a list.  Otherwise it's a dotted pair.
   */
  public Cons (Object car, Object cdr)
  {
    super();
    m_car = car;
    m_cdr = cdr;
  }

  /**
   * car is a synonym for "first"
   */
  public Object car ()
  {
    return m_car;
  }

  /**
   * cdr is a synonym for "rest", that is, all but the first
   */
  public Object cdr ()
  {
    return m_cdr;
  }

  /**
   * Cons an object on the front.  Result is a new cons, with the
   * given object as its car and this cons as its cdr.
   */
  public Cons cons (Object car)
  {
    return new Cons(car, this);
  }

  /**
   * Returns true just in case thing matches the car of some tail of
   * list.
   */
  public boolean contains (Object thing)
  {
    if (m_car.equals(thing))
      return true;
    else if (m_cdr != null && m_cdr instanceof Cons)
      return ((Cons)m_cdr).contains(thing);
    else
      return false;
  }

  /**
   * Returns the length of a list.  Note that the length of a dotted
   * pair is 1.  Note that we don't call it "length" because that's
   * used for array size in Java.  Here we stick with java standards
   * and call it size.
   */
  public int size ()
  {
    if (m_cdr == null)
      return 1;
    else if (m_cdr instanceof Cons)
      return (1 + ((Cons)m_cdr).size());
    else
      return 1;
  }

  /**
   * Make these printable.  Not pretty-printed, unfortunately...
   */
  public String toString ()
  {
    return ("(" + toStringNoParens() + ")");
  }

  private String toStringNoParens ()
  {
    if (m_cdr == null)
      return m_car.toString();
    else if (m_cdr instanceof Cons)
      return (m_car.toString() + " " + ((Cons)m_cdr).toStringNoParens());
    else // dotted pair
      return (m_car.toString() + " . " + m_cdr.toString());
  }
}
