/*------------------------------------------------------------------------
 * Magnet utility components
 * $Id: MagnetProperties.java,v 1.2.2.1 2002/05/05 19:42:04 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Fri Apr 20 23:23:23 2001  John Collins
 *   Initial Entry, intended replacement for PropertyFile
 */

package edu.umn.magnet.util;

import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * Use one of these to read a set of property = value pairs from a
 * file and add them to the Java system properties.
 * @author John Collins
 */
public class MagnetProperties
{
  private static final int NAME = 1;
  private static final int EQ = 2;
  private static final int VALUE = 3;

  private static Properties props = new Properties();

  public static void readProps (String filename)
  {
    FileInputStream str = null;
    // open the file
    try {
      str = new FileInputStream(filename);
    }
    catch (IOException ioe) {
      System.out.println("Can't open property file " + filename);
      ioe.printStackTrace();
      return;
    }

    // read the properties
    try {
      props.load(str);
    }
    catch (IOException ioe) {
      System.out.println("Can't read property file " + filename);
      ioe.printStackTrace();
      return;
    }
  }

  /**
   * Returns an individual property value, given a key
   */
  public static String getProperty (String key)
  {
    return props.getProperty(key);
  }

  /**
   * Returns an individual property value, given a key
   */
  public static String getProperty (String key, String df)
  {
    String result = props.getProperty(key);
    if (result != null)
      return result;
    else
      return df;
  }

  /**
   * Sets an individual property, augmenting or overriding the
   * contents of the properties file.
   */
  public static void setProperty (String key, String value)
  {
    props.setProperty(key, value);
  }

  /**
   * Returns the whole Properties object
   */
  public static Properties getProperties ()
  {
    return props;
  }
}
