/*------------------------------------------------------------------------
 * Magnet utility components
 * $Id: PropertyFile.java,v 1.1.2.2 2002/05/05 19:42:04 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * Sun Jan  3 17:12:23 1999  John E. Collins
 *   Don't use Log.logln for output -- it prevents you from using the
 *   properties file to set the log output stream.
 * 
 * Sun Jan  3 12:02:26 1999  John E. Collins
 *   Change comment character to #
 * 
 * Fri Jan  1 23:01:04 1999  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.util;

import java.util.Properties;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.StreamTokenizer;
import java.io.IOException;

/**
 * Use one of these to read a set of property = value pairs from a
 * file and add them to the Java system properties.
 * @author John Collins
 * @deprecated This needs to be replaced by MagnetProperties, with std
 *      property file serialization.
 */
public class PropertyFile
{
  private static final int NAME = 1;
  private static final int EQ = 2;
  private static final int VALUE = 3;

  public static void readProps (String filename)
  {
    StreamTokenizer st = null;
    try
    {
      st = new StreamTokenizer
	    (new BufferedReader
	       (new FileReader(filename)));
    }
    catch (IOException ioe)
    {
      System.out.println("Can't open property file " + filename);
      ioe.printStackTrace();
      return;
    }
    st.commentChar('#');
    st.wordChars(0x41, 0x7e);
    boolean more = true;
    int state = NAME;
    int tt = 0;
    String name = null;
    while (more)
    {
      try
      {
	tt = st.nextToken();
      }
      catch (IOException ioe)
      {
	System.out.println("Error: IO Error in PropertyFile");
	break;
      }
      if (tt == StreamTokenizer.TT_EOF)
	more = false;
      else if (state == NAME)
      { // we only use words in this state
	if (tt == StreamTokenizer.TT_WORD)
	{
	  name = st.sval;
	  System.out.print("Property name = " + name);
	  state = EQ;
	}
	else
	  System.out.println("Error: PropertyFile syntax error (NAME) " + tt);
      }
      else if (state == EQ)
      { // if we don't see the "=" here, we bump back to NAME
	if (tt == (int)'=')
	  state = VALUE;
	else
	{
	  System.out.println("Error: PropertyFile syntax error (=) " + tt
		    + (int)'=');
	  state = NAME;
	}
      }
      else if (state == VALUE)
      { // expect a value here, which may be a string, a word, a
	// number, or a token.  Everything must be converted to
	// strings before filling out the properties.
	String value = null;
	if (tt == StreamTokenizer.TT_WORD
	    || tt == (int)'"')
	  value = st.sval;
	else if (tt == StreamTokenizer.TT_NUMBER)
	  value = String.valueOf(st.nval);
	else // we've got a single character...
	  value = String.valueOf((char)tt);
	System.out.println(", value = " + value);
	System.getProperties().put(name, value);
	state = NAME;
      }
    }
  }
}
