/*------------------------------------------------------------------------
 * Magnet utility components
 * $Id: ExponentialVariable.java,v 1.1.2.1 2002/05/05 19:42:04 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Sep 23 08:24:31 1998  John E. Collins
 *   Initial Entry, copied from jec.util
 */

package edu.umn.magnet.util.rvars;

import java.util.Random;

// ==================================================================

/**
   Produce random values with a exponential distribution, having a
   specified mean.
*/
public class ExponentialVariable extends RandomVariable 
{
  protected double m_mean = 1.0;
  
  /**
     Create a new exponential variable with a mean of 1.0.
  */
  public ExponentialVariable () {
    super();
  }
  
  /**
     Create a new exponential variable with the specified mean.
  */
  public ExponentialVariable (RandomVariableFactory factory,
			      double mean) {
    super(factory);
    m_mean = mean;
  }
  
  /**
     Return a string naming this variable type
  */
  public String getLabel () {
    return "Exponential";
  }
  
  /**
     Return the value of the mean of this variable.
  */
  public double getMean () {
    return m_mean;
  }
  
  /**
     Set the value of the mean of this variable.
  */
  public double setMean (double newMean) {
     return (m_mean = newMean);
  }
  
  /**
     Return the next value of this variable as a double.
  */
  public double nextDouble () {
    return -m_mean * Math.log(m_factory.nextDouble());
  }
  
  /**
     Return the next value of this variable as a float.
  */
  public float nextFloat () {
    return (float)this.nextDouble();
  }
  
  /**
     Return the next value of this variable as an integer.
  */
  public int nextInt () {
    return (int)Math.round(this.nextDouble());
  }
  
  /**
     Return the next value of this variable as a long.
  */
  public long nextLong () {
    return (long)Math.round(this.nextDouble());
  }
}
