/*------------------------------------------------------------------------
 * Magnet utility components
 * $Id: MErlangVariable.java,v 1.1.2.1 2002/05/05 19:42:04 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Fri Sep 25 20:52:51 1998  John E. Collins
 *   Correct errors
 * 
 * Wed Sep 23 08:19:58 1998  John E. Collins
 *   Initial Entry, copied from jec.util
 */

package edu.umn.magnet.util.rvars;

// ==================================================================

/**
   Produce random values with a m-erlang distribution. Shape and mean
   are specifiable.
*/
public class MErlangVariable extends ExponentialVariable 
{
  private int m_shape = 2;
  
  /**
     Return a new 2-erlang variable with a mean of 1.0.
  */
  public MErlangVariable () {
    super();
  }

  /**
     Return a new m-erlang variable with a mean of 1.0 and where m is
     equal to the specified shape parameter.
  */
  public MErlangVariable (int shape) {

    super();
    this.m_shape = shape;
  }

  /**
     Return a new m-erlang variable with specified mean and shape
  */
  public MErlangVariable (double mean, int shape) {

    super();
    this.m_mean = mean;
    this.m_shape = shape;
  }

  /**
     Return a new m-erlang variable with specified mean and shape
  */
  public MErlangVariable (RandomVariableFactory factory,
			  double mean, int shape)
    {
      super(factory, mean);
      m_shape = shape;
    }

  /**
   * Return a string naming this variable type
   */
  public String getLabel () {
    return "m-Erlang";
  }
  
  /**
     Return the shape parameter of this variable.
  */
  public int getShape () {
    return m_shape;
  }

  /**
     Set the shape parameter of this variable.
  */
  public int setShape (int newShape) {
    return m_shape = newShape;
  }

  /**
     Return the next value of this variable as a double.
  */
  public double nextDouble () {
    double rv = 0.0;
    
    for (int i = 0; i < m_shape; i++)
      rv = rv + super.nextDouble();
    
    return (m_mean / m_shape) * rv;
  }

  /**
     Return the next value of this variable as a float.
  */
  public float nextFloat () {
    return (float)this.nextDouble();
  }

  /**
     Return the next value of this variable as an int.
  */
  public int nextInt () {
    return (int)Math.round(this.nextDouble());
  }

  /**
     Return the next value of this variable as a long.
  */
  public long nextLong () {
    return (long)Math.round(this.nextDouble());
  }
}

