/*------------------------------------------------------------------------
 * Magnet utility components
 * $Id: UniformVariable.java,v 1.1.2.1 2002/05/05 19:42:04 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Sep 23 08:07:44 1998  John E. Collins
 *   Initial Entry, copied from jec.util
 */

package edu.umn.magnet.util.rvars;

import java.util.Random;

// ==================================================================

/**
   Produce random values with a uniform distribution, over a
   specified range.
*/
public class UniformVariable extends RandomVariable 
{
  private double m_offset = 0.0;
  private double m_range = 1.0;
  
  /**
     Create a new UniformVariable instance with offset of 0.0 and range
     of 1.0.
  */
  public UniformVariable () {
    super();
  }

  /**
     Create a new UniformVariable instance with the specified offset and
     range values.
  */
  public UniformVariable (RandomVariableFactory factory,
			  double offset, double range) {
    super(factory);
    m_offset = offset;
    m_range = range;
  }

  /**
   * Return a string naming this variable type
   */
  public String getLabel () {
    return "Uniform";
  }
  
  /**
     Return the current offset.
  */
  public double getOffset () {
    return m_offset;
  }
  
  /**
     Return the current range over which values will be spread uniformly.
  */
  public double getRange () {
    return m_range;
  }
  
  /**
     Change the value of the offset. The new value is returned.
  */
  public double setOffset (double newOffset) {
    return (m_offset = newOffset);
  }
  
  /**
     Change the value of the range. The new value is returned.
  */
  public double setRange (double newRange) {
    return (m_range = newRange);
  }
  
  /**
     Return a random value as a double between offset and offset+range.
  */
  public double nextDouble () {
    return (m_factory.nextDouble() * m_range) + m_offset;
  }
  
  /**
     Return a float value randomly distributed between offset and offset+range.
  */
  public float nextFloat () {
    return (float)this.nextDouble();
  }
  
  /**
     Return an integer randomly distributed between offset and offset+range.
  */
  public int nextInt () {
    return (int)Math.round(this.nextDouble());
  }
  
  /**
     Return a long randomly distributed between offset and offset+range.
  */
  public long nextLong () {
    return (long)Math.round(this.nextDouble());
  }
}
