/*------------------------------------------------------------------------
 * Magnet Utilities
 * $Id: HistogramBin.java,v 1.2.2.2 2002/05/05 19:42:06 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Thu Aug 27 22:01:22 1998  John E. Collins
 *   Initial entry, copied from jec.util
 */

package edu.umn.magnet.util.stats;

import java.util.Hashtable;

/**
 * Represents a bin in a histogram. A bin has a base value, a range,
 * and a count. The count represents the number of observations that
 * have fallen between the base value and base value + range of a bin.
 * @author John Collins
 */
public class HistogramBin {

    private double base_;
    private double size_;
    private int count_;

    /**
     * Constructs a bin with the specified base value and width.
     */
    public HistogramBin(double base, double size) {
        super();
        base_ = base;
        size_ = size;
        count_ = 0;
    }

    /**
     * Returns the base (or offset) value for a bin
     */
    public double getBaseValue() {
        return base_;
    }

    /**
     * Returns the width of a bin
     */
    public double getBinRange() {
        return size_;
    }

    /**
     * Returns the number of observations that have fallen within the
     * range of this bin.
     */
    public int getBinCount() {
        return count_;
    }

    /**
     * Increments the observation count for this bin.
     */
    public void newEntry() {
        count_++;
    }

}

