/*---------------------------------------------------------------------
 * Title:        Magnet Server
 * Version:      $Id: AbsoluteDelay.java,v 1.1 2001/04/09 15:58:25 jcollins Exp $
 * Copyright:    Copyright (c) 1998
 * Author:       John Collins
 * Company:      University of Minnesota
 * Description:  Magnet server component: timers
 *---------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Apr  8 21:34:30 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.util.time;

/**
 * You use one of these to make something happen in the future.
 * @author John Collins
 */
public class AbsoluteDelay extends AbstractTimer
{
  long m_delay;

  /**
   * Creates and starts a new Timer. At the end of the delay period, the
   * <tt>action()</tt> method of the TimerAction object will be invoked.
   * Note that <tt>delay</tt> is actually the minimum time before which
   * <tt>action</tt> will take place. The actual time is dependent on the
   * Java scheduler and other thread activity.
   * @param  delay   the period in milliseconds before invoking
   *     <tt>action</tt>.  This value is scaled by the current
   *     AgentTime clock rate.
   * @param action   a TimerAction instance whose <tt>action()</tt> method
   *     is invoked at the end of the delay period.
   */
  public AbsoluteDelay(long delay, TimerAction action)
  {
    super();
    m_delay = delay;
    this.setAction(action);
    this.start();
  }

  /**
   * Delays by the system clock, not the AgentTime clock.
   */
  protected void delay () throws InterruptedException
  {
    Thread.sleep(m_delay);
  }
}
