/*---------------------------------------------------------------------
 * Title:        Magnet Server
 * Version:      $Id: AbstractTimer.java,v 1.1 2001/04/09 15:58:25 jcollins Exp $
 * Copyright:    Copyright (c) 1998
 * Author:       John Collins
 * Company:      University of Minnesota
 * Description:  Magnet server component: timers
 *---------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Apr  8 21:25:42 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.util.time;

/**
 * You use an instance of a subclass of this class to make something
 * happen in the future.  What happens is that the <tt>action()</tt>
 * method of a <tt>TimerAction</tt> object gets called.
 * @author John Collins
 */
public abstract class AbstractTimer extends Thread
{
  TimerAction m_action;

  /**
   * Subclasses call this to set the action.
   */
  protected void setAction (TimerAction action)
  {
    m_action = action;
  }
  
  /**
   * When you create one of these, it runs out its time and then
   * triggers its action.  The actual delay is handled by the
   * delay method, which must be overridden in subclasses.
   */
  public void run ()
  {
    try {
      this.delay();
    }
    catch (InterruptedException e) {
      return;
    }
    m_action.action();
  }

  /**
   * Delays according to the chosen subclass method
   */
  abstract protected void delay () throws InterruptedException;
}
