/*---------------------------------------------------------------------
 * Title:        Magnet Server
 * Version:      $Id: Delay.java,v 1.1 2001/04/09 15:58:26 jcollins Exp $
 * Copyright:    Copyright (c) 1998
 * Author:       John Collins
 * Company:      University of Minnesota
 * Description:  Magnet server component: timers
 *---------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Apr  8 21:30:12 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.util.time;

/**
 * You use one of these to make something happen after a specific
 * interval.  The interval is scaled according to the current scale
 * factor given by <tt>AgentTime.getRate()</tt>.
 * @author John Collins
 */
public class Delay extends AbstractTimer
{
  long m_delay;

  /**
   * Creates and starts a new Timer. At the end of the delay period, the
   * <tt>action()</tt> method of the TimerAction object will be invoked.
   * Note that <tt>delay</tt> is actually the minimum time before which
   * <tt>action</tt> will take place. The actual time is dependent on the
   * Java scheduler and other thread activity.
   * @param  delay   the period in milliseconds before invoking
   *     <tt>action</tt>.  This value is scaled by the current
   *     AgentTime clock rate.
   * @param action   a TimerAction instance whose <tt>action()</tt> method
   *     is invoked at the end of the delay period.
   */
  public Delay(long delay, TimerAction action)
  {
    super();
    m_delay = delay;
    this.setAction(action);
    this.start();
  }

  /**
   * Delays an amount of time given by the delay, scaled by the
   * current clock rate given by AgentTime.
   */
  protected void delay () throws InterruptedException
  {
    AgentTime.waitFor(m_delay);
  }
}
