/*------------------------------------------------------------------------
 * MAGNET unit test
 * $Id: SearchQueueTest.java,v 1.1.2.1 2002/05/05 19:42:06 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Tue Sep  4 18:21:35 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.util.search;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * This is a tester for edu.umn.magnet.client.Something
 * @author John Collins
 */
public class SearchQueueTest extends TestCase
{
  static final double SLOP = 0.000001;
  
  public SearchQueueTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(SearchQueueTest.class);
  }

  // ------------ data and setups --------------
  private SearchQueue sq = null;

  private void setup1 ()
  {
    sq = SearchQueue.create(SearchQueue.DEPTH_FIRST);
  }
  
  public void testInit ()
  {
    setup1();
    assertEquals("Correct nodeCount", 0, sq.nodeCount());
    assertEquals("Correct size", 0, sq.size());
    assert("Correct default first value",
	   (Math.abs(0.0 - sq.firstValue()) < SLOP));
    assert("Correct default last value",
	   (Math.abs(0.0 - sq.lastValue()) < SLOP));
    assert("Initially empty", sq.isEmpty());
  }

  // --------------- Depth first ---------------

  public void testDepthFirstA ()
  {
    setup1();
    Node n1 = new Node(100.0);
    sq.insert(n1);
    assertEquals("Correct count 1", 1, sq.size());
    assertEquals("Correct first element", n1, sq.select());
    assertEquals("Correct extract", n1, sq.extract());
    assertEquals("Correct count 2", 0, sq.size());
    assert("Finally empty", sq.isEmpty());
  }

  public void testDepthFirstB ()
  {
    setup1();
    Node n1 = new Node(100.0);
    sq.insert(n1);
    Node n2 = new Node(200.0);
    sq.insert(n2);
    assertEquals("Correct count 1", 2, sq.size());
    assertEquals("Correct first element", n2, sq.select());
    assertEquals("Correct extract", n2, sq.extract());
    assertEquals("Correct count 2", 1, sq.size());
    assertEquals("Correct first element b", n1, sq.select());
    assertEquals("Correct extract", n1, sq.extract());
    assertEquals("Correct count 3", 0, sq.size());
    assert("Finally empty", sq.isEmpty());
  }

  public void testDepthFirstMax ()
  // No, you probably don't want to do this at home...
  {
    setup1();
    sq.setMaxSize(10);
    assertEquals("Correct maxSize", 10, sq.getMaxSize());

    for (int i = 0; i < 5; i++) {
      sq.insert(new Node((double)(i + 1)));
    } // end of for ()
    assertEquals("Correct size a", 5, sq.size());
    assert("Correct first value a",
	   (Math.abs(5.0 - sq.select().value()) < SLOP));

    for (int i = 0; i < 5; i++) {
      sq.insert(new Node((double)(i + 6)));
    } // end of for ()
    assertEquals("Correct size b", 10, sq.size());
    assert("Correct first value b",
	   (Math.abs(10.0 - sq.select().value()) < SLOP));

    // now we create the overflow - add two nodes to overcome the tolerance
    sq.insert(new Node(11.0));
    sq.insert(new Node(12.0));
    assertEquals("Correct size c", 10, sq.size());
    assert("Correct first value c",
	   (Math.abs(12.0 - sq.select().value()) < SLOP));
    assert("Correct firstValue",
	   (Math.abs(12.0 - sq.firstValue()) < SLOP));
    assert("Correct lastValue",
	   (Math.abs(3.0 - sq.lastValue()) < SLOP));
  }
  
  // --------------- Breadth first --------------

  private void setup2 ()
  {
    sq = SearchQueue.create(SearchQueue.BREADTH_FIRST);
  }

  public void testBreadthFirstA ()
  {
    setup2();
    Node n1 = new Node(100.0);
    sq.insert(n1);
    assertEquals("Correct count 1", 1, sq.size());
    assertEquals("Correct first element", n1, sq.select());
    assertEquals("Correct extract", n1, sq.extract());
    assertEquals("Correct count 2", 0, sq.size());
    assert("Finally empty", sq.isEmpty());
  }

  public void testBreadthFirstB ()
  {
    setup2();
    Node n1 = new Node(100.0);
    sq.insert(n1);
    Node n2 = new Node(200.0);
    sq.insert(n2);
    assertEquals("Correct count 1", 2, sq.size());
    assertEquals("Correct first element", n1, sq.select());
    assertEquals("Correct extract", n1, sq.extract());
    assertEquals("Correct count 2", 1, sq.size());
    assertEquals("Correct first element b", n2, sq.select());
    assertEquals("Correct extract", n2, sq.extract());
    assertEquals("Correct count 3", 0, sq.size());
    assert("Finally empty", sq.isEmpty());
  }

  public void testBreadthFirstMax ()
  // No, you probably don't want to do this at home...
  {
    setup2();
    sq.setMaxSize(10);
    assertEquals("Correct maxSize", 10, sq.getMaxSize());

    for (int i = 0; i < 5; i++) {
      sq.insert(new Node((double)(i + 1)));
    } // end of for ()
    assertEquals("Correct size a", 5, sq.size());
    assert("Correct first value a",
	   (Math.abs(1.0 - sq.select().value()) < SLOP));

    for (int i = 0; i < 5; i++) {
      sq.insert(new Node((double)(i + 6)));
    } // end of for ()
    assertEquals("Correct size b", 10, sq.size());
    assert("Correct first value b",
	   (Math.abs(1.0 - sq.select().value()) < SLOP));

    // now we create the overflow - add two nodes to overcome the tolerance
    sq.insert(new Node(11.0));
    sq.insert(new Node(12.0));
    assertEquals("Correct size c", 10, sq.size());
    assert("Correct first value c",
	   (Math.abs(1.0 - sq.select().value()) < SLOP));
    assert("Correct firstValue",
	   (Math.abs(1.0 - sq.firstValue()) < SLOP));
    assert("Correct lastValue",
	   (Math.abs(10.0 - sq.lastValue()) < SLOP));
  }

  // ----------------- local SearchNode class -------------------
  class Node implements SearchNode
  {
    private double value = 0.0;

    protected Node (double val)
    {
      super();
      value = val;
    }

    public double value ()
    {
      return value;
    }
  }
}
