;;; Process the files that result from grep FinalEval x.log into
;;; spreadsheet input files.
;;;
;;; $Id: result-munger.el,v 1.2 2001/10/31 14:50:40 jcollins Exp $

;;; The result of process-file is that each line corresponds to one
;;; plan.  The format is <solved, cost, total-duration, node-count>,
;;; suitable for loading into a spreadsheet.
;;; If solved is 0, then cost and total-duration are not meaningful.

(defun do-abandoned ()
  "Process the <plan abandoned> lines"
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (while (search-forward "plan abandoned" nil t)
      (let ((end (point)))
	(beginning-of-line)
	(delete-region (point) end))
      (insert-string "0, 0, 0")
      (forward-char 2)
      (delete-char 4)
      )))

(defun do-cost ()
  "Process non-abandoned lines"
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (while (search-forward "cost =" nil t)
      (let ((end (point)))
	(beginning-of-line)
	(delete-region (point) end))
      (insert-string "1,"))
    (goto-char (point-min))
    (replace-string " compl = " " ")
    (goto-char (point-min))
    (replace-string " nc = " " ")
    ))

(defun eliminate (string)
  (search-forward string nil t)
  (replace-match "" nil t)
  )

(defun replace-with (old-string new-string)
  (search-forward old-string nil t)
  (replace-match new-string nil t)
  )

(defun eliminate-eol ()
  (end-of-line)
  (insert-string ",")
  (kill-line)
  )

(defun eliminate-to (string)
  (let ((start (point)))
    (search-forward string nil t)
    (delete-region start (point))
    ))

(defun do-head ()
  "Process test header ->
    taskCount, fanIn, depth, nbids, bidSize, ccTime, fcTime"
  (interactive)
  (eliminate-to "TaskCount: ")
  (eliminate " FanIn: ")
  (eliminate " Depth: ")
  (eliminate-eol)
  (eliminate "BidCount: ")
  (eliminate " avgSize: ")
  (eliminate-eol)
  (eliminate "Coverage Constraints: time = ")
  (eliminate-eol)
  (if (looking-at "No coverage")
      (insert-string "0,")
    (progn
      (eliminate "Feasibility Constraints: time = ")
      (eliminate-eol))))

(defun do-SA ()
  "Process SA data ->
    nodeCount, fcNodes, resultEval, time"
  (interactive)
  (eliminate-to "Node Count = ")
  (eliminate " Feasible and Covered Nodes : ")
  (if (looking-at "0")
      (progn
	(eliminate-eol)
	(insert-string "0,0,0,")
	(eliminate-to "time = "))
    (progn
      (eliminate-eol)
      (insert-string "1,")
      (eliminate-to "SA@")
      (replace-with ")" ",")
      (eliminate-to "Overall=")
      (search-forward "," nil t)
      (eliminate-to "time = "))))

(defun do-IP ()
  "Process IPcon data ->
    eqnCount, resultEval, time"
  (interactive)
  (eliminate-to "eqn count = ")
  (eliminate-eol)
  (eliminate-to "Result = ")
  (if (looking-at "failed")
      (insert-string "0,0,")
    (progn
      (insert-string "1,")
      ;;(eliminate-to "IPCon@")
      ;;(replace-with ")" ",")
      (eliminate-to "Overall=")
      (search-forward "," nil t)))
  (eliminate-to "time = ")
  (end-of-line))

(defun do-oldIP ()
  "Process IP data ->
    eqnCount, resultEval, time"
  (interactive)
  (eliminate-to "eqn count = ")
  (eliminate-eol)
  (eliminate-to "Result = ")
  (if (looking-at "failed")
      (insert-string "0,0,")
    (progn
      (insert-string "1,")
      ;;(eliminate-to "IP@")
      ;;(replace-with ")" ",")
      (eliminate-to "Overall=")
      (search-forward "," nil t)))
  (eliminate-to "time = ")
  (end-of-line))

(defun process-file ()
  "do everything"
  (interactive)
  (while (looking-at "----")
    (progn
      (do-head)
      ;;(end-of-line)
      ;;(eliminate-eol)
      (if (looking-at "No coverage")
	  (progn
	    (insert-string "0")
	    (kill-line)
	    (forward-char))
	(progn
	  (insert-string "1,")
	  (do-SA)
	  (eliminate-eol)
	  (do-IP)
	  (forward-char)
	  (if (looking-at "** ")
	      ;; we have an IP row here
	      (progn
		(backward-char)
		(eliminate-eol)
		(do-oldIP)
		(forward-char))
	    ))))))

(defun process-mas-buffer ()
  "A file from the mas_experiments_2001 set consists of a brief
header, a record from an IPCon search, and some number of SA search
results.  The only data we really care about is the time, node count,
and eval result."
  (interactive)
  (while (looking-at "----")
    ;; process the single IPCon result
    (eliminate-to "search type = ")
    (end-of-line) (insert-string " ")
    (eliminate-to "time = ")
    (end-of-line) (insert-string " ")
    (eliminate-to "eqn count = ")
    (end-of-line) (insert-string " ")
    ;; at this point the next line can be "Result = failed" or
    ;; "Result = TBP..."
    (eliminate-to "Result = ")
    (if (looking-at "TBP")
	(progn 
	  ;; we have a successful result
	  (eliminate-to "Overall=")
	  (search-forward "," nil t)
	  (backward-char) (insert-string " ")
	  (eliminate-to "time = ")
	  (end-of-line) (forward-char)

	  ;; process SA results until we run out of them
	  (while (looking-at "** ")
	    (eliminate-to "search type = ")
	    (end-of-line) ;(insert-string " ")
	    ;; expect 3 "tolerance" results, although some may be missing
	    (kill-line 1)
	    (if (looking-at "SM: ")
		(do-tolerance-line)
	      (insert-string " 0 0"))
	    (if (looking-at "SM: ")
		(do-tolerance-line)
	      (insert-string " 0 0"))
	    (if (looking-at "SM: ")
		(do-tolerance-line)
	      (insert-string " 0 0"))
	    (eliminate-to "Result = ")
	    (if (looking-at "TBP")
		(eliminate-to "Overall="))
	    (insert-string " ")
	    (search-forward ",")
	    (backward-char)
	    (kill-line) (forward-char))

	  ;; Process the MLE data
	  (if (looking-at "Weibull ")
	      (progn
		(eliminate-to "Weibull ")
		(forward-word 1)
		(eliminate-to "Parameters:")
		(dotimes (i 6)
		  (search-forward ",") (delete-char -1))
		(delete-char 1))))
      (progn
	;; Deal with IP search failure - there's no feasible solution
	(search-forward "failed")
	(eliminate-to "time =")
	(end-of-line) ;(forward-char)
	))
    (forward-char)
    ))

(defun do-tolerance-line ()
  "Handle a line of the form <<SM: tol= nn, time = tt, nc = cc>>"
  (eliminate-to "time =")
  (search-forward ",") (backward-char)
  (eliminate-to "nc =")
  (end-of-line) (kill-line 1))

(defun process-mas-file (result-filename)
  "Given a filename-root, open filename-root.result,
process-mas-buffer, and save out filename-root.data"
  (interactive "fname of result file")
  (let ((filename-root (file-name-sans-extension result-filename)))
    (find-file (concat filename-root ".result"))
    (process-mas-buffer)
    (write-file (concat filename-root ".data"))
    (kill-buffer nil)))

(defun process-all-mas-files (directory)
  "Run process-mas-file on all .result files in the given directory
that do not have accompanying .data files"
  (interactive "Ddirectory")
  (let ((files (file-expand-wildcards (concat directory "*.result"))))
    (mapc (lambda (file) 
	    (if (not (file-exists-p
		      (concat (file-name-sans-extension file)
			      ".data")))
		(process-mas-file file)))
	  files)))
